/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.graphics;
using rhodes.physics;

namespace rhodes.engine
{
    /*! A compound entity that keeps track of multiple simple entities that are jointed together.
     * \remarks Joints break when certain collision occurs on the entity. This done through a callback 
     *      which is handled on the game side */
    public class CompoundEntity: Entity
    {        
        //! Key - Body, Value - Joint
        //! This way we can find out which joint the body is connected to.
        //! And from the joint we can find the body the joint is connected to.
        private SortedDictionary<String, SimpleEntity> _entities;
        private SortedDictionary<String, List<Joint>> _joints;

        /*! Constructur */
        public CompoundEntity(String name) : base(name)
        {
            _joints = new SortedDictionary<String, List<Joint>>();
        }

        /*! Constructur */
        public CompoundEntity(String name, 
            SortedDictionary<String, SimpleEntity> entities, 
            SortedDictionary<String, List<Joint>> joints): base(name)
        {
            _entities = entities;
            _joints = joints;
        }

        /*! Tries to get a simple entity from the compound
         * \param name The name of the entity.
         * \return A SimpleEntity or NULL */
        public SimpleEntity getSimple(String name)
        {
            SimpleEntity retVal = null;

            if (_entities.ContainsKey(name))
            {
                retVal = _entities[name];
            }

            return retVal;
        }

        /*! Updates all the simple entities in the compound
         * \copydoc Entity::update */
        public override void update(float elapsedTime) 
        {
            if (!_active)
                return;

            foreach (SimpleEntity entity in _entities.Values)
            {
                if (!entity.PhysicsBody.Body.IsStatic)
                {
                    entity.update(elapsedTime);
                }
            }
        }

        /*! Essentially removes all joint connected to the joint.
         * \param name The name of the simple entity. */
        public List<SimpleEntity> seperate(String name)
        {
            List<SimpleEntity> ret = new List<SimpleEntity>();
            String tic = name;
            List<Joint> ping = _joints[tic];

            // Remove tic before you loop. That way you don't loop to yourself
            _joints.Remove(tic);

            //! Loop through all the joints of tic and remove all of tic's joint from all other entities in the compound
            foreach (Joint joint in ping)
            {
                // Figure out if it's a RevoluteJoint or an AngleJoint
                if (joint.GetType() == typeof(RevoluteJoint))
                {
                    // Figure out what the other connected entity is... You don't want to compare yourself with yourself
                    String tag = (String)((RevoluteJoint)joint).Body1.Tag;
                    
                    if (tag.Contains(tic))
                    {
                        tag = (String)((RevoluteJoint)joint).Body2.Tag;
                    }

                    String tac = tag.Substring(tag.LastIndexOf('/') + 1);

                    // Get tac's joint list and remove tic's joint from it.
                    List<Joint> pong = _joints[tac];
                    pong.Remove(joint);

                    // If this is the last joint in tac's list then he's free from the compound, so we:
                    // - Remove tac from _joints
                    // - Add tac to the entitymanager's simple list
                    // - Remove compound tag from body
                    if (pong.Count == 0)
                    {
                        _joints.Remove(tac);
                        _entities[tac].PhysicsBody.Body.Tag = tac;
                        EntityManager.getSingleton.addSimple(_entities[tac]);
                        ret.Add(_entities[tac]);
                        _entities.Remove(tac);
                    }
                }
                else if (joint.GetType() == typeof(AngleJoint))
                {
                    // Figure out what the other connected entity is... You don't want to compare yourself with yourself
                    String tag = (String)((AngleJoint)joint).Body1.Tag;

                    if (tag.Contains(tic))
                    {
                        tag = (String)((AngleJoint)joint).Body2.Tag;
                    }

                    String tac = tag.Substring(tag.LastIndexOf('/') + 1);

                    // Get tac's joint list and remove tic's joint from it.
                    List<Joint> pong = _joints[tac];
                    pong.Remove(joint);

                    // If this is the last joint in tac's list then he's free from the compound, so we:
                    // - Remove tac from _joints
                    // - Add tac to the entitymanager's simple list
                    // - Remove compound tag from body
                    if (pong.Count == 0)
                    {
                        _joints.Remove(tac);
                        _entities[tac].PhysicsBody.Body.Tag = tac;
                        EntityManager.getSingleton.addSimple(_entities[tac]);
                        ret.Add(_entities[tac]);
                        _entities.Remove(tac);
                    }
                }

                // remove the actual joint from the physics manager
                PhysicsManager.getSingleton.PhysicsSimulator.Remove(joint);         
            }

            // 
            _entities[tic].PhysicsBody.Body.Tag = tic;
            EntityManager.getSingleton.addSimple(_entities[tic]);
            ret.Add(_entities[tic]);
            _entities.Remove(tic);
               
            // At the end of this call.. ask if compound is empty, if so then remove from entity manager
            // both the joints and entities should be empty at this point
            //if (_entities.Count == 0)
            //if (_joints.Count == 0)
            //    return true;
            //else
                //return false;

            return ret;
        }

        /*! Properties */
        public SortedDictionary<String, SimpleEntity>.ValueCollection Entities { get { return _entities.Values; } }
        public Boolean Active
        {
            get
            {
                return _active;
            }
            set
            {
                foreach (SimpleEntity entity in _entities.Values)
                {
                    entity.Active = value;
                }
            }
        }
    }
}
