/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using rhodes.graphics;

namespace rhodes.engine
{
    /*! 
     * EntityManager will update all dynamic entities in the world.
     * The game should only call updates to entities here.
     * Note: If you load multiple instances of the same entity,
	 * only one instance of that entity will be displayed. */
    public class EntityManager
    {
        private static volatile EntityManager _instance;
        private static object _syncRoot = new Object();
        
        private SortedDictionary<String, SimpleEntity> _simples;
        private SortedDictionary<String, CompoundEntity> _compounds;
       
        private EntityManager() 
        {
            _simples = new SortedDictionary<String, SimpleEntity>();
            _compounds = new SortedDictionary<String, CompoundEntity>();
        }

        /*! Returns singleton instance of the EntityManager */
        public static EntityManager getSingleton
        {
            get
            {
                if (_instance == null)
                {
                    lock (_syncRoot)
                    {
                        if (_instance == null)
                            _instance = new EntityManager();
                    }
                }

                return _instance;
            }
        }

        /*! Loads or returns a simple entity 
         * \param filename The path and name of the entity to load 
         * \return SimpleEntity */
        public SimpleEntity loadSimple(String filename)
        {
            String name = filename.Substring(filename.LastIndexOf('/') + 1);

            if (_simples.ContainsKey(name) )
            {
                Console.Error.WriteLine("Entity already loaded: " + filename);
                return _simples[name];
            }

            SimpleEntity entity = GraphicsCore.getSingleton.Content.Load<SimpleEntity>(filename);
            _simples.Add(name, entity);

            return entity;
        }

        /*! Loads or returns a compound entity
         * \param filename The path and name of the entity to load
         * \return CompoundEntity */
        public CompoundEntity loadCompound(String filename)
        {
            int lastIndex = filename.LastIndexOf('.');
            if (lastIndex != -1)
                filename = filename.Replace(filename.Substring(lastIndex), "");

            String name = filename.Substring(filename.LastIndexOf('/') + 1);

            if (_compounds.ContainsKey(name))
            {
                Console.Error.WriteLine("Entity already loaded: " + filename);
                return _compounds[name];
            }

            CompoundEntity entity = GraphicsCore.getSingleton.Content.Load<CompoundEntity>(filename);
            _compounds.Add(name, entity);

            return entity;
        }

        /*! Add a simple entity to the manager.
         * \param entity A simple entity to add to the manager
         * \remarks This method shouldn't be used. DEPRECATED */
        public void addSimple(SimpleEntity entity)
        {
            if (_simples.ContainsKey(entity.Name))
            {
                Console.Error.WriteLine("Entity " + entity.Name + " already exists.");
            }
            else
            {
                _simples.Add(entity.Name, entity);
            }
        }

        /*! Gets a simple entity by name
         * \param name The name of the entity to get.
         * \return Returns a SimpleEntity or NULL if it doesn't exist. */
        public SimpleEntity getSimple(String name)
        {
            SimpleEntity retVal = null;

            if (_simples.ContainsKey(name))
            {
                retVal = _simples[name];
            }

            return retVal;
        }

        /*! Gets a compound entity by name
         * \param name The name of the entity to get.
         * \return Returns a CompoundEntity or NULL if it doesn't exist. */
        public CompoundEntity getCompound(String name)
        {
            CompoundEntity retVal = null;

            if(_compounds.ContainsKey(name))
            {
                retVal = _compounds[name];
            }

            return retVal;
        }

        /*! Updates all entities in manager.
         * \param elapsedTime The time elapsed since last tick in milliseconds. */
        public void update(float elapsedTime)
        {
            // remember to call physics update before this
            // update all dynamic simples
            foreach (SimpleEntity entity in _simples.Values)
            {
                if (!entity.PhysicsBody.Body.IsStatic)
                {
                    entity.update(elapsedTime);
                }
            }

            foreach (CompoundEntity entity in _compounds.Values)
            {
                entity.update(elapsedTime);
            }
        }

        /*! Seperates a joing from a compound by the simple entity's name.
         * \param name The name of the simple entity */
        public List<SimpleEntity> seperateCompound(String name)
        {
            List<SimpleEntity> ret = new List<SimpleEntity>();
            // The name of the simple entity is combined with the name of the 
            // compound entity. So we need to find out which compund this simple belongs to.
            String[] pair = name.Split('/');

            if (_compounds.ContainsKey(pair[0]))
            {
                ret = _compounds[pair[0]].seperate(pair[1]);

                if (_compounds[pair[0]].Entities.Count == 0)
                {
                    _compounds.Remove(pair[0]);
                }
            }
            else
            {
                Console.Error.WriteLine("No such compound: " + pair[0]);
            }

            return ret;
        }

        /*! Dumps content to output. */
        public void dump()
        {
            Console.Out.WriteLine("Simple Entity List: ");
            foreach (SimpleEntity entity in _simples.Values)
            {
                Console.Out.WriteLine(entity.Name);
            }

            Console.Out.WriteLine("Compound Entity List: ");
            foreach (CompoundEntity entity in _compounds.Values)
            {
                Console.Out.WriteLine(entity.Name);
            }

        }
    }
}
