/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;
using RhodesProcessorLib;

using rhodes.graphics;
using rhodes.physics;

namespace rhodes.engine
{
    /*! Reads & process simple entities */
    public class SimpleEntityReader: ContentTypeReader<SimpleEntity>
    {
        /*! \copydoc ContentTypeReader::Read */
        protected override SimpleEntity Read(ContentReader input, SimpleEntity existingInstance)
        {
            SimpleEntity retVal;

            //! general
            String name = input.ReadString();
            Vector2 position = input.ReadObject<Vector2>();
            float rotation = input.ReadSingle();
            Boolean active = input.ReadBoolean();

            Console.Out.WriteLine(active);
            //! sprite
            String spriteMap = input.ReadString().Replace(".spritemap","");
            Boolean cycle = input.ReadBoolean();
            String animation = input.ReadString();

            Sprite sprite = SpriteManager.getSingleton.load(name, spriteMap);
            sprite.Cycle = cycle;
            sprite.setAnimation(animation);
            sprite.Position = position;
            sprite.Rotation = rotation;

            //! body
            float mass = input.ReadSingle();
            float momentOfInertia = input.ReadSingle();
            Boolean ignoreGravity = input.ReadBoolean();
            Boolean isStatic = input.ReadBoolean();
            BodyInfo bodyInfo = input.ReadObject<BodyInfo>();

            Body body;
            if (bodyInfo.Type == BodyInfo.BodyType.Rectangle)
            {
                body = BodyFactory.Instance.CreateRectangleBody( 
                                    PhysicsManager.getSingleton.PhysicsSimulator,
                                    ((RectangleBodyInfo)bodyInfo).Width,
                                    ((RectangleBodyInfo)bodyInfo).Height, mass);
            }
            else if (bodyInfo.Type == BodyInfo.BodyType.Circle)
            {
                body = BodyFactory.Instance.CreateCircleBody(
                                PhysicsManager.getSingleton.PhysicsSimulator,
                                ((CircleBodyInfo)bodyInfo).Radius, mass);
            }
            else
            {
                throw new Exception("EntityReader::Read - Invalid BodyInfo type");
            }

            body.Tag = name;
            body.Position = position;
            body.Rotation = rotation;
            body.IsStatic = isStatic;
			body.IgnoreGravity = ignoreGravity;

            //! geom
            Boolean collisionEnabled = input.ReadBoolean();
            Enums.CollisionCategories collisionCategory = input.ReadObject<Enums.CollisionCategories>();
            Enums.CollisionCategories collidesWith = input.ReadObject<Enums.CollisionCategories>();
            Boolean collisionResponseEnabled = input.ReadBoolean();

            float frictionCoefficient = input.ReadSingle();

            List<CollisionInfo> collisionList = input.ReadObject<List<CollisionInfo>>();

            List<Geom> geomList = new List<Geom>();
            foreach (CollisionInfo info in collisionList)
            {
                Geom geom;
                
                if (info.GetType() == typeof(RectangleCollisionInfo))
                {
                    geom = GeomFactory.Instance.CreateRectangleGeom(
                        PhysicsManager.getSingleton.PhysicsSimulator,
                        body,
                        ((RectangleCollisionInfo)info).Width,
                        ((RectangleCollisionInfo)info).Height,
                        info.Offset,
                        info.RotationOffset);

                }
                else if (info.GetType() == typeof(CircleCollisionInfo))
                {
                    geom = GeomFactory.Instance.CreateCircleGeom(
                        PhysicsManager.getSingleton.PhysicsSimulator,
                        body,
                        ((CircleCollisionInfo)info).Radius,
                        ((CircleCollisionInfo)info).Edges,
                        info.Offset,
                        info.RotationOffset);
                }
                else if (info.GetType() == typeof(PolygonCollisionInfo))
                {
                    geom = GeomFactory.Instance.CreatePolygonGeom(
                        PhysicsManager.getSingleton.PhysicsSimulator,
                        body,
                        new Vertices(((PolygonCollisionInfo)info).Vertices.ToArray()),
                        info.Offset,
                        info.RotationOffset,
                        (((PolygonCollisionInfo)info).CollisionGridCellSize));
                }
                else
                {
                    throw new Exception("EntityReader::Read - Invalid CollisionInfo type");
                }

                geom.Tag = name;
                geom.CollisionEnabled = collisionEnabled;
                geom.CollisionCategories = collisionCategory;
                geom.CollidesWith = collidesWith;
                geom.CollisionResponseEnabled = collisionResponseEnabled;
                geom.FrictionCoefficient = frictionCoefficient;

                geomList.Add(geom);
            }

            PhysicsBody physicsBody = PhysicsManager.getSingleton.addPhysicsBody(name, body, geomList);

            retVal = new SimpleEntity(name, physicsBody, sprite);
            retVal.Active = active;

            return retVal;
        }
    }
}
