using System;

using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
	/* 
	 * FileName: Font.cs
	 * Author:  Anthony Reese
	 * 
	 * Represents a font used to draw text to the screen
	 */
	public class Font
	{
		private Dictionary<int, Texture2D> _images;
		private Dictionary<char, TextCharacter> _characters;
		private Dictionary<char, Dictionary<char, int>> _kernings;

		bool kerning = true;

		/// <summary>
		/// Creates a font with images used, the characters that can be used, and the kerning for the characters.
		/// </summary>
		/// <param name="name"></param>
		/// <param name="imageInfo"></param>
		/// <param name="textures"></param>
		/// <param name="characters"></param>
		/// <param name="kern"></param>
		public Font(string name, Dictionary<int, Texture2D> images, Dictionary<char, TextCharacter> characters,
			Dictionary<char, Dictionary<char, int>> kernings)
		{
			_images = images;
			_characters = characters;
			_kernings = kernings;
		}

		/// <summary>
		/// Get the kern value for the given pair of characters
		/// </summary>
		/// <param name="chLeft">Left character</param>
		/// <param name="chRight">Right character</param>
		/// <returns>Amount to kern (in pixels)</returns>
		private int CalcKern(char chLeft, char chRight)
		{
			if (_kernings.ContainsKey(chLeft))
			{
				Dictionary<char, int> kern2 = _kernings[chLeft];
				if (kern2.ContainsKey(chRight))
					return kern2[chRight];
			}
			return 0;
		}

		/// <summary>
		/// Draw the given string at vOrigin using the specified color
		/// </summary>
		/// <param name="origin">(x,y) coord</param>
		/// <param name="color">Text color</param>
		/// <param name="format">String format</param>
		/// <param name="args">String format args</param>
		/// <returns>Width of string (in pixels)</returns>
		public void render(Vector2 origin, Color color, string message, float scale)
		{
			SpriteBatch spriteBatch = GraphicsCore.getSingleton.SpriteBatch;
			//string str = string.Format(format, message);

			Vector2 vAt = origin;
			int nWidth = 0;
			char cLast = '\0';

			// gget the image of each character and send it to be rendered
			foreach (char c in message)
			{
				if (!_characters.ContainsKey(c))
				{
					//TODO: print out undefined char glyph
					continue;
				}

				TextCharacter character = _characters[c];

				// if kerning is enabled, get the kern adjustment for this char pair
				if (kerning)
				{
					int nKern = CalcKern(cLast, c);
					vAt.X += nKern;
					nWidth += nKern;
					cLast = c;
				}

				// draw the character
				vAt.X += character.leftSideBearing;
				if (character.width  != 0 && character.height != 0)
				{
					// being done elswhere
					Rectangle r = new Rectangle(character.xPosition, character.yPosition, character.width, character.height);
					//spriteBatch.Draw(images[character.imageID], vAt, r, color); // Texture, Position, Source Rectangle, Color
					spriteBatch.Draw(_images[character.imageID], vAt * scale, r, color, 0.0f, Vector2.Zero, scale, SpriteEffects.None, 0);
				}

				// update the string width and advance the pen to the next drawing position
				nWidth += character.advancedWidth;
				vAt.X += character.advancedWidth - character.leftSideBearing;
			}
		}


	}
}
