using System;
using System.Xml;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
	/* 
	 * FileName: TextManager.cs
	 * Author:  Anthony Reese
	 * 
	 * Based off of BitmapFont.cs
	 * Bitmap Font class for XNA
	 * Copyright 2006 Microsoft Corp.
	 * Revision: 2006-Aug-30
	 *  
	 * Handles the interpretation of a string into an image 
	 */
	public class FontManager
	{
		private static volatile FontManager _instance;
        private static object _syncRoot = new Object();

        private SortedDictionary<String, Font> _fonts;

        //!
		private FontManager() 
        {
            _fonts = new SortedDictionary<string, Font>();
        }

        /// <summary>
        /// Returns a single of instance of the FontManager
        /// </summary>
		public static FontManager getSingleton
        {
            get
            {
                if (_instance == null)
                {
                    lock (_syncRoot)
                    {
                        if (_instance == null)
							_instance = new FontManager();
                    }
                }

                return _instance;
            }
        }

        /// <summary>
        /// retrievs the images and retrive information from the xml file for the characters of the font
        /// </summary>
        /// <param name="fontName"></param>
        /// <returns></returns>
        public Font load(String fontName)
        {
            Console.Out.WriteLine("loaded " + fontName);
            Font retVal = null;

            if(_fonts.ContainsKey(fontName))
            {
                Console.Error.WriteLine( "Font entity: " + fontName + " already exists." );
                retVal = _fonts[fontName];
            }
            else
            {	
				Dictionary<int, Texture2D> images = new Dictionary<int, Texture2D>();
				Dictionary<char, TextCharacter> characters = new Dictionary<char, TextCharacter>();
				Dictionary<char, Dictionary<char, int>> kernings = new Dictionary<char, Dictionary<char, int>>();

				// load in images used for font
				images.Add(0, TextureResourceManager.getSingleton.load("assets/fonts/" + fontName + "-0"));
				//images.Add(1, TextureResourceManager.getSingleton.load("assets/fonts/" + fontName + "-1"));

				// load in character data
				XmlDocument fontXML = new XmlDocument();
				fontXML.Load("../../../assets/fonts/" + fontName + ".xml");


				foreach (XmlNode node in fontXML)
				{
					if (node.Name == "font")
					{
					retVal= loadFont(fontName, node.ChildNodes, images);
					}
				}

                _fonts.Add(fontName, retVal);
            }
            return retVal;
        }


		/// <summary>
		/// Load the data from the "font" node
		/// </summary>
		/// <param name="xnl">XML node list containing the "font" node's children</param>
		private Font loadFont(string name, XmlNodeList xnl, Dictionary<int, Texture2D> images)
		{
			Dictionary<char, TextCharacter> characters = new Dictionary<char, TextCharacter>();
			Dictionary<char, Dictionary<char, int>> kernings = new Dictionary<char, Dictionary<char, int>>();
			foreach (XmlNode xn in xnl)
			{
				if (xn.Name == "glyphs")
					characters = loadCharacters(xn.ChildNodes);
				if (xn.Name == "kernpairs")
					kernings = loadKerningPairs(xn.ChildNodes);
			}
			return new Font(name, images, characters, kernings);
		}
		/// <summary>
		/// Load the data from the "glyphs" node
		/// </summary>
		/// <param name="xnl">XML node list containing the "glyphs" node's children</param>
		private Dictionary<char, TextCharacter> loadCharacters(XmlNodeList xnl)
		{
			Dictionary<char, TextCharacter> characters = new Dictionary<char, TextCharacter>();
			foreach (XmlNode xn in xnl)
			{
				if (xn.Name == "glyph")
				{
					string strChar = GetXMLAttribute(xn, "ch");
					string strBitmapID = GetXMLAttribute(xn, "bm");
					string strOrigin = GetXMLAttribute(xn, "origin");
					string strSize = GetXMLAttribute(xn, "size");
					string strAW = GetXMLAttribute(xn, "aw");
					string strLSB = GetXMLAttribute(xn, "lsb");

					string[] aOrigin = strOrigin.Split(',');
					string[] aSize = strSize.Split('x');

					TextCharacter character = new TextCharacter(Int32.Parse(strBitmapID), 
						new Vector2(Int32.Parse(aOrigin[0]), Int32.Parse(aOrigin[1])),
						new Vector2(Int32.Parse(aSize[0]), Int32.Parse(aSize[1])),
						Int32.Parse(strAW), Int32.Parse(strLSB) );

					characters[strChar[0]] = character;
				}
			}
			return characters;
		}



		/// <summary>
		/// Load the data from the "kernpairs" node
		/// </summary>
		/// <param name="xnl">XML node list containing the "kernpairs" node's children</param>
		private Dictionary<char, Dictionary<char, int>> loadKerningPairs(XmlNodeList xnl)
		{
			Dictionary<char, Dictionary<char, int>> kernings = new Dictionary<char, Dictionary<char, int>>();
			foreach (XmlNode xn in xnl)
			{
				if (xn.Name == "kernpair")
				{
					string strLeft = GetXMLAttribute(xn, "left");
					string strRight = GetXMLAttribute(xn, "right");
					string strAdjust = GetXMLAttribute(xn, "adjust");

					char chLeft = strLeft[0];
					char chRight = strRight[0];

					// create a kern dict for the left char if needed
					if (!kernings.ContainsKey(chLeft))
						kernings[chLeft] = new Dictionary<char, int>();

					// add the right char to the left char's kern dict
					Dictionary<char, int> kern2 = kernings[chLeft];
					kern2[chRight] = Int32.Parse(strAdjust);
				}
			}
			return kernings;
		}

		/// <summary>
		/// Get the XML attribute value (without throwing if the attribute doesn't exist)
		/// </summary>
		/// <param name="n">XML node</param>
		/// <param name="strAttr">Attribute name</param>
		/// <returns>Attribute value, or the empty string if the attribute doesn't exist</returns>
		private static string GetXMLAttribute(XmlNode n, string strAttr)
		{
			string strVal = "";
			try
			{
				strVal = n.Attributes[strAttr].Value;
			}
			catch
			{
				strVal = "";
			}
			return strVal;
		}

        /// <summary>
        /// Get a font
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public Font get(String name)
        {
            if (_fonts.ContainsKey(name))
            {
                return _fonts[name];
            }

            return null;
        }

        /// <summary>
        /// Removes all font
        /// </summary>
        public void removeAll()
        {
            _fonts.Clear();
        }

	}
}
