/*
 *  Author: David Huynh
 *  Author: Anthony Reese
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace rhodes.graphics
{
    /*! Loads and manages all the GUI in the engine */
    public class GuiManager
    {
        private static volatile GuiManager _instance;
        private static object _syncRoot = new Object();
        private SortedDictionary<String, Sprite> _huds;
		private SortedDictionary<String, Message> _messages;

        private GuiManager() 
        {
            _huds = new SortedDictionary<string, Sprite>();
			_messages = new SortedDictionary<string, Message>();
        }

        /*! Returns singleton instance of the GuiManagers */
        public static GuiManager getSingleton
        {
            get
            {
                if (_instance == null)
                {
                    lock (_syncRoot)
                    {
                        if (_instance == null)
                            _instance = new GuiManager();
                    }
                }

                return _instance;
            }
        }

        /*! Updates all gui elements 
         * \param elapsedTime The time elapsed since last tick in milliseconds. */       
        public void update(float elapsedTime)
        {
            foreach (Sprite sprite in _huds.Values)
            {
                sprite.update(elapsedTime);
            }
        }

        /*! Render all visible gui elements */
        public void render()
        {
            GraphicsCore.getSingleton.SpriteBatch.Begin(
                SpriteBlendMode.AlphaBlend,
                SpriteSortMode.Immediate,
                SaveStateMode.None);

            foreach (Sprite sprite in _huds.Values)
            {
                sprite.render(SpriteMap.DIFFUSE, null, Vector2.Zero, Vector2.One);
            }

			foreach (Message message in _messages.Values)
			{
				if (message.visible)
					message.render();
			}

            GraphicsCore.getSingleton.SpriteBatch.End();
        }

        /*! Loads a gui element 
         * \param hudName The name of the gui element 
         * \param spriteMapFilename The name of the sprite map to load */
        public Sprite load(String hudName, String spriteMapFilename)
        {
            Sprite retVal = null;

            if (_huds.ContainsKey(hudName))
            {
                retVal = _huds[hudName];
            }
            else
            {
                Sprite sprite = SpriteManager.getSingleton.load(hudName, spriteMapFilename);
                
                if (sprite != null)
                {
                    retVal = sprite;
                    _huds.Add(hudName, sprite);
                }
            }

            return retVal;
        }

		/*! adds a gui element if nothing has to be loaded 
         * \param messageName The name of the gui element 
         * \param message the element*/
		public Message load(String messageName,  Message message)
		{
			Message retVal = null;

			if (_messages.ContainsKey(messageName))
			{
				retVal = _messages[messageName];
			}
			else
			{
				retVal = message;
				_messages.Add(messageName, message);
			}

			return retVal;
		}

        /*! Get the GUI element
         * \param name The name of the GUI.
         * \return The sprite */
        public Sprite get(String name)
        {
            if (_huds.ContainsKey(name))
            {
                return _huds[name];
            }

            return null;
        }

		/*! Get the GUI element
         * \param name The name of the GUI.
         * \return The message */
		public Message getMessage(String name)
		{
			if (_messages.ContainsKey(name))
			{
				return _messages[name];
			}

			return null;
		}

        /*! Removes a gui element 
         * \param hudName The name of the HUD */
        public void remove(String hudName)
        {
            if (_huds.ContainsKey(hudName))
            {
                _huds.Remove(hudName);
            }
        }

        /*! Removes a gui element 
         * \param messageName The name of the Message */
        public void removeMessage(String messageName)
        {
            if (_messages.ContainsKey(messageName))
            {
                _messages.Remove(messageName);
            }
        }

        /*! Removes all gui elements */
        public void removeAll()
        {
            _huds.Clear();
            _messages.Clear();
        }
    }
}
