/*
 *  Author: Josh Wilson & David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using rhodes.graphics.particles;

namespace rhodes.graphics
{
    /*! Represents a parallax layer in the 2D scene */
    public class Layer
    {
        protected Dictionary<string, Sprite> _sprites;
		protected Dictionary<string, ParticleEngine> _particles;
        protected List<Light> _lights;
        protected Vector2 _position;
		protected float _proportion;
		protected Vector2 _offset;

        public Layer()
		{
			_proportion = 1.0f;
			_offset = new Vector2();
			_sprites = new Dictionary<string, Sprite>();
			_particles = new Dictionary<string, ParticleEngine>();
            _lights = new List<Light>();
		}

        public Layer(float proportion)
        {
			_proportion = proportion;
			_offset = new Vector2();
            _sprites = new Dictionary<string, Sprite>();
			_particles = new Dictionary<string, ParticleEngine>();
            _lights = new List<Light>();
        }

        /*! Determines if this layer contains a specific sprite and 
		 * \either adds the sprite to the layer or change teh sprite.
         * \param spriteName Name of the sprite to add to the layer
         * \param inSprite Sprite to add to the layer */
        public void AddSprite(string spriteName, Sprite inSprite)
        {
			if (!_sprites.ContainsKey(spriteName))
			{
				_sprites.Add(spriteName, inSprite);
			}
			else
			{
				_sprites[spriteName] = inSprite;
			}
        }

        /*! Add a light to the layer 
         * \param lightName Name of the light.
         * \param inLight The light to be added */
        public void AddLight(Light inLight)
        {
            _lights.Add(inLight);
        } 

		/*! Add a light to the layer 
         * \param lightName Name of the light.
         * \param inLight The light to be added */
		public void AddParticle(string particleName, ParticleEngine particle)
		{
			_particles.Add(particleName, particle);
		}

        /*! Renders all the Sprites in the layer. */
        public void Render(byte type, Effect effect)
        {
			// Render the particle engine before the other stuff in the layer. critical.
			foreach (ParticleEngine particle in _particles.Values)
			{
				particle.render(type, effect, _offset, new Vector2(Camera2D.getSingleton.Zoom));
			}

			// Render all other entities.
			GraphicsCore.getSingleton.SpriteBatch.Begin(
				SpriteBlendMode.AlphaBlend,
				SpriteSortMode.Immediate,
				SaveStateMode.None);

			foreach (Sprite thisSprite in _sprites.Values)
			{
				thisSprite.render(type, effect, _offset, new Vector2(Camera2D.getSingleton.Zoom));
			}
			
			GraphicsCore.getSingleton.SpriteBatch.End();
        }

        /*! Gets a list of lights that are effecting the current view 
         * \todo implement!
         * \return A list of lights */
        public List<Light> getEffectiveLights()
        {
            // IMPLEMENT ME
            return new List<Light>();
        }

        /*! Updates the position of all the sprites in the Layer.
         * \param displacement The offset to update the sprites by. */
        public void Update(Vector2 displacement)
        {
            _offset += (_proportion * displacement);
        }

		/*! Determines if this layer contains a specific sprite
		 * \param spriteName Name of the sprite to check in this layer
		 * \return True, if the sprite is in this layer. False, if it isn't. */
		public bool ContainsSpriteName(string spriteName)
        {
			return _sprites.ContainsKey(spriteName);
        }

        /*! Removes a sprite from the layer.
         * \param spriteName Name of the sprite to remove from the layer */
        public void Remove(string spriteName)
        {
            _sprites.Remove(spriteName);
        }

        /*! Removes all sprites from the layer. */
        public void RemoveAll()
        {
            _sprites.Clear();
        }

        /*! Properties */
        public Vector2 Offset { get { return _offset; } }
        public List<Light> Lights { get { return _lights; } }
		public Dictionary<string, Sprite> Sprites { get { return _sprites; } }
        public Vector2 Position { get { return _position; } set { _position = value; } }
        public float Proportion { get { return _proportion; } set { _proportion = value; } }
    }
}
