/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    public class Sprite
    {
        public static float DEFAULT_SPRITE_DELAY = .05f;
        public static bool DEFAULT_SPRITE_CYCLE = true;
		private const float COLOR_SATURATION = .5f; 

        public delegate void PositionChangedDelegate(String name, Sprite sprite);
        public PositionChangedDelegate PositionChangedEvent;

        private String _name;   // unique to SpriteManager
        private SpriteMap _map; // unique to SpriteResourceManager
        
        private Vector2 _position;
        private float _rotation;
        private Vector4 _color;

        private float _elapsedTime;      
        private float _delay;
        private bool _paused;
        private bool _cycle;
        private int _currentFrame;
        private String _currentAnimation;

        private SpriteEffects _orientation;

        private Vector2 _scale;
        private float _depth;

        //private uint _visibilityMask;
        private Boolean _visible;

        public delegate void AnimationHandlerDelegate(String lastAnimation);
        public AnimationHandlerDelegate _animationHandler;

        public Sprite( String name, SpriteMap map)
        {
            _name = name;
            _map = map;
            _currentAnimation = "default";
            _currentFrame = 0;
            _cycle = DEFAULT_SPRITE_CYCLE;
            _delay = DEFAULT_SPRITE_DELAY;
            _scale = Vector2.One;
            _rotation = 0.0f;
            _position = Vector2.Zero;
            _depth = 0.5f;
            _elapsedTime = 0.0f;
            _orientation = SpriteEffects.None;
            _visible = true;
            _color = Vector4.One;
        }

        public void render(byte mode, Effect effect, Vector2 offset, Vector2 zoom) {
            if(!_visible)
                return;

            if(effect != null) {
                effect.Begin();
                effect.CurrentTechnique.Passes[0].Begin();

                effect.Parameters["rotation"].SetValue(_rotation);
            }

            _map.render(mode, _currentFrame, (offset + _position) * zoom, zoom * _scale, _color, _orientation, _rotation, _depth);

            if(effect != null) {
                effect.CurrentTechnique.Passes[0].End();
                effect.End();
            }
        }

        //! update - only update the animation if there are any
        public void update( float elapsedTime )
        {
            if (_paused)
            {
                return;
            }

            _elapsedTime = _elapsedTime + elapsedTime;
            
            if (_elapsedTime > _delay)
            {
                _currentFrame++;
                _elapsedTime = _elapsedTime - _delay;
                
                // 
                if (_currentFrame >= _map.Animations[_currentAnimation].Y)
                {
                    if (!_cycle)
                    {
                        // Animation callback
                        if (_animationHandler != null)
                        {
                            _animationHandler(_currentAnimation);
                        }
                        _paused = true;
                    }

                    _currentFrame = _map.Animations[_currentAnimation].X;
                }
            }
        }

        public void pause() { _paused = true; }
        public void unpause() { _paused = false; }

        public void setAnimation(String animation)
        {
            setAnimation(animation, DEFAULT_SPRITE_DELAY, DEFAULT_SPRITE_CYCLE);
        }

        public void setAnimation(String animation, float delay)
        {
            setAnimation(animation, delay, DEFAULT_SPRITE_CYCLE);
        }

        public void setAnimation(String animation, bool cycle)
        {
            setAnimation(animation, DEFAULT_SPRITE_DELAY, cycle);
        }

        public void setAnimation(String animation, float delay, bool cycle)
        {
            if (_map.Animations.ContainsKey(animation))
            {
                _currentAnimation = animation;
                _currentFrame = _map.Animations[animation].X;
                _cycle = cycle;
                _delay = delay;
                _paused = false;
            }
        }

        public void flipLeft()
        {
            if ((_orientation & SpriteEffects.FlipHorizontally) == SpriteEffects.None)
            {
                _orientation = _orientation | SpriteEffects.FlipHorizontally;
            }
        }

        public void flipRight()
        {
            if ((_orientation & SpriteEffects.FlipHorizontally) == SpriteEffects.FlipHorizontally)
            {
                _orientation = _orientation & SpriteEffects.FlipVertically;
            }
        }
		
		public void flipUp()
		{
			if ((_orientation & SpriteEffects.FlipVertically) == SpriteEffects.None)
			{
				_orientation = _orientation | SpriteEffects.FlipVertically;
			}
		}

		public void flipDown()
		{
			if ((_orientation & SpriteEffects.FlipVertically) == SpriteEffects.FlipVertically)
			{
				_orientation = _orientation & SpriteEffects.FlipHorizontally;
			}
		}

        // Properties
        public String Name { get { return _name; } }
        public Vector2 Position 
        { 
            get 
            { 
                return _position;
            } 
            set 
            { 
                _position = value;

				// ******************************* ANTHONY CODE ******************************//
				// ***************************************************************************//
				if (PositionChangedEvent != null)
				{
					PositionChangedEvent(Name, this);
				}
            } 
        } 
        public float Rotation { get { return _rotation; } set { _rotation = value; } }
        public String CurrentAnimation { get { return _currentAnimation; } }
        public Vector2 Scale { get { return _scale; } set { _scale = value; } }
        public float Delay { get { return _delay; } set { _delay = value; } }
        public bool Cycle { get { return _cycle; } set { _cycle = value; } }
        public float Depth { get { return _depth; } set { _depth = value; } }
        public SpriteEffects Orientation { get { return _orientation; } set { _orientation = value; } }
        public Boolean Visible { get { return _visible; } set { _visible = value; } }
        public short Width { get { return _map.FrameWidth; } }
        public short Height { get { return _map.FrameHeight; } }
		public Vector4 Color { get { return _color; } set { _color = value; } }
    }
}
