/*
 *  Author: Brian Murphy
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace rhodes.graphics.particles.emitters {
    public class Emitter : IIsDisposable {

		protected Particle[] _particles;

        protected Random _randomizer;

		private Boolean _isActive;
		/*! Returns and sets if the emitter is currently active */
		public Boolean IsActive
		{
			get { return _isActive; }
			set
			{
				_isActive = value;
				if (_isActive) _currentEmitLife = 0;
			}
		}

		private int _currentIndex;
		private float _timeRatio, _tempVel;

		protected SpriteMap _spriteMap;
		protected string _spriteMapName;

		protected SpriteBlendMode _blendEffect;
		/*! Returns the emitters blend effect */
		public SpriteBlendMode blendEffect
		{
			get { return _blendEffect; }
		}

		protected short _maxFrame, _tempIndex;

		protected float _particlesPerSec;
		protected short _particleAmount;

		protected float _minStartRotSpeed, _maxStartRotSpeed, _minEndRotSpeed, 
			_maxEndRotSpeed, _startRotationRange;

		protected float _currentEmitLife, _emitLifetime, _particleLifetime, _particleLifeRandomizer;

		protected Vector2 _minStartSize, _maxStartSize, _minEndSize, _maxEndSize;

		protected float _minStartSpeed, _maxStartSpeed, _minEndSpeed, _maxEndSpeed;

		protected Vector4 _minStartColor, _maxStartColor, _minEndColor, _maxEndColor;

		/*! Returns whether or not the emitter is alive or not based on lifetime */
		protected bool Alive
		{
			get
			{
				if (_emitLifetime < 0)
				{
					return true;
				}
				else
				{
					return _currentEmitLife < _emitLifetime;
				}
			}
		}

		/*! Constructor
         * \param blendEffect SpriteBlendMode to use
         * \param spriteMapName Name of sprite
		 * \param maxFrame Max amount of frames for sprite
		 * \param particlesPerSec How many particles per second are made
		 * \param particleAmount The max amount of particles available at one time
		 * \param emitLifetime How long the emitter stays active for
		 * \param particleLifetime How long the patrticle stays active for
		 * \param particleLifeRandomizer Randomizer to particleLifeTime
		 * \param startRotationRange Range of radians to start between
		 * \param minStartRotSpeed How slow a particle can start rotating
		 * \param maxStartRotSpeed How fast a particle can start rotating
		 * \param minEndRotSpeed How slow a particle can end rotating
		 * \param maxEndRotSpeed How fast a particle can end rotating
		 * \param minStartSize How small a particle can start
		 * \param maxStartSize How large a particle can start
		 * \param minEndSize How small a particle can end
		 * \param maxEndSpeed How large a particle can end
		 * \param minStartColor Minimum values for color to start with
		 * \param maxStartColor Maximum values for color to start with
		 * \param minEndColor Minimum values for color to end with
		 * \param maxEndColor Maximum values for color to end with */
		public Emitter(SpriteBlendMode blendEffect, string spriteMapName, short maxFrame,
			float particlesPerSec, short particleAmount, float emitLifetime, 
			float particleLifetime, float particleLifeRandomizer, float startRotationRange,  
			float minStartRotSpeed, float maxStartRotSpeed, float minEndRotSpeed,float maxEndRotSpeed,
			Vector2 minStartSize, Vector2 maxStartSize, Vector2 minEndSize, Vector2 maxEndSize, 
			float minStartSpeed, float maxStartSpeed, float minEndSpeed, float maxEndSpeed, 
			Vector4 minStartColor, Vector4 maxStartColor, Vector4 minEndColor, Vector4 maxEndColor)
		{
            _particles = new Particle[particleAmount];
            _randomizer = new Random();
			_isActive = false;
			_blendEffect = blendEffect;

			_spriteMapName = spriteMapName;
			_spriteMap = SpriteResourceManager.getSingleton.load(spriteMapName);

			_maxFrame = maxFrame;

			_particlesPerSec = particlesPerSec;
			_particleAmount = particleAmount;

			_minStartRotSpeed = minStartRotSpeed;
			_maxStartRotSpeed = maxStartRotSpeed;
			_minEndRotSpeed = minEndRotSpeed;
			_maxEndRotSpeed = maxEndRotSpeed;
			_startRotationRange = startRotationRange;

			_currentEmitLife = 0;
			_emitLifetime = emitLifetime;

			_particleLifetime = particleLifetime;
			_particleLifeRandomizer = particleLifeRandomizer;
			
			_minStartSpeed = minStartSpeed;
			_maxStartSpeed = maxStartSpeed;
			_minEndSpeed = minEndSpeed;
			_maxEndSpeed = maxEndSpeed;

            _startRotationRange = startRotationRange;
            _minStartRotSpeed = minStartRotSpeed;
			_maxStartRotSpeed = maxStartRotSpeed; 
			_minEndRotSpeed = minEndRotSpeed;
			_maxEndRotSpeed = maxEndRotSpeed;

			_minStartSize = minStartSize;
			_maxStartSize = maxStartSize;
			_minEndSize = minEndSize;
			_maxEndSize = maxEndSize;

			_minStartColor = minStartColor;
			_maxStartColor = maxStartColor;
			_minEndColor = minEndColor;
			_maxEndColor = maxEndColor;
        }

		/*! Initializes particles using preset ranges
         * \param position Starting position for particle
         * \param velocity Starting velocity of particle
		 * \param index Array index of particle */
		public void intializeParticle(Vector2 position, Vector2 velocity, int index) {
			_currentIndex = index;
			_particles[index].currentFrame = 0;
            _particles[index].maxFrame = _maxFrame;
            
			_particles[index].currentLife = 0;
            _particles[index].lifetime =  _particleLifetime + (((float)_randomizer.NextDouble() * 100) %
				_particleLifeRandomizer);
			
			_particles[index].startSize = new Vector2(calcRandomVal(_minStartSize.X, _maxStartSize.X),
				calcRandomVal(_minStartSize.Y, _maxStartSize.Y));
			_particles[index].endSize = new Vector2(calcRandomVal(_minEndSize.X, _maxEndSize.X),
				calcRandomVal(_minEndSize.Y, _maxEndSize.Y));
			_particles[index].size = _particles[index].startSize;

			_tempVel = calcRandomVal(_minStartSpeed, _maxStartSpeed);
            _particles[index].startVelocity = new Vector2(_tempVel * velocity.X, _tempVel * velocity.Y);
			_tempVel = calcRandomVal(_minEndSpeed, _maxEndSpeed);
			_particles[index].endVelocity = new Vector2(_tempVel * velocity.X, _tempVel * velocity.Y);

            _particles[index].rotation = ((float)(_randomizer.NextDouble()) * _startRotationRange -
                _startRotationRange / 2.0f) * (_randomizer.Next(2) * 2 - 1);
			_particles[index].startRotationSpeed = calcRandomVal(_minStartRotSpeed, _maxStartRotSpeed);
			_particles[index].endRotationSpeed = calcRandomVal(_minEndRotSpeed, _maxEndRotSpeed);

            _particles[index].position = position;

			_particles[index].startColor = new Vector4(calcRandomVal(_minStartColor.X, _maxStartColor.X),
				calcRandomVal(_minStartColor.Y, _maxStartColor.Y), calcRandomVal(_minStartColor.Z, _maxStartColor.Z),
				calcRandomVal(_minStartColor.W, _maxStartColor.W));
			_particles[index].endColor = new Vector4(calcRandomVal(_minEndColor.X, _maxEndColor.X),
				calcRandomVal(_minEndColor.Y, _maxEndColor.Y), calcRandomVal(_minEndColor.Z, _maxEndColor.Z),
				calcRandomVal(_minEndColor.W, _maxEndColor.W));
			_particles[index].color = _particles[index].startColor;
        }

		/*! Returns a random value between the given range
         * \param min Starting position for particle
         * \param max Starting velocity of particle 
		 * \return Random float value between min and max*/
		private float calcRandomVal(float min, float max)
		{
			if (min == max)
			{
				return min;
			}
			else
			{
				return (float)(_randomizer.NextDouble() * 100) % (max - min) + min;
			}
		}

		/*! Renders all particles inside of the emitter 
         * \param type Type of graphical display
         * \param effect Graphical effect
		 * \param offset Camera coordinate offset
		 * \param zoom Camera zoom*/
		public void render(byte mode, Effect effect, Vector2 offset, Vector2 zoom) {
			int tempIndex = _currentIndex;
			Particle particle;
            for(int i = tempIndex + 1; i < _particles.Length; i++) {
				particle = _particles[i];
                if (particle.Alive) {
					if (effect != null)
					{
						effect.Begin();
						effect.CurrentTechnique.Passes[0].Begin();

						effect.Parameters["rotation"].SetValue(particle.rotation);

						Vector4 color = particle.color;
						color.W = 1.0f;

						_spriteMap.render(mode, particle.currentFrame, (offset + particle.position) * zoom,
							zoom * particle.size, color, SpriteEffects.None, particle.rotation, 0);
					}
					else
					{
						_spriteMap.render(mode, particle.currentFrame, (offset + particle.position) * zoom,
							zoom * particle.size, particle.color, SpriteEffects.None, particle.rotation, 0);
					}

					if (effect != null)
					{
						effect.CurrentTechnique.Passes[0].End();
						effect.End();
					}
                }
            }
			for (int i = 0; i <= tempIndex; i++)
			{
				particle = _particles[i];
				if (particle.Alive)
				{
					if (effect != null)
					{
						effect.Begin();
						effect.CurrentTechnique.Passes[0].Begin();

						effect.Parameters["rotation"].SetValue(particle.rotation);

						Vector4 color = particle.color;
						color.W = 1.0f;

						_spriteMap.render(mode, particle.currentFrame, (offset + particle.position) * zoom,
							zoom * particle.size, color, SpriteEffects.None, particle.rotation, 0);
					}
					else
					{
						_spriteMap.render(mode, particle.currentFrame, (offset + particle.position) * zoom,
							zoom * particle.size, particle.color, SpriteEffects.None, particle.rotation, 0);
					}

					if (effect != null)
					{
						effect.CurrentTechnique.Passes[0].End();
						effect.End();
					}
				}
			}
        }

		/*! Updates the emitter and all particles inside of it
         * \param elapsedTime Time passed since last call cycle */
        public virtual void Update(float elapsedTime) {
			_currentEmitLife += elapsedTime;
            for (int i = 0; i < _particles.Length; i++) {
                if (_particles[i].Alive) {

					_timeRatio = _particles[i].currentLife / _particles[i].lifetime;

                    _particles[i].position += ((_particles[i].startVelocity * (1.0f - _timeRatio) ) +
						(_particles[i].endVelocity * _timeRatio)) * elapsedTime;

					_particles[i].rotation += ((_particles[i].startRotationSpeed * (1.0f - _timeRatio)) +
						(_particles[i].endRotationSpeed * _timeRatio)) * elapsedTime;

					_particles[i].size = ((_particles[i].startSize * (1.0f - _timeRatio)) +
						(_particles[i].endSize * _timeRatio));

					_particles[i].color.X = ((_particles[i].startColor.X * (1.0f - _timeRatio)) +
						(_particles[i].endColor.X * _timeRatio));
					_particles[i].color.Y = ((_particles[i].startColor.Y * (1.0f - _timeRatio)) +
						(_particles[i].endColor.Y * _timeRatio));
					_particles[i].color.Z = ((_particles[i].startColor.Z * (1.0f - _timeRatio)) +
						(_particles[i].endColor.Z * _timeRatio));
					_particles[i].color.W = ((_particles[i].startColor.W * (1.0f - _timeRatio)) +
						(_particles[i].endColor.W * _timeRatio));

                    _particles[i].currentLife += elapsedTime;
					
                }
            }
        }

        protected bool _isDisposed = false;
		/*! Returns if this emitters is disposed */
        public bool IsDisposed {
            get { return _isDisposed; }
        }

		/* Disposes the emitter */
        public void Dispose() {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

		/*! Disposes any extra components if needed
		* \param disposing Whether or not currently disposing */
        protected virtual void Dispose(bool disposing) {
            //subclasses can override incase they need to dispose of resources
            //otherwise do nothing.
            if (!_isDisposed) {
                if (disposing) { };
                _isDisposed = true;
            }
        }
    }
}
