/*
 *  Author: Brian Murphy
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace rhodes.graphics.particles.emitters {
    public class SplineEmitter : Emitter {

        private List<Vector2> _emitterPositions;
		private Vector2[] _returnVal;
        private int _tempAmount;
        private int _tempIndex;
		private int _tempIndex2;
		private Vector2 _beginPoint;
		private Vector2 _endPoint;
		private double _randVal;
        private int _currentIndex;
        private float _totalElapsedTime;

		/*! Sets the current positions of the spline base */
		public Vector2 EmitterPosition
		{
			set
			{
				_emitterPositions[0] = value;
			}
		}

		/*! Sets the currents offsets for all of the splines additional points */
		public List<Vector2> EmitterOffsets
		{
			set
			{
				_emitterPositions.RemoveRange(1, _emitterPositions.Count - 1);
				_emitterPositions.AddRange(value);
			}
		}

		/*! Constructor
		 * \param positions Starting positions of emitter
         * \param blendEffect SpriteBlendMode to use
         * \param spriteMapName Name of sprite
		 * \param maxFrame Max amount of frames for sprite
		 * \param particlesPerSec How many particles per second are made
		 * \param particleAmount The max amount of particles available at one time
		 * \param emitLifetime How long the emitter stays active for
		 * \param particleLifetime How long the patrticle stays active for
		 * \param particleLifeRandomizer Randomizer to particleLifeTime
		 * \param startRotationRange Range of radians to start between
		 * \param minStartRotSpeed How slow a particle can start rotating
		 * \param maxStartRotSpeed How fast a particle can start rotating
		 * \param minEndRotSpeed How slow a particle can end rotating
		 * \param maxEndRotSpeed How fast a particle can end rotating
		 * \param minStartSize How small a particle can start
		 * \param maxStartSize How large a particle can start
		 * \param minEndSize How small a particle can end
		 * \param maxEndSpeed How large a particle can end
		 * \param minStartColor Minimum values for color to start with
		 * \param maxStartColor Maximum values for color to start with
		 * \param minEndColor Minimum values for color to end with
		 * \param maxEndColor Maximum values for color to end with */
		public SplineEmitter(List<Vector2> positions, SpriteBlendMode blendEffect,
			string spriteMapName, short maxFrame, float particlesPerSec, short particleAmount,
			float emitLifetime, float particleLifetime, float particleLifeRandomizer, float startRotRange,
			float minStartRotSpeed, float maxStartRotSpeed, float minEndRotSpeed, float maxEndRotSpeed,
			Vector2 minStartSize, Vector2 maxStartSize, Vector2 minEndSize, Vector2 maxEndSize,
			float minStartSpeed, float maxStartSpeed, float minEndSpeed, float maxEndSpeed,
			Vector4 minStartColor, Vector4 maxStartColor, Vector4 minEndColor, Vector4 maxEndColor)
			: base(blendEffect, spriteMapName, maxFrame, particlesPerSec, particleAmount, emitLifetime,
			particleLifetime, particleLifeRandomizer, startRotRange, minStartRotSpeed, maxStartRotSpeed,
			minEndRotSpeed, maxEndRotSpeed, minStartSize, maxStartSize, minEndSize, maxEndSize,
			minStartSpeed, maxStartSpeed, minEndSpeed, maxEndSpeed, minStartColor, maxStartColor,
			minEndColor, maxEndColor)
		{
			_currentIndex = 0;
			_emitterPositions = positions;
			_totalElapsedTime = 0;
			_returnVal = new Vector2[2];
		}

		/*! Updates spline emitter
         * \param elapsedTime The time elapsed since last tick in milliseconds. */
        public override void Update(float elapsedTime) {
            base.Update(elapsedTime);
			if (base.Alive && base.IsActive)
			{
				_totalElapsedTime += elapsedTime;
				_tempAmount = (int)(_totalElapsedTime * base._particlesPerSec);
				if (_tempAmount > 0)
				{
					_tempIndex = (_currentIndex + _tempAmount) % base._particles.Length;
					if (_currentIndex + _tempAmount < base._particles.Length)
					{
						for (; _currentIndex < _tempIndex; _currentIndex++)
						{
							calcPosition();
							base.intializeParticle(_returnVal[0], _returnVal[1], _currentIndex);
						}
					}
					else
					{
						for (; _currentIndex < base._particles.Length; _currentIndex++)
						{
							calcPosition();
							base.intializeParticle(_returnVal[0], _returnVal[1], _currentIndex);
						}
						_currentIndex = 0;
						for (; _currentIndex < _tempIndex; _currentIndex++)
						{
							calcPosition();
							base.intializeParticle(_returnVal[0], _returnVal[1], _currentIndex);
						}
					}
					_totalElapsedTime = 0;
				}
			}
        }

		/*! Randomly caluclates what spot along the spline and in which direction (normal or
		 * anit-normal) for a particle to start at and with */
		private void calcPosition()
		{
			if (_emitterPositions.Count == 1)
			{
				_returnVal[0] = _emitterPositions[0];
				_returnVal[1] = new Vector2((float)(base._randomizer.NextDouble() * 100 *
					(base._randomizer.Next(2) * 2 - 1)), (float)(base._randomizer.NextDouble() *
					100 * (base._randomizer.Next(2) * 2 - 1)));
				_returnVal[1].Normalize();
			}
			else
			{
				_tempIndex2 = base._randomizer.Next(_emitterPositions.Count - 1);
				_randVal = _randomizer.NextDouble();
				if (_tempIndex2 == 0)
				{
					_beginPoint = 2 * _emitterPositions[_tempIndex2] - _emitterPositions[_tempIndex2 + 1];
				}
				else
				{
					_beginPoint = _emitterPositions[_tempIndex2 - 1];
				}

				if (_tempIndex2 + 2 >= _emitterPositions.Count)
				{
					_endPoint = 2 * _emitterPositions[_tempIndex2 + 1] - _emitterPositions[_tempIndex2];
				}
				else
				{
					_endPoint = _emitterPositions[_tempIndex2 + 2];
				}

				_returnVal[0] = 0.5f * ((2 * _emitterPositions[_tempIndex2]) + (_emitterPositions[_tempIndex2 + 1] -
					_beginPoint) * (float)_randVal + (2 * _beginPoint - 5 * _emitterPositions[_tempIndex2] + 4 *
					_emitterPositions[_tempIndex2 + 1] - _endPoint) * (float)Math.Pow(_randVal, 2.0) + (3 *
					_emitterPositions[_tempIndex2] - _beginPoint - 3 * _emitterPositions[_tempIndex2 + 1] +
					_endPoint) * (float)Math.Pow(_randVal, 3.0));
				_returnVal[1] = 0.5f * ((_emitterPositions[_tempIndex2 + 1] - _beginPoint) + 2 * (2 * _beginPoint -
					5 * _emitterPositions[_tempIndex2] + 4 * _emitterPositions[_tempIndex2 + 1] - _endPoint) *
					(float)_randVal + 3 * (3 * _emitterPositions[_tempIndex2] - _beginPoint - 3 *
					_emitterPositions[_tempIndex2 + 1] + _endPoint) * (float)Math.Pow(_randVal, 2.0));
				_returnVal[1].Normalize();
				_returnVal[1] = new Vector2(-1 * _returnVal[1].Y, _returnVal[1].X);
			}
		}

		/*! Makes a copy of the given spline emitter
		* \param point SplineEmitter to be set */
		public void clone(ref SplineEmitter spline)
		{
			Vector2[] tempArray = _emitterPositions.ToArray();
			List<Vector2> positions = new List<Vector2>();
			for (int i = 0; i < tempArray.Length; i++)
			{
				positions.Add(tempArray[i]);
			}
			spline = new SplineEmitter(positions, base._blendEffect, base._spriteMapName, base._maxFrame, 
				base._particlesPerSec, base._particleAmount, base._emitLifetime, base._particleLifetime, 
				base._particleLifeRandomizer, base._startRotationRange, base._minStartRotSpeed, 
				base._maxStartRotSpeed, base._minEndRotSpeed, base._maxEndRotSpeed, base._minStartSize, 
				base._maxStartSize, base._minEndSize, base._maxEndSize, base._minStartSpeed, base._maxStartSpeed, 
				base._minEndSpeed, base._maxEndSpeed, base._minStartColor, base._maxStartColor, base._minEndColor, 
				base._maxEndColor);
		}

		/*! Disposes any extra components if needed
		* \param disposing Whether or not currently disposing */
		protected override void Dispose(bool disposing)
		{
            base.Dispose(disposing);
        }

    }
}
