/* \class PlayerInput
 * \class Input
 * Input.cs
 * 
 * \author Andrew Kane
 *
 *  Defines various classes used to handle the input, based upon an XML file
 */

#region Using Statements
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using System.IO;
using System.Xml.Serialization;
#endregion

using rhodes.graphics;

namespace rhodes.input
{
    /// <summary>
    /// Class that contains a single player's input state
    /// </summary>
    public class PlayerInput
    {
        #region Attributes
        /// <summary>
        /// The PlayerIndex of the GamePad that this player is using
        /// </summary>
        private PlayerIndex index;

        /// <summary>
        /// The sensitivity of the controller's thumbsticks
        /// </summary>
        public float thumbstickSens;
        /// <summary>
        /// The sensitivty of the controller's triggers
        /// </summary>
        public float triggerSens;
        /// <summary>
        /// Which inputs will be used for movement
        /// </summary>
        public Input.Controls[] moveControl;
        /// <summary>
        /// Which inputs will be used for aiming
        /// </summary>
        public Input.Controls[] aimControl;
        /// <summary>
        /// Which inputs will be used for firing
        /// </summary>
        public Input.Controls[] fireControl;
		/// <summary>
		/// Which inputs will be used for jumping
		/// </summary>
		public Input.Controls[] controlJump;
        /// <summary>
        /// Which inputs will be used for selecting the laser weapon
        /// </summary>
        public Input.Controls[] controlLaser;
        /// <summary>
        /// Which inputs will be used for selecting the missile weapon
        /// </summary>
        public Input.Controls[] controlMissile;
        /// <summary>
        /// Which inputs will be used for using the melee weapon
        /// </summary>
        public Input.Controls[] controlMelee;
        /// <summary>
        /// Which inputs will be used for using the gun
        /// </summary>
        public Input.Controls[] controlGun;
        /// <summary>
        /// Which inputs will be used for pausing the game
        /// </summary>
        public Input.Controls[] controlPause;
#if !XBOX
        /// <summary>
        /// Whether this player can use the keyboard or not
        /// </summary>
        private bool keyboard;
        /// <summary>
        /// The state of the keyboard
        /// </summary>
        private KeyboardState keyboardState;
        /// <summary>
        /// The state of the mouse
        /// </summary>
        private MouseState mouseState;

        /// <summary>
        /// Which keys will be used for moving upwards
        /// </summary>
        public Keys[] keyboardMoveUp;
        /// <summary>
        /// Which keys will be used for moving downwards
        /// </summary>
        public Keys[] keyboardMoveDown;
        /// <summary>
        /// Which keys will be used for moving to the left
        /// </summary>
        public Keys[] keyboardMoveLeft;
        /// <summary>
        /// Which keys will be used for moving to the right
        /// </summary>
        public Keys[] keyboardMoveRight;
        /// <summary>
        /// Which keys will be used for firing
        /// </summary>
        public Keys[] keyboardFireControl;
		/// <summary>
		/// Which keys will be used for jumping
		/// </summary>
		public Keys[] keyboardJump;
        /// <summary>
        /// Which keys will be used for selecting the laser weapon
        /// </summary>
        public Keys[] keyboardLaser;
        /// <summary>
        /// Which keys will be used for selecting the missile weapon
        /// </summary>
        public Keys[] keyboardMissile;
        /// <summary>
        /// Which keys will be used for using the melee weapon
        /// </summary>
        public Keys[] keyboardMelee;
        /// <summary>
        /// Which keys will be used for using the gun
        /// </summary>
        public Keys[] keyboardGun;
        /// <summary>
        /// Which keys will be used for pausing the game
        /// </summary>
        public Keys[] keyboardPause;
        /// <summary>
        /// Which mouse buttons will be using for firing
        /// </summary>
        public Input.MouseButtons[] fireButtons;
#endif

        /// <summary>
        /// The state of this player's GamePad
        /// </summary>
        private GamePadState gamePadState;
        #endregion

        #region Constructors
        /// <summary>
        /// Default Constructor
        /// </summary>
        public PlayerInput()
        {
#if !XBOX
            keyboard = false;
#endif
            index = PlayerIndex.One;
            gamePadState = GamePad.GetState(PlayerIndex.One);
            loadDefaultControls();
            return;
        }

        /// <summary>
        /// Constructor for PlayerInput
        /// </summary>
        /// <param name="pi">The PlayerIndex of the input</param>
        public PlayerInput(PlayerIndex pi)
        {
#if !XBOX
            keyboard = false;
#endif
            index = pi;
            gamePadState = GamePad.GetState(pi);
            loadDefaultControls();
            return;
        }

        /// <summary>
        /// Constructor for PlayerInput
        /// </summary>
        /// <param name="pi">The PlayerIndex of the input</param>
        /// <param name="filename">The filename containing the player's settings</param>
        public PlayerInput(PlayerIndex pi, string filename)
        {
#if !XBOX
            keyboard = false;
#endif
            index = pi;
            gamePadState = GamePad.GetState(pi);
            if(load(filename) == false)
                loadDefaultControls();
            return;
        }
        #endregion

        #region Settings Loading & Saving
        /// <summary>
        /// Loads controller/keyboard stetings from a XML file
        /// </summary>
        /// <param name="filename">The XML file of which to load</param>
        /// <returns>Whether the loading succeeded or not</returns>
        public bool load(string filename)
        {
            bool ret = true;
            FileStream fs = null;
            PlayerInput pi = null;
            try
            {
                fs = File.OpenRead(filename);
                XmlSerializer serializer = new XmlSerializer(typeof(PlayerInput));
                pi = (PlayerInput)serializer.Deserialize(fs);
                moveControl = pi.moveControl;
                aimControl = pi.aimControl;
                fireControl = pi.fireControl;
				controlJump = pi.controlJump;
                controlLaser = pi.controlLaser;
                controlMissile = pi.controlMissile;
                controlMelee = pi.controlMelee;
                controlGun = pi.controlGun;
                controlPause = pi.controlPause;
#if !XBOX
                keyboardMoveUp = pi.keyboardMoveUp;
                keyboardMoveDown = pi.keyboardMoveDown;
                keyboardMoveLeft = pi.keyboardMoveLeft;
                keyboardMoveRight = pi.keyboardMoveRight;
                keyboardFireControl = pi.keyboardFireControl;
				keyboardJump = pi.keyboardJump;
                keyboardLaser = pi.keyboardLaser;
                keyboardMissile = pi.keyboardMissile;
                keyboardMelee = pi.keyboardMelee;
                keyboardGun = pi.keyboardGun;
                keyboardPause = pi.keyboardPause;
                fireButtons = pi.fireButtons;
                useKeyboard = pi.useKeyboard;
#endif
            }
            catch (Exception)
            {
                ret = false;
            }
            finally
            {
                if (fs != null)
                    fs.Close();
            }

            return ret;
        }

        /// <summary>
        /// Saves the current controller/keyboard settings into an XML file
        /// </summary>
        /// <param name="filename">The file of which to save the settings to</param>
        /// <returns>Whether the file was successfully saved or not</returns>
        public bool save(string filename)
        {
            bool ret = true;
            Stream stream = null;
            try
            {
                stream = File.Create(filename);
                XmlSerializer serializer = new XmlSerializer(typeof(PlayerInput));
                serializer.Serialize(stream, this);
            }
            catch (Exception)
            {
                ret = false;
            }
            finally
            {
                if (stream != null)
                    stream.Close();
            }
            return ret;
        }

        /// <summary>
        /// Loads the default configuration for the controller
        /// </summary>
        protected void loadDefaultControls()
        {
#if !XBOX
            keyboardMoveUp = new Keys[] { Keys.Up, Keys.W };
            keyboardMoveDown = new Keys[] { Keys.Down, Keys.S };
            keyboardMoveLeft = new Keys[] { Keys.Left, Keys.A };
            keyboardMoveRight = new Keys[] { Keys.Right, Keys.D };
            keyboardFireControl = new Keys[] { Keys.LeftControl, Keys.RightControl };
            keyboardMissile = new Keys[] { Keys.D2, Keys.NumPad2 };
            keyboardLaser = new Keys[] { Keys.D1, Keys.NumPad1 };
            keyboardMelee = new Keys[] { Keys.LeftShift, Keys.RightShift };
            keyboardGun = new Keys[] { Keys.D4, Keys.NumPad4 };
			keyboardJump = new Keys[] { Keys.Space };
            keyboardPause = new Keys[] { Keys.Enter };
            fireButtons = new Input.MouseButtons[] { Input.MouseButtons.Left };
#endif
            moveControl = new Input.Controls[] { Input.Controls.LeftThumbstick, Input.Controls.DPad };
            aimControl = new Input.Controls[] { Input.Controls.RightThumbstick };
            fireControl = new Input.Controls[] { Input.Controls.RightTrigger };
			controlJump = new Input.Controls[] { Input.Controls.A, Input.Controls.LeftStick, Input.Controls.LeftTrigger };
            controlLaser = new Input.Controls[] { Input.Controls.Y };
            controlMissile = new Input.Controls[] { Input.Controls.B };
			controlMelee = new Input.Controls[] { Input.Controls.LeftShoulder, Input.Controls.RightShoulder };
            controlGun = new Input.Controls[] { Input.Controls.X };
            controlPause = new Input.Controls[] { Input.Controls.Start };
            thumbstickSens = 0.5f;
            triggerSens = 0.5f;
            return;
        }
#endregion

        #region Raw Exchange
        /// <summary>
        /// Set the current attributes to another PlayerInput
        /// </summary>
        /// <param name="pi">The PlayerInput we are setting this one to</param>
        public void set(PlayerInput pi)
        {
            this.gamePadState = new GamePadState(pi.rawGamePadState.ThumbSticks, pi.rawGamePadState.Triggers, pi.rawGamePadState.Buttons, pi.rawGamePadState.DPad);
            this.keyboardState = new KeyboardState(pi.rawKeyboardState.GetPressedKeys());
            this.mouseState = new MouseState(pi.rawMouseState.X, pi.rawMouseState.Y,
                pi.rawMouseState.ScrollWheelValue, pi.rawMouseState.LeftButton, pi.rawMouseState.MiddleButton,
                pi.rawMouseState.RightButton, pi.rawMouseState.XButton1, pi.rawMouseState.XButton2);
            return;
        }

#if !XBOX
        /// <summary>
        /// Obtains the raw keyboard state of this PlayerInput
        /// </summary>
        public KeyboardState rawKeyboardState
        {
            get { return keyboardState; }
        }

        /// <summary>
        /// Obtains the raw mouse state of this PlayerInput
        /// </summary>
        public MouseState rawMouseState
        {
            get { return mouseState; }
        }
#endif

        /// <summary>
        /// Obtains the raw game pad state of this PlayerInput
        /// </summary>
        public GamePadState rawGamePadState
        {
            get { return gamePadState; }
        }
        #endregion

        #region Keyboard Accessibility
#if !XBOX
        /// <summary>
        /// Whether to monitor the keyboard for this player or not
        /// </summary>
        public bool useKeyboard
        {
            get
            {
                return keyboard;
            }
            set
            {
                keyboard = value;
            }
        }

        /// <summary>
        /// Check if any of a set of keys is pressed
        /// </summary>
        /// <param name="controlArr">Which keys to check</param>
        /// <returns>Whether any of the keys are pressed or not</returns>
        protected ButtonState checkKeysPressed(Keys[] controlArr)
        {
            foreach (Keys key in controlArr)
                if (keyboardState.IsKeyDown(key))
                    return ButtonState.Pressed;
            return ButtonState.Released;
        }
#endif
        #endregion

        #region Updating Functions
        /// <summary>
        /// Updates the input (both controllers and keyboard)
        /// </summary>
        public void update()
        {
            updateController();
#if !XBOX
            if(keyboard)
                updateKeyboard();
#endif
            return;
        }

        /// <summary>
        /// Updates the controller input state
        /// </summary>
        protected void updateController()
        {
            gamePadState = GamePad.GetState(index);
            return;
        }

#if !XBOX
        /// <summary>
        /// Updates the keyboard input state
        /// </summary>
        protected void updateKeyboard()
        {
            keyboardState = Keyboard.GetState();
            mouseState = Mouse.GetState();

			// We may add clamping in future releases
			//Vector2 cursorPos = new Vector2(
			//    MathHelper.Clamp(Mouse.GetState().X, 0f, GraphicsCore.getSingleton.Width),
			//    MathHelper.Clamp(Mouse.GetState().Y, 0f, GraphicsCore.getSingleton.Height)
			//);

			Vector2 cursorPos = new Vector2(Mouse.GetState().X, Mouse.GetState().Y);

			Mouse.SetPosition((int)cursorPos.X, (int)cursorPos.Y);

            return;
        }
#endif
        #endregion

        #region Controller Functions
        /// <summary>
        /// Makes the GamePad rumble
        /// </summary>
        public void rumble()
        {
            GamePad.SetVibration(index, 0.5f, 0.5f);
            return;
        }

        /// <summary>
        /// Makes the GamePad rumble
        /// </summary>
        /// <param name="leftMotor">The amount of rumble for the left motor (0.0 ~ 1.0)</param>
        /// <param name="rightMotor">The amount of rumble for the right motor (0.0 ~ 1.0)</param>
        public void rumble(float leftMotor, float rightMotor)
        {
            GamePad.SetVibration(index, leftMotor, rightMotor);
            return;
        }

        /// <summary>
        /// Whether the controller is connected or not
        /// </summary>
        public bool connected
        {
            get
            {
                return gamePadState.IsConnected;
            }
        }

        /// <summary>
        /// Gets the maximum amount of movement on a number of thumbstikc
        /// </summary>
        /// <param name="thumbsticks">Which thumbsticks to check</param>
        /// <param name="direction">Which direction to check</param>
        /// <returns>The amount of maximum movement</returns>
        protected float getThumbstickAmount(Input.Controls[] thumbsticks, Input.Direction direction)
        {
            if (connected == false)
                return 0.0f;
            float best = 0.0f;
            float cur = 0.0f;
            foreach(Input.Controls control in thumbsticks)
            {
                cur = getThumbstickAmount(control, direction);
                if(best < Math.Abs(cur))
                    best = cur;
            }
            if(direction == Input.Direction.Down || direction == Input.Direction.Left)
                best *= -1.0f;
            return best;
        }

        /// <summary>
        /// Gets the amount of movement on a single thumbstick
        /// </summary>
        /// <param name="thumbstick">Which thumbstick to check</param>
        /// <param name="direction">Which direction to check</param>
        /// <returns>The amount of movement</returns>
        protected float getThumbstickAmount(Input.Controls thumbstick, Input.Direction direction)
        {
            if (connected == false)
                return 0;
            if(thumbstick == Input.Controls.LeftThumbstick)
            {
                switch(direction)
                {
                    case Input.Direction.Up:
                        return gamePadState.ThumbSticks.Left.Y;
                    case Input.Direction.Down:
                        return -gamePadState.ThumbSticks.Left.Y;
                    case Input.Direction.Left:
                        return -gamePadState.ThumbSticks.Left.X;
                    case Input.Direction.Right:
                        return gamePadState.ThumbSticks.Left.X;
                }
            }
            else if(thumbstick == Input.Controls.RightThumbstick)
            {
                switch(direction)
                {
                    case Input.Direction.Up:
                        return gamePadState.ThumbSticks.Right.Y;
                    case Input.Direction.Down:
                        return -gamePadState.ThumbSticks.Right.Y;
                    case Input.Direction.Left:
                        return -gamePadState.ThumbSticks.Right.X;
                    case Input.Direction.Right:
                        return gamePadState.ThumbSticks.Right.X;
                }
            }
            else if(thumbstick == Input.Controls.DPad)
            {
                switch(direction)
                {
                    case Input.Direction.Up:
                        return (gamePadState.DPad.Up == ButtonState.Pressed) ? 1 : 0;
                    case Input.Direction.Down:
                        return (gamePadState.DPad.Down == ButtonState.Pressed) ? 1 : 0;
                    case Input.Direction.Left:
                        return (gamePadState.DPad.Left == ButtonState.Pressed) ? 1 : 0;
                    case Input.Direction.Right:
                        return (gamePadState.DPad.Right == ButtonState.Pressed) ? 1 : 0;
                }
            }

            return 0;
        }

        /// <summary>
        /// Gets the state of a number of thumbsticks
        /// </summary>
        /// <param name="thumbsticks">Which thumbsticks to check</param>
        /// <param name="direction">Which direction to check</param>
        /// <returns>Whether the direction is considered "pressed" or not</returns>
        protected ButtonState getThumbstickState(Input.Controls[] thumbsticks, Input.Direction direction)
        {
            if (connected == false)
                return ButtonState.Released;
            foreach(Input.Controls control in thumbsticks)
                if(getThumbstickState(control, direction) == ButtonState.Pressed)
                    return ButtonState.Pressed;
            return ButtonState.Released;
        }

        /// <summary>
        /// Gets the state of a thumbstick
        /// </summary>
        /// <param name="thumbstick">Which thumbstick to check</param>
        /// <param name="direction">Which direction to check</param>
        /// <returns>Whether the direction is considered "pressed" or not</returns>
        protected ButtonState getThumbstickState(Input.Controls thumbstick, Input.Direction direction)
        {
            if (connected == false)
                return ButtonState.Released;
            if(thumbstick == Input.Controls.LeftThumbstick)
            {
                switch(direction)
                {
                    case Input.Direction.Up:
                        return (gamePadState.ThumbSticks.Left.Y > thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
                    case Input.Direction.Down:
                        return (gamePadState.ThumbSticks.Left.Y < -thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
                    case Input.Direction.Left:
                        return (gamePadState.ThumbSticks.Left.X < -thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
                    case Input.Direction.Right:
                        return (gamePadState.ThumbSticks.Left.X > thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
                }
            }
            else if(thumbstick == Input.Controls.RightThumbstick)
            {
                switch(direction)
                {
                    case Input.Direction.Up:
                        return (gamePadState.ThumbSticks.Right.Y > thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
                    case Input.Direction.Down:
                        return (gamePadState.ThumbSticks.Right.Y < -thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
                    case Input.Direction.Left:
                        return (gamePadState.ThumbSticks.Right.X < -thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
                    case Input.Direction.Right:
                        return (gamePadState.ThumbSticks.Right.X > thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
                }
            }
            else if(thumbstick == Input.Controls.DPad)
            {
                switch(direction)
                {
                    case Input.Direction.Up:
                        return gamePadState.DPad.Up;
                    case Input.Direction.Down:
                        return gamePadState.DPad.Down;
                    case Input.Direction.Left:
                        return gamePadState.DPad.Left;
                    case Input.Direction.Right:
                        return gamePadState.DPad.Right;
                }
            }

            return ButtonState.Released;
        }

        /// <summary>
        /// Check if any of a set of controls is pressed
        /// </summary>
        /// <param name="controlArr">Which controls to check</param>
        /// <returns>Whether any of the controls are pressed or not</returns>
        protected ButtonState checkControlsPressed(Input.Controls[] controlArr)
        {
            if (connected == false)
                return ButtonState.Released;
            foreach(Input.Controls bind in controlArr)
            {
                if(getButtonState(bind) == ButtonState.Pressed)
                    return ButtonState.Pressed;
            }
            return ButtonState.Released;
        }

        /// <summary>
        /// Check if a control is pressed
        /// </summary>
        /// <param name="control">Which control to check</param>
        /// <returns>Whether the control is pressed or not</returns>
        protected ButtonState getButtonState(Input.Controls control)
        {
            if (connected == false)
                return ButtonState.Released;
            else if (control == Input.Controls.Back)
                return gamePadState.Buttons.Back;
            else if (control == Input.Controls.Start)
                return gamePadState.Buttons.Start;
            else if (control == Input.Controls.A)
                return gamePadState.Buttons.A;
            else if (control == Input.Controls.B)
                return gamePadState.Buttons.B;
            else if (control == Input.Controls.X)
                return gamePadState.Buttons.X;
            else if (control == Input.Controls.Y)
                return gamePadState.Buttons.Y;
            else if (control == Input.Controls.LeftShoulder)
                return gamePadState.Buttons.LeftShoulder;
            else if (control == Input.Controls.LeftTrigger)
                return leftTrigger();
            else if (control == Input.Controls.LeftStick)
                return gamePadState.Buttons.LeftStick;
            else if (control == Input.Controls.RightShoulder)
                return gamePadState.Buttons.RightShoulder;
            else if (control == Input.Controls.RightTrigger)
                return rightTrigger();
            else if (control == Input.Controls.RightStick)
                return gamePadState.Buttons.RightStick;

            return ButtonState.Released;
        }

        #region Left Thumbstick
        protected ButtonState leftThumbstickUp()
        {
            return (gamePadState.ThumbSticks.Left.Y >= thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
        }

        protected ButtonState leftThumbstickDown()
        {
            return (gamePadState.ThumbSticks.Left.Y < (-thumbstickSens)) ? ButtonState.Pressed : ButtonState.Released;
        }

        protected ButtonState leftThumbstickLeft()
        {
            return (gamePadState.ThumbSticks.Left.X < (-thumbstickSens)) ? ButtonState.Pressed : ButtonState.Released;
        }

        protected ButtonState leftThumbstickRight()
        {
            return (gamePadState.ThumbSticks.Left.X >= thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
        }
        #endregion

        #region Right Thumbstick
        protected ButtonState rightThumbstickUp()
        {
            return (gamePadState.ThumbSticks.Right.Y >= thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
        }

        protected ButtonState rightThumbstickDown()
        {
            return (gamePadState.ThumbSticks.Right.Y < thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
        }

        protected ButtonState rightThumbstickLeft()
        {
            return (gamePadState.ThumbSticks.Right.X < thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
        }

        protected ButtonState rightThumbstickRight()
        {
            return (gamePadState.ThumbSticks.Right.X >= thumbstickSens) ? ButtonState.Pressed : ButtonState.Released;
        }
        #endregion

        #region Triggers
        protected ButtonState leftTrigger()
        {
            return (gamePadState.Triggers.Left >= triggerSens) ? ButtonState.Pressed : ButtonState.Released;
        }

        protected ButtonState rightTrigger()
        {
            return (gamePadState.Triggers.Right >= triggerSens) ? ButtonState.Pressed : ButtonState.Released;
        }
        #endregion
        #endregion

        #region Mouse Functions
#if !XBOX
        /// <summary>
        /// Check if any of a set of mouse buttons is pressed
        /// </summary>
        /// <param name="buttons">Which buttons to check</param>
        /// <returns>If any of the mouse buttons are pressed</returns>
        protected ButtonState checkMousePressed(Input.MouseButtons[] buttons)
        {
            foreach (Input.MouseButtons button in buttons)
                if (getMouseState(button) == ButtonState.Pressed)
                    return ButtonState.Pressed;
            return ButtonState.Released;
        }

        /// <summary>
        /// Check if a mouse button is pressed
        /// </summary>
        /// <param name="button">Which buttons to check</param>
        /// <returns>If the mouse button is pressed</returns>
        protected ButtonState getMouseState(Input.MouseButtons button)
        {
            if (button == Input.MouseButtons.Left)
                return mouseState.LeftButton;
            else if (button == Input.MouseButtons.Middle)
                return mouseState.MiddleButton;
            else if (button == Input.MouseButtons.Right)
                return mouseState.RightButton;
            return ButtonState.Released;
        }
#endif
        #endregion

        #region Player Action Functions
        #region Player Movement and Aiming
        /// <summary>
        /// Obtains the value (from -1.0 to +1.0) of where the thumbstick is at vertically
        /// </summary>
        public float moveVertical
        {
            get
            {
				float amount = getThumbstickAmount(moveControl, Input.Direction.Down);
				if (amount == 0)
					amount = getThumbstickAmount(moveControl, Input.Direction.Up);
				else
					amount *= 1.0f;
				return amount;
            }
        }

        /// <summary>
        /// Obtains the value (from -1.0 to +1.0) of where the thumbstick is at horizontally
        /// </summary>
        public float moveHorizontal
        {
            get
            {
				float amount = getThumbstickAmount(moveControl, Input.Direction.Left);
				if (amount == 0)
					amount = getThumbstickAmount(moveControl, Input.Direction.Right);
				else
					amount *= 1.0f;
                return amount;
            }
        }

        /// <summary>
        /// Obtains whether the thumbstick is pushed up or not
        /// </summary>
        protected ButtonState moveUp
        {
            get
            {
                return getThumbstickState(moveControl, Input.Direction.Up);
            }
        }

        /// <summary>
        /// Obtains whether the thumbstick is pushed down or not
        /// </summary>
        protected ButtonState moveDown
        {
            get
            {
                return getThumbstickState(moveControl, Input.Direction.Down);
            }
        }

        /// <summary>
        /// Obtains whether the thumbstick is pushed left or not
        /// </summary>
        protected ButtonState moveLeft
        {
            get
            {
                return getThumbstickState(moveControl, Input.Direction.Left);
            }
        }

        /// <summary>
        /// Obtains whether the thumbstick is pushed right or not
        /// </summary>
        protected ButtonState moveRight
        {
            get
            {
                return getThumbstickState(moveControl, Input.Direction.Right);
            }
        }

        /// <summary>
        /// Obtains the value (from -1.0 to +1.0) of where the thumbstick is at vertically
        /// </summary>
        public float aimVertical
        {
            get
            {
                return getThumbstickAmount(aimControl, Input.Direction.Up);
            }
        }

        /// <summary>
        /// Obtains the value (from -1.0 to +1.0) of where the thumbstick is at horizontally
        /// </summary>
        public float aimHorizontal
        {
            get
            {
                return getThumbstickAmount(aimControl, Input.Direction.Left);
            }
        }

        /// <summary>
        /// Whether the thumbstick is being aimed up or not
        /// </summary>
        public ButtonState aimUp
        {
            get
            {
                return getThumbstickState(aimControl, Input.Direction.Up);
            }
        }

        /// <summary>
        /// Whether the thumbstick is being aimed down or not
        /// </summary>
        public ButtonState aimDown
        {
            get
            {
                return getThumbstickState(aimControl, Input.Direction.Down);
            }
        }

        /// <summary>
        /// Whether the thumbstick is being aimed to the left or not
        /// </summary>
        public ButtonState aimLeft
        {
            get
            {
                return getThumbstickState(aimControl, Input.Direction.Left);
            }
        }

        /// <summary>
        /// Whether the thumbstick is being aimed to the right or not
        /// </summary>
        public ButtonState aimRight
        {
            get
            {
                return getThumbstickState(aimControl, Input.Direction.Right);
            }
        }

#if !XBOX
        /// <summary>
        /// Whether the UP button is pressed or not
        /// </summary>
        public ButtonState up
        {
            get
            {
                return (checkKeysPressed(keyboardMoveUp) == ButtonState.Pressed ||
                    moveUp == ButtonState.Pressed) ? ButtonState.Pressed : ButtonState.Released;
            }
        }

        /// <summary>
        /// Whether the DOWN button is pressed or not
        /// </summary>
        public ButtonState down
        {
            get
            {
                return (checkKeysPressed(keyboardMoveDown) == ButtonState.Pressed ||
                    moveDown == ButtonState.Pressed) ? ButtonState.Pressed : ButtonState.Released;
            }
        }

        /// <summary>
        /// Whether the LEFT button is pressed or not
        /// </summary>
        public ButtonState left
        {
            get
            {
                return (checkKeysPressed(keyboardMoveLeft) == ButtonState.Pressed ||
                    moveLeft == ButtonState.Pressed) ? ButtonState.Pressed : ButtonState.Released;
            }
        }

        /// <summary>
        /// Whether the RIGHT button is pressed or not
        /// </summary>
        public ButtonState right
        {
            get
            {
                return (checkKeysPressed(keyboardMoveRight) == ButtonState.Pressed ||
                    moveRight == ButtonState.Pressed) ? ButtonState.Pressed : ButtonState.Released;
            }
        }

        /// <summary>
        /// Obtains the amount of scroll from the mouse wheel
        /// </summary>
        public int mouseWheel
        {
            get
            {
                return mouseState.ScrollWheelValue;
            }
        }

        /// <summary>
        /// Obtains the current mouse position (X, Y).
		/// (The camera's position and zoom must be factored in since
		/// the cammera is always moving and zooming in and out.)
        /// </summary>
        public Vector2 mousePosition
        {
            get
            {
				return (Camera2D.getSingleton.Position +
					(new Vector2(mouseState.X, mouseState.Y)/Camera2D.getSingleton.Zoom));
            }
        }
#endif
        #endregion

        #region Attack/Fire
        /// <summary>
        /// Whether the firing button is pressed or not
        /// </summary>
        public ButtonState fire
        {
            get
            {
#if !XBOX
                return (checkControlsPressed(fireControl) == ButtonState.Pressed ||
                    checkKeysPressed(keyboardFireControl) == ButtonState.Pressed ||
                    checkMousePressed(fireButtons) == ButtonState.Pressed) ?
                    ButtonState.Pressed : ButtonState.Released;
#else
                return checkControlsPressed(fireControl);
#endif
            }
		}
		#endregion

		#region Weapon Buttons
		/// <summary>
        /// Whether the jump button is pressed or not
        /// </summary>
        public ButtonState jump
        {
            get
            {
#if !XBOX
                //return (moveUp == ButtonState.Pressed || up == ButtonState.Pressed)
                   // ? ButtonState.Pressed : ButtonState.Released;
				//return (up == ButtonState.Pressed || moveUp == ButtonState.Pressed)
				//	? ButtonState.Pressed : ButtonState.Released;
				return (checkControlsPressed(controlJump) == ButtonState.Pressed ||
					checkKeysPressed(keyboardJump) == ButtonState.Pressed) ?
					ButtonState.Pressed : ButtonState.Released;
#else
				return (checkControlsPressed(controlJump) == ButtonState.Pressed
                    ? ButtonState.Pressed : ButtonState.Released;
#endif
            }
        }

        /// <summary>
        /// Whether the laser attack button is pressed or not
        /// </summary>
        public ButtonState laser
        {
            get
            {
#if !XBOX
                return (checkControlsPressed(controlLaser) == ButtonState.Pressed ||
                    checkKeysPressed(keyboardLaser) == ButtonState.Pressed) ?
                    ButtonState.Pressed : ButtonState.Released;
#else
                
                return checkControlsPressed(controlLaser);
#endif
            }
        }

        /// <summary>
        /// Whether the missile attack button is pressed or not
        /// </summary>
        public ButtonState missile
        {
            get
            {
#if !XBOX
                return (checkControlsPressed(controlMissile) == ButtonState.Pressed ||
                    checkKeysPressed(keyboardMissile) == ButtonState.Pressed) ?
                    ButtonState.Pressed : ButtonState.Released;
#else
                return checkControlsPressed(controlMissile);
#endif
            }
        }

        /// <summary>
        /// Whether the Melee attack button is pressed or not
        /// </summary>
        public ButtonState melee
        {
            get
            {
#if !XBOX
                return (checkControlsPressed(controlMelee) == ButtonState.Pressed ||
                    checkKeysPressed(keyboardMelee) == ButtonState.Pressed) ?
                    ButtonState.Pressed : ButtonState.Released;
#else
                return checkControlsPressed(controlMelee);
#endif
            }
        }

        /// <summary>
        /// Whether the Gun button is pressed or not
        /// </summary>
        public ButtonState gun
        {
            get
            {
#if !XBOX
                return (checkControlsPressed(controlGun) == ButtonState.Pressed ||
                    checkKeysPressed(keyboardGun) == ButtonState.Pressed) ?
                    ButtonState.Pressed : ButtonState.Released;
#else
                return checkControlsPressed(controlGun);
#endif
            }
        }
        #endregion

        #region Miscellaneous
        /// <summary>
        /// Whether the Pause button is pressed or not
        /// </summary>
        public ButtonState pause
        {
            get
            {
#if !XBOX
                return (checkControlsPressed(controlPause) == ButtonState.Pressed ||
                    checkKeysPressed(keyboardPause) == ButtonState.Pressed) ?
                    ButtonState.Pressed : ButtonState.Released;
#else
                return checkControlsPressed(controlPause);
#endif
            }
        }
        #endregion
        #endregion
    }

    /// <summary>
    /// Input class that will contain the current and previous input states for each player
    /// </summary>
    public class Input
    {
        #region Input Enumerations
        /// <summary>
        /// Enumeration to set various Controller buttons, joysticks, etc.
        /// </summary>
        public enum Controls
        {
            LeftThumbstick,
            RightThumbstick,
            DPad,
            Back,
            Start,
            A,
            B,
            X,
            Y,
            LeftShoulder,
            LeftTrigger,
            LeftStick,
            RightShoulder,
            RightTrigger,
            RightStick
        };

        /// <summary>
        /// Enumeration to specify directions
        /// </summary>
        public enum Direction
        {
            Up,
            Down,
            Left,
            Right
        };

#if !XBOX
        /// <summary>
        /// Enumeration to specify mouse buttons
        /// </summary>
        public enum MouseButtons
        {
            Left,
            Right,
            Middle
        };
#endif
        #endregion

        #region Attributes
        /// <summary>
        /// Holds the current input for the players
        /// </summary>
        private PlayerInput[] currentInput;
        /// <summary>
        /// Holds the previous input for the players
        /// </summary>
        private PlayerInput[] previousInput;
        #endregion

        #region Constructor
        /// <summary>
        /// The default Input constructor
        /// </summary>
        public Input()
        {
            currentInput = new PlayerInput[4] {
				new PlayerInput(PlayerIndex.One, "player1.xml"),
                new PlayerInput(PlayerIndex.Two, "player2.xml"),
                new PlayerInput(PlayerIndex.Three, "player3.xml"),
                new PlayerInput(PlayerIndex.Four, "player4.xml")
            };
            previousInput = new PlayerInput[4] {
			    new PlayerInput(PlayerIndex.One, "player1.xml"),
                new PlayerInput(PlayerIndex.Two, "player2.xml"),
                new PlayerInput(PlayerIndex.Three, "player3.xml"),
                new PlayerInput(PlayerIndex.Four, "player4.xml")
            };
            //previousInput = currentInput;
			//currentInput[0].save("player1.xml");
#if !XBOX
            // By default, player One is using the keyboard if it has no controller connected
			if (!currentInput[0].connected)
				currentInput[0].useKeyboard = true;
#endif
            return;
        }
        #endregion

        #region Keyboard Accessability
#if !XBOX
        /// <summary>
        /// Sets which player can use the keyboard
        /// </summary>
        /// <param name="pi">The PlayerIndex of which player can be using the keyboard</param>
        public void setKeyboardPlayer(PlayerIndex pi)
        {
            currentInput[(int)pi].useKeyboard = true;
            //previousInput[(int)pi].useKeyboard = true;
            return;
        }
#endif
        #endregion

        #region Input Access
        /// <summary>
        /// Provides easy access to each player's current input
        /// </summary>
        public PlayerInput[] current
        {
            get
            {
                return currentInput;
            }
        }

        /// <summary>
        /// Provides easy access to each player's previous input
        /// </summary>
        public PlayerInput[] previous
        {
            get
            {
                return previousInput;
            }
        }
        #endregion

        #region Updating Function
        /// <summary>
        /// Updates the current and previous inputs for each player
        /// </summary>
        public void update()
        {
            //previousInput = currentInput;
            //Array.Copy(currentInput, previousInput, currentInput.Length);
            for(int i = 0; i < previousInput.Length; ++i)
                previousInput[i].set(currentInput[i]);
            foreach(PlayerInput input in currentInput)
/*
#if !XBOX
                if(input.connected || input.useKeyboard)
#else
				if(input.connected)
#endif
*/
                    input.update();
            return;
        }
        #endregion
    }
}
