/*
 *  Author: Josh Wilson, David Huynh and Robert Yates
 *  Copyright 2007. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using RhodesProcessorLib;
using SedohrProcessorLib;
using rhodes.graphics;
using rhodes.graphics.particles;
using rhodes.graphics.particles.emitters;
using rhodes.engine;
using rhodes.physics;
using rhodes.audio;
using rhodes.input;

using sedohr.player;
using sedohr.weapon;
using sedohr.objects;

namespace sedohr
{
	/// <summary>
	/// Represents the main game that is played with 1 to 4 people.
	/// </summary>
	class DemoGameState : GameState
	{
		/// <summary>
		/// Contains the name of the game state
		/// </summary>
		public readonly static string GAME_STATE_NAME = "demo";

		/// <summary>
		/// The number of kills to required for a player
		/// to win the game
		/// </summary>
		private const int MAX_KILL_COUNT = 10;

		/// <summary>
		/// True, if we want to display the mouse cursor for a player using
		/// the keyboard and mouse. False, to hide it.
		/// </summary>
		private const bool ENABLE_MOUSE_CURSOR = false;

		/// <summary>
		/// Contains the PlayerManager that controls all the players in the game state
		/// </summary>
		private PlayerManager playerManager;

		/// <summary>
		/// Contains the mouse cursor sprite
		/// </summary>
		private Sprite mouseCursor;

		/// <summary>
		/// Contains the background sprite
		/// </summary>
		private Sprite background;
    
		/// <summary>
		/// Contains the input used by the player
		/// </summary>
		private Input input;

		/// <summary>
		/// The manager of objects in the scene
		/// </summary>
		private ObjectManager objectManager;

		/// <summary>
		/// Contains a list of all sprite items in the HUD
		/// </summary>
		private SortedDictionary<String, Sprite> _hudItems;

		/// <summary>
		/// Contains a list of all message items in the hud
		/// </summary>
		private SortedDictionary<String, Message> _hudMsgItems;

		/// <summary>
		/// Contains a list of the initial Hud Positions of the healthbars
		/// </summary>
		private Vector2[] _initHealthbarHUDPositions = new Vector2 [4];

		/// <summary>
		/// Contains a list of the initial HUD Positions of the cooldown bars
		/// </summary>
		private Vector2[] _initCooldownHUDPositions = new Vector2 [4];

		/// <summary>
		/// The position of the HUD sprite.
		/// </summary>
		private Vector2 hudSpritePosition;

		/// <summary>
		/// The scale vector of the HUD sprite.
		/// </summary>
		private Vector2 hudSpriteScale;

		private ParticleEngine _engineSmoke1, _engineSmoke2, _engineFire, _laser1, _laser2,
			_smokeLeft1, _smokeLeft2, _smokeRight, _fire1, _fire2, _fireRight;

		/// <summary>
		/// Creates the demo game state with a given GameStateManager.
		/// </summary>
		/// <param name="parent">The GameStateManager that is storing this game state.</param>
		public DemoGameState(GameStateManager parent)
			: base(parent)
		{
			input = parent.getInput();
			_hudItems = new SortedDictionary<string, Sprite>();
			_hudMsgItems = new SortedDictionary<string, Message>();
			objectManager = new ObjectManager();
		}

		/// <summary>
		/// Loads all the assets for this state.
		/// </summary>
		public override void load()
		{
            Level level = GraphicsCore.getSingleton.Content.Load<Level>("assets/levels/space");
            level.load(GAME_STATE_NAME, objectManager);

            Layer mainLayer = SceneManager.getSingleton.getActiveScene().getLayer(5);

			// Create a new layer that is behind the hull tiles
			Layer newLayer = new Layer(1.0f);
			SceneManager.getSingleton.getActiveScene().AddLayer(2, newLayer);

			// Create layer for behind scene particles
			Layer backLayer = new Layer(1.0f);
			SceneManager.getSingleton.getActiveScene().AddLayer(4, backLayer);

			// Create layer for behind scene particles
			Layer frontLayer = new Layer(1.0f);
			SceneManager.getSingleton.getActiveScene().AddLayer(8, frontLayer);

            // Load the players and their weapons
			playerManager = new PlayerManager(input, mainLayer, objectManager);
			ProjectileManager.getSingleton.Players = playerManager.Players;

			// Start sound
			AudioManager.getSingleton.initialize();
			AudioManager.getSingleton.addMusic("music");
			AudioManager.getSingleton.playMusic("music");
			AudioManager.getSingleton.addSoundFX("explosion");
			AudioManager.getSingleton.addSoundFX("pistolShot");
			AudioManager.getSingleton.addSoundFX("rocketShot");
			AudioManager.getSingleton.addSoundFX("alarm");
			AudioManager.getSingleton.addSoundFX("laser");
			AudioManager.getSingleton.addSoundFX("ricochet");

			// Load HUD
			loadHUD();

            // Load the Hull
            loadHull();

			// Load the crosshair
			if (ENABLE_MOUSE_CURSOR)
			{
				mouseCursor = SpriteManager.getSingleton.load("mouseCursor", "assets/sprites/crosshair");
				mainLayer.AddSprite("mouseCursor", mouseCursor);
			}

			// Hardcoded level particle effect additions
			Vector2 tempVec;
			List<Vector2> tempList = new List<Vector2>();

			//Load the particle affects that will appear throughout the level.
			_engineSmoke1 = ParticleManager.getSingleton.load("smoke", "assets/particles/smokeEffect");
			backLayer.AddParticle("engine1", _engineSmoke1);
			tempVec = new Vector2(470, 705);
			tempList.Clear();
			tempList.Add(new Vector2(655, 520));
			foreach (SplineEmitter sEmitter in _engineSmoke1.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}

			_engineSmoke2 = ParticleManager.getSingleton.load("smoke", "assets/particles/engSmokeEffect");
			backLayer.AddParticle("engine2", _engineSmoke2);
			tempVec = new Vector2(685, 550);
			tempList.Clear();
			tempList.Add(new Vector2(435, 540));
			foreach (SplineEmitter sEmitter in _engineSmoke2.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}

			_engineFire = ParticleManager.getSingleton.load("fire", "assets/particles/fireEffect");
			backLayer.AddParticle("engineFire", _engineFire);
			tempVec = new Vector2(685, 550);
			tempList.Clear();
			tempList.Add(new Vector2(435, 540));
			foreach (SplineEmitter sEmitter in _engineFire.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}

			_laser1 = ParticleManager.getSingleton.load("laser", "assets/particles/levelLaserEffect");
			backLayer.AddParticle("laser1", _laser1);
			tempVec = new Vector2(562, 1640);
			tempList.Clear();
			tempList.Add(new Vector2(764, 1762));
			foreach (SplineEmitter sEmitter in _laser1.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}
			LevelLaser levelLaser1 = new LevelLaser("level_laser1", playerManager, objectManager);
			levelLaser1.Entity.Sprite.Visible = false;
			mainLayer.AddSprite(levelLaser1.Entity.Sprite.Name, levelLaser1.Entity.Sprite);

			_laser2 = ParticleManager.getSingleton.load("laser", "assets/particles/levelLaserEffect");
			backLayer.AddParticle("laser2", _laser2);
			tempVec = new Vector2(1285, 1772);
			tempList.Clear();
			tempList.Add(new Vector2(1581, 1765));
			foreach (SplineEmitter sEmitter in _laser2.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}
			LevelLaser levelLaser2 = new LevelLaser("level_laser2", playerManager, objectManager);
			levelLaser2.Entity.Sprite.Visible = false;
			mainLayer.AddSprite(levelLaser2.Entity.Sprite.Name, levelLaser2.Entity.Sprite);

			_smokeLeft1 = ParticleManager.getSingleton.load("smoke", "assets/particles/smokeEffect");
			backLayer.AddParticle("smokeLeft1", _smokeLeft1);
			tempVec = new Vector2(550, 1557);
			tempList.Clear();
			tempList.Add(new Vector2(311, 1557));
			foreach (SplineEmitter sEmitter in _smokeLeft1.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}

			_smokeLeft2 = ParticleManager.getSingleton.load("smoke", "assets/particles/smokeEffect");
			backLayer.AddParticle("smokeLeft2", _smokeLeft2);
			tempVec = new Vector2(1300, 1820);
			tempList.Clear();
			tempList.Add(new Vector2(820, 1820));
			foreach (SplineEmitter sEmitter in _smokeLeft2.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}

			//_fire1 = ParticleManager.getSingleton.load("fire", "assets/particles/fireEffect");
			//backLayer.AddParticle("fire1", _fire1);
			//tempVec = new Vector2(620, 1770);
			//tempList.Clear();
			//tempList.Add(new Vector2(660, 1800));
			//foreach (SplineEmitter sEmitter in _fire1.Emitters)
			//{
			//    sEmitter.EmitterPosition = tempVec;
			//    sEmitter.EmitterOffsets = tempList;
			//    sEmitter.IsActive = true;
			//}

			_fire2 = ParticleManager.getSingleton.load("fire", "assets/particles/fireEffect");
			backLayer.AddParticle("fire2", _fire2);
			tempVec = new Vector2(1600, 1820);
			tempList.Clear();
			tempList.Add(new Vector2(1720, 1780));
			foreach (SplineEmitter sEmitter in _fire2.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}

			_smokeRight = ParticleManager.getSingleton.load("smoke", "assets/particles/smokeEffect");
			backLayer.AddParticle("smokeRight", _smokeRight);
			tempVec = new Vector2(2800, 1250);
			tempList.Clear();
			tempList.Add(new Vector2(2700, 1250));
			foreach (SplineEmitter sEmitter in _smokeRight.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}

			_fireRight = ParticleManager.getSingleton.load("fire", "assets/particles/fireEffect");
			backLayer.AddParticle("fireRight", _fireRight);
			tempVec = new Vector2(2800, 1250);
			tempList.Clear();
			tempList.Add(new Vector2(2700, 1250));
			foreach (SplineEmitter sEmitter in _fireRight.Emitters)
			{
				sEmitter.EmitterPosition = tempVec;
				sEmitter.EmitterOffsets = tempList;
				sEmitter.IsActive = true;
			}

			SimpleObject lavaLamp = objectManager.getSimple("lava_lamp_1");
			if (lavaLamp != null)
			{
				Light lavaLight = new Light();
				lavaLight.Color = new Vector4(1,1,1,1);
				lavaLight.IsAttenuated = true;
				lavaLight.Attenuation = new Vector3(1,0,0);

				mainLayer.AddLight(lavaLight);
				lavaLamp.Light = lavaLight;
				lavaLight.Position = new Vector3(lavaLamp.SimpleEntity.PhysicsBody.Body.Position, 10);
			}

			lavaLamp = objectManager.getSimple("lava_lamp_2");
			if (lavaLamp != null)
			{
				Light lavaLight = new Light();
				lavaLight.Color = new Vector4(1, 1, 1, 1);
				lavaLight.IsAttenuated = true;
				lavaLight.Attenuation = new Vector3(0, .001f, .0001f);

				mainLayer.AddLight(lavaLight);
				lavaLamp.Light = lavaLight;
				lavaLight.Position = new Vector3(lavaLamp.SimpleEntity.PhysicsBody.Body.Position, 10);
			}
			

			//Scale background stars to cover the entire background
			Sprite stars = SpriteManager.getSingleton.get("background_stars");
			stars.Scale = new Vector2(4.0f, 4.0f);
        }

		/// <summary>
		/// Unloads all the assets for this state. Usually called when shutdown is called.
		/// </summary>
		public override void unload()
		{

		}

		/// <summary>
		/// Tells the engine to render this game state
		/// </summary>
		public override void run()
		{
			GraphicsCore.getSingleton.present();
		}

		/// <summary>
		/// What this state does when it becomes the main state.
		/// </summary>
		public override void enter()
		{
			SceneManager.getSingleton.setActiveScene(GAME_STATE_NAME);

			foreach (Sprite sprite in _hudItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _hudMsgItems.Values)
				msg.visible = true;

			_pause = false;
		}

		/// <summary>
		/// What this state does when it exits.
		/// </summary>
		public override void exit()
		{
			_pause = true;

			foreach (Sprite sprite in _hudItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _hudMsgItems.Values)
				msg.visible = false;
		}

		/// <summary>
		/// Pauses the game state.
		/// </summary>
		/// <returns>
		/// True, if the game is paused. False, if it is running.
		/// </returns>
		public override bool pause()
		{
			_pause = true;

			foreach (Sprite sprite in _hudItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _hudMsgItems.Values)
				msg.visible = false;

			return true;
		}

		/// <summary>
		/// Resumes the game state.
		/// </summary>
		public override void resume()
		{
			_pause = false;

			foreach (Sprite sprite in _hudItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _hudMsgItems.Values)
				msg.visible = true;
		}

		/// <summary>
		/// Updates the game state and parts of the engine
		/// that is important to the state. Does state change here.
		/// </summary>
		/// <param name="gameTime">The current values for time.</param>
		public override void update(GameTime gameTime)
		{
			base.update(gameTime);

            // Update hull tiles
            HullManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);		

			// Update and handle player manager
			playerManager.update(gameTime.ElapsedGameTime.Milliseconds * .001f);
			playerManager.handle();

			// Update HUD info
			updateHUDInfo();

			// Update GUI
			GuiManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);

			// Rotates the background
			background = SpriteManager.getSingleton.get("background");
			background.Rotation -= 0.0001f;
			if (background.Rotation <= -360)
			{
				background.Rotation += 360;
			}

			// Checks if a player has won
			foreach (Player player in playerManager.Players.Values)
			{
				if (player.Kills >= MAX_KILL_COUNT)
				{
					// Player has won the game!
					WinGameState win = (WinGameState)_parent.getGameState(WinGameState.GAME_STATE_NAME);
					win.setWinningPlayerInfo(player);
					_parent.pushGameState(_parent.getGameState(WinGameState.GAME_STATE_NAME));
				}
			}

			// Update the crosshairs position to that of the mousecursor
			if (ENABLE_MOUSE_CURSOR)
			{
				mouseCursor.Position = input.current[(int)_parent.PlayerControllingMenu].mousePosition;
			}

			// Check input
			if (Keyboard.GetState().IsKeyDown(Keys.F11))
			{
				GraphicsCore.getSingleton.Graphics.ToggleFullScreen();
			}
			if (Keyboard.GetState().IsKeyDown(Keys.Escape))
			{
				Console.Out.WriteLine(GAME_STATE_NAME + " escape");
				_parent.popGameState();
			}

			if (input.current[(int)_parent.PlayerControllingMenu].pause == ButtonState.Pressed &&
				input.previous[(int)_parent.PlayerControllingMenu].pause == ButtonState.Released)
			{
				//Switch to Pause Game State
				_parent.pushGameState(_parent.getGameState(PauseGameState.GAME_STATE_NAME));
			}

			AudioManager.getSingleton.update();
            ExplosionManager.getSingleton.update(gameTime.ElapsedRealTime.Milliseconds);
			objectManager.update(gameTime.ElapsedRealTime.Milliseconds);
		}

		/// <summary>
		/// Ends the program.
		/// </summary>
		public override void shutdown()
		{

		}

		/// <summary>
		/// Loads the HUD for the Game
		/// </summary>
		private void loadHUD()
		{
			Sprite sprite;
			Message msg;

			foreach (Player player in playerManager.Players.Values)
			{
				int i = (int)player.PlayerNumber;

				//Add the player's avatar
				sprite = GuiManager.getSingleton.load("player" + (i + 1) + "avatar", "assets/sprites/playeravatar");
				sprite.Position = new Vector2(i * 200 + 60, 35);
				switch (i + 1)
				{
					case 1:
						sprite.Color = PlayerManager.PLAYER_ONE_COLOR;
						break;
					case 2:
						sprite.Color = PlayerManager.PLAYER_TWO_COLOR;
						break;
					case 3:
						sprite.Color = PlayerManager.PLAYER_THREE_COLOR;
						break;
					case 4:
						sprite.Color = PlayerManager.PLAYER_FOUR_COLOR;
						break;
				}
				sprite.pause();
				_hudItems.Add("player" + (i + 1) + "avatar", sprite);

				// Set the initial Healthbar HUD and Cooldown HUD Positions for each player
				_initHealthbarHUDPositions[i] = new Vector2(i * 200 + 128, 13);
				_initCooldownHUDPositions[i] = new Vector2(i * 200 + 128, 45);

				// Draw positive health bar
				sprite = GuiManager.getSingleton.load("player" + (i+1) + "poshealth", "assets/sprites/healthbar");
				sprite.Position = new Vector2(i * 200 + 128, 13);
				sprite.pause();
				_hudItems.Add("player" + (i+1) + "poshealth", sprite);

				// Draw negative health bar
				sprite = GuiManager.getSingleton.load("player" + (i+1) + "neghealth", "assets/sprites/neghealthbar");
				sprite.Position = new Vector2(i * 200 + 128, 13);
				sprite.pause();
				_hudItems.Add("player" + (i+1) + "neghealth", sprite);

				// Draw cooldown bar
                sprite = GuiManager.getSingleton.load("player" + (i+1) + "kdoutline", "assets/sprites/cdoutline");
				sprite.Position = new Vector2(i * 200 + 128, 29);
				sprite.pause();
				_hudItems.Add("player" + (i+1) + "kdoutline", sprite);

				// Add "K:" and "D:"
                msg = new Message("K:", new Vector2(i * 200 + 80, 20), Color.Orange, "ug", "player" + (i+1) + "k");
                GuiManager.getSingleton.load("player" + (i+1) + "k", msg);
                _hudMsgItems.Add("player" + (i+1) + "k", msg);
                msg = new Message("D:", new Vector2(i * 200 + 130, 20), Color.Orange, "ug", "player" + (i+1) + "d");
                GuiManager.getSingleton.load("player" + (i+1) + "d", msg);
				
				// Add number of kills and number of deaths
                _hudMsgItems.Add("player" + (i+1) + "d", msg);
				msg = new Message("0", new Vector2(i * 200 + 100, 19), Color.Orange, "ug", "player" + (i+1) + "kills");
				GuiManager.getSingleton.load("player" + (i+1) + "kills", msg);
				_hudMsgItems.Add("player" + (i+1) + "kills", msg);
				msg = new Message("0", new Vector2(i * 200 + 150, 19), Color.Orange, "ug", "player" + (i+1) + "deaths");
				GuiManager.getSingleton.load("player" + (i+1) + "deaths", msg);
				_hudMsgItems.Add("player" + (i+1) + "deaths", msg);

				//Add player names above the Avatars in ug font with orange color
				msg = new Message("P" + (i + 1), new Vector2(i * 200 + 50, 1), Color.Orange, "ug", "player" + (i + 1) + "HUDname");
				GuiManager.getSingleton.load("player" + i + "HUDname", msg);
				switch (i + 1)
				{
					case 1:
						msg.color = Color.Red;
						break;
					case 2:
						msg.color = Color.Blue;
						break;
					case 3:
						msg.color = Color.Green;
						break;
					case 4:
						msg.color = Color.Yellow;
						break;
				}
				_hudMsgItems.Add("player" + (i + 1) + "HUDname", msg);
              
				// Add Cooldown bars for weapons
				sprite = GuiManager.getSingleton.load("player" + (i+1) + "cdOutline", "assets/sprites/cdoutline");
				sprite.Position = new Vector2(i * 200 + 128, 45);
				sprite.Scale = new Vector2(1.0f, 0.5f);
				sprite.pause();
				_hudItems.Add("player" + (i+1) + "cdOutline", sprite);

				sprite = GuiManager.getSingleton.load("player" + (i+1) + "weaponCD", "assets/sprites/neghealthbar");
				sprite.Position = new Vector2(i * 200 + 128, 45);
				sprite.pause();
				_hudItems.Add("player" + (i+1) + "weaponCD", sprite);
			}
		}

		/// <summary>
		/// Updates the HUD info for each player
		/// </summary>
		private void updateHUDInfo()
		{
			foreach (Player player in playerManager.Players.Values)
			{
				int playerNum = (int)player.PlayerNumber + 1;
				Sprite sprite = null;
				Message msg = null;

				if (_hudItems.ContainsKey("player" + playerNum + "poshealth"))
				{
					sprite = _hudItems["player" + playerNum + "poshealth"];
					//sprite.setAnimation(player.HealthPoints.ToString());
					float scale = (player.HealthPoints / (float)Player.MAX_HEALTH);
					float scalePos = (1 - scale) * (sprite.Width / 2);
					
					hudSpritePosition.X = _initHealthbarHUDPositions[playerNum - 1].X - scalePos;
					hudSpritePosition.Y = _initHealthbarHUDPositions[playerNum - 1].Y;
					sprite.Position = hudSpritePosition;

					hudSpriteScale.X = scale;
					hudSpriteScale.Y = 1.0f;
					sprite.Scale = hudSpriteScale;
				}

				if (_hudItems.ContainsKey("player" + playerNum + "weaponCD"))
				{
					sprite = _hudItems["player" + playerNum + "weaponCD"];
					float scale = (player.PlayerControl.cooldown / player.CurrentWeapon.cooldown);
					float scalePos = (1 - scale) * (sprite.Width / 2);

					hudSpritePosition.X = _initCooldownHUDPositions[playerNum - 1].X - scalePos;
					hudSpritePosition.Y = _initCooldownHUDPositions[playerNum - 1].Y;
					sprite.Position = hudSpritePosition;

					hudSpriteScale.X = scale;
					hudSpriteScale.Y = 0.6f;
					sprite.Scale = hudSpriteScale;
				}

				if (_hudMsgItems.ContainsKey("player" + playerNum + "kills"))
				{
					msg = _hudMsgItems["player" + playerNum + "kills"];
					msg.text = player.Kills.ToString();
				}

				if (_hudMsgItems.ContainsKey("player" + playerNum + "deaths"))
				{
					msg = _hudMsgItems["player" + playerNum + "deaths"];
					msg.text = player.Deaths.ToString();
				}
			}
		}

		/// <summary>
		/// Loads the hull onto the scene layer.
		/// </summary>
        private void loadHull()
        {
            Layer hullLayer = SceneManager.getSingleton.getActiveScene().getLayer(3);

            // Starting position is (327,150)

            Hull hull;
            int numHulls = 70;
            for (int k = 1; k <= numHulls; ++k)
            {
				if (k != 51 && k != 61 && k != 62 && k != 65 && k != 70)
				{
					SimpleEntity entity = EntityManager.getSingleton.getSimple("hulltile" + k);
					hull = new Hull("hulltile" + k, entity);

					hullLayer.AddSprite(hull.Name, hull.Entity.Sprite);
					HullManager.getSingleton.Add(hull);
				}
            }
        }
	}
}
