/*
 *  Author: Josh Wilson
 *  Copyright 2007. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using RhodesProcessorLib;
using SedohrProcessorLib;
using rhodes.graphics;
using rhodes.engine;
using rhodes.physics;
using rhodes.audio;
using rhodes.input;

using sedohr.player;
using sedohr.weapon;
using sedohr.objects;

namespace sedohr
{
	/// <summary>
	/// Stores information about the josh "screen" being displayed in the program. 
	/// </summary>
	class JoshGameState : GameState
	{
		/// <summary>
		/// Contains the name of the game state
		/// </summary>
		public readonly static string GAME_STATE_NAME = "josh";

		/// <summary>
		/// Contains the number of kills it takes to win the game
		/// </summary>
		public readonly static int WIN_COUNT = 10;

		/// <summary>
		/// Contains the PlayerManager that controls all the players in the game state
		/// </summary>
		PlayerManager playerManager;

		/// <summary>
		/// Contains the mouse cursor sprite
		/// </summary>
		Sprite mouseCursor;

		/// <summary>
		/// Contains the background sprite
		/// </summary>
		Sprite background;

		/// <summary>
		/// Contains the input used by the player
		/// </summary>
		Input _input;

		/// <summary>
		/// Contains the light in the scene
		/// </summary>
		Light light1;

		/// <summary>
		/// Contains the flag on whether the camera is in full view mode or zoomed mode
		/// </summary>
        Boolean cameraFixed = true;

		ObjectManager objectmanager;
        /// <summary>
        /// Contains a list of all sprite items in the HUD
        /// </summary>
        private SortedDictionary<String, Sprite> _hudItems;

        /// <summary>
        /// Contains a list of all message items in the hud
        /// </summary>
        private SortedDictionary<String, Message> _hudMsgItems;

        /// <summary>
        /// Contains a list of the initial Hud Positions of the healthbars
        /// </summary>
        private List<Vector2> _initHealthbarHUDPositions;

        /// <summary>
        /// Contains a list of the initial HUD Positions of the cooldown bars
        /// </summary>
        private List<Vector2> _initCDHUDPositions;

        /// <summary>
        /// The Black Hole.
        /// </summary>
		//private Hole _hole;

        private SortedDictionary<String, Hull> _hullTiles;
        public SortedDictionary<String, Hull> HullTile
        {
            get { return _hullTiles; }
            set { _hullTiles = value; }
        }

		private HullManager _hull;

		/// <summary>
		/// Creates the josh game state with a given GameStateManager.
		/// </summary>
		/// <param name="parent">The GameStateManager that is storing this game state.</param>
		public JoshGameState(GameStateManager parent)
			: base(parent)
		{
			_input = parent.getInput();
            _hudItems = new SortedDictionary<string, Sprite>();
            _hudMsgItems = new SortedDictionary<string, Message>();
            _initHealthbarHUDPositions = new List<Vector2>();
            _initCDHUDPositions = new List<Vector2>();
			objectmanager = new ObjectManager();
			_hull = new HullManager();
		}

		/// <summary>
		/// Loads all the assets for this state.
		/// </summary>
		public override void load()
		{
			Level level = GraphicsCore.getSingleton.Content.Load<Level>("assets/levels/space");
			level.load(GAME_STATE_NAME, objectmanager);

			Layer mainLayer = SceneManager.getSingleton.getActiveScene().getLayer(5);

			//Create a new layer that is behind the hull tiles
			Layer newLayer = new Layer(1.0f); //0.5
			SceneManager.getSingleton.getActiveScene().AddLayer(2, newLayer);

			//Load the players and their weapons
            playerManager = new PlayerManager(_input, mainLayer, objectmanager);
			ProjectileManager.getSingleton.Players = playerManager.Players;

            //Creates the default healthbar and cooldownbar positions for each player
            for (int i = 1; i <= playerManager.Players.Count; i++)
            {
                _initHealthbarHUDPositions.Add(Vector2.Zero);
                _initCDHUDPositions.Add(Vector2.Zero);
            }


            light1 = new Light();
            light1.Position = new Vector3(0, 300, 10);
            light1.IsAttenuated = false;
            mainLayer.AddLight(light1);

            //Load HUD
            loadHUD();

			//Load the crosshair
			mouseCursor = SpriteManager.getSingleton.load("mouseCursor", "assets/sprites/crosshair");
			mainLayer.AddSprite("mouseCursor", mouseCursor);

			//Sets the Camera to Full Level View
			for (int i = 1; i <= playerManager.Players.Count; i++)
			{
				//Unregisters a player from the camera
				Camera2D.getSingleton.unregister("player" + i);
			}
			Camera2D.getSingleton.Position = new Vector2(0, 0);
			Camera2D.getSingleton.Zoom = 0.25f;

            //Scale the Background
            background = SpriteManager.getSingleton.get("background");
            background.Scale = new Vector2(3.0f, 3.0f);
            Console.Out.WriteLine(background.Position);
            background.Position = new Vector2(1500.0f, 1000.0f);

			//Create the Black Hole
			//_hole = new Hole("hole", playerManager, objectmanager);
			//mainLayer.AddSprite(_hole.Entity.Sprite.Name, _hole.Entity.Sprite);

			//Load the Hull
			loadHull();
			
		}

		/// <summary>
		/// Unloads all the assets for this state. Usually called when shutdown is called.
		/// </summary>
		public override void unload()
		{

		}

		/// <summary>
		/// Tells the engine to render this game state
		/// </summary>
		public override void run()
		{
			GraphicsCore.getSingleton.present();
		}

		/// <summary>
		/// What this state does when it becomes the main state.
		/// </summary>
		public override void enter()
		{
			SceneManager.getSingleton.setActiveScene(GAME_STATE_NAME);

            foreach (Sprite sprite in _hudItems.Values)
            {
                sprite.Visible = true;
            }

			foreach (Message msg in _hudMsgItems.Values)
				msg.visible = true;

			_pause = false;
		}

		/// <summary>
		/// What this state does when it exits.
		/// </summary>
		public override void exit()
		{
			//unload();

			_pause = true;

			foreach (Sprite sprite in _hudItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _hudMsgItems.Values)
				msg.visible = false;
		}

		/// <summary>
		/// Pauses the game state.
		/// </summary>
		/// <returns>
		/// True, if the game is paused. False, if it is running.
		/// </returns>
		public override bool pause()
		{
			_pause = true;

			foreach (Sprite sprite in _hudItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _hudMsgItems.Values)
				msg.visible = false;

			return true;
		}

		/// <summary>
		/// Resumes the game state.
		/// </summary>
		public override void resume()
		{
			_pause = false;

			foreach (Sprite sprite in _hudItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _hudMsgItems.Values)
				msg.visible = true;
		}

		/// <summary>
		/// Updates the game state. Updates parts of the engine that's important to the states. Does state change here.
		/// </summary>
		/// <param name="gameTime">The current values for time.</param>
		public override void update(GameTime gameTime)
		{
			if (cameraFixed)
			{
				for (int i = 1; i <= playerManager.Players.Count; i++)
				{
					Camera2D.getSingleton.unregister("player" + i);
				}
			}

            //Update hulltiles
			HullManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);
			//_hull.update(gameTime.ElapsedGameTime.Milliseconds * .001f);

			base.update(gameTime);

			//Update Hole
			//_hole.update(gameTime.ElapsedGameTime.Milliseconds * .001f);

			playerManager.update(gameTime.ElapsedGameTime.Milliseconds * .001f);
            playerManager.handle();

			// update HUD info
			updateHUDInfo();

            GuiManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);

			//Rotates the background
			background = SpriteManager.getSingleton.get("background");
			background.Rotation -= 0.0003f; //.0001f
			if (background.Rotation <= -360)
			{
				background.Rotation += 360;
			}

            //Tests if a player has won
			foreach (Player player in playerManager.Players.Values)
			{
				if (player.Kills >= WIN_COUNT)
				{
					//Player has won the game!!!
					WinGameState win = (WinGameState)_parent.getGameState(WinGameState.GAME_STATE_NAME);
					win.setWinningPlayerInfo(player);
					_parent.pushGameState(_parent.getGameState(WinGameState.GAME_STATE_NAME));
				}
			}

			//Updates the crosshairs position
			mouseCursor.Position = _input.current[(int)PlayerIndex.One].mousePosition;
			light1.Position = new Vector3(mouseCursor.Position.X, mouseCursor.Position.Y, 10);
			//Console.Out.WriteLine(mouseCursor.Position);

			/* Check input */
			if (Keyboard.GetState().IsKeyDown(Keys.F11))
			{
				GraphicsCore.getSingleton.Graphics.ToggleFullScreen();
			}

			if (_input.current[(int)PlayerIndex.One].pause == ButtonState.Pressed &&
				_input.previous[(int)PlayerIndex.One].pause == ButtonState.Released)
			{
                //Switch to Pause Game State
                //AudioManager.getSingleton.pauseMusic("spaceTech");
                _parent.pushGameState(_parent.getGameState(PauseGameState.GAME_STATE_NAME));
			}

			if (Keyboard.GetState().IsKeyDown(Keys.Escape))
			{
				Console.Out.WriteLine(GAME_STATE_NAME + " escape");
				_parent.popGameState();
			}

			//Toggles full level view and zoomed in view
            if (Keyboard.GetState().IsKeyDown(Keys.Tab))
            {
                if (cameraFixed)
                {
					for (int i = 1; i <= playerManager.Players.Count; i++)
					{
						Camera2D.getSingleton.register("player" + i);
					}
                    cameraFixed = false;
                }
                else
                {
					for (int i = 1; i <= playerManager.Players.Count; i++)
					{
						Camera2D.getSingleton.unregister("player" + i);
					}
                    Camera2D.getSingleton.Position = new Vector2(400, 0);
                    Camera2D.getSingleton.Zoom = 0.4f;
                    cameraFixed = true;
                }
			}

			ExplosionManager.getSingleton.update(gameTime.ElapsedRealTime.Milliseconds);

			#region test region
			//if (Keyboard.GetState().IsKeyDown(Keys.D))
			//{
			//    //Kill player
			//    playerManager.Players["player1"].changeState("Dead");
			//}

			////***Start Healthbar testing
			//if (Keyboard.GetState().IsKeyDown(Keys.D1))
			//{
			//    //Update Player1 health
			//    playerManager.Players["player1"].takeDamage(1);
			//    //updateHealthHUD(1);

			//    //playerManager.Players["player1"].Kills += 1;
			//    //updateKillsHUD(1);
			//}

			//if (Keyboard.GetState().IsKeyDown(Keys.D2))
			//{
			//    //Update Player1 health
			//    playerManager.Players["player2"].HealthPoints -= 1;
			//    //updateHealthHUD(2);

			//    playerManager.Players["player2"].Kills += 1;
			//    //updateKillsHUD(2);
			//}

			//if (Keyboard.GetState().IsKeyDown(Keys.D3))
			//{
			//    //Update Player1 health
			//    playerManager.Players["player3"].HealthPoints -= 1;
			//    //updateHealthHUD(3);

			//    playerManager.Players["player3"].Kills += 1;
			//    //updateKillsHUD(3);
			//}

			//if (Keyboard.GetState().IsKeyDown(Keys.D4))
			//{
			//    //Update Player1 health
			//    playerManager.Players["player4"].HealthPoints -= 1;
			//    //updateHealthHUD(4);

			//    playerManager.Players["player4"].Kills += 1;
			//    //updateKillsHUD(4);
			//}

			//if (Keyboard.GetState().IsKeyDown(Keys.D5))
			//{
			//    //Update Player1 health
			//    playerManager.Players["player1"].HealthPoints += 1;
			//    //updateHealthHUD(1);
			//}

			//if (Keyboard.GetState().IsKeyDown(Keys.D6))
			//{
			//    //Update Player1 health
			//    playerManager.Players["player2"].HealthPoints += 1;
			//    //updateHealthHUD(2);
			//}

			//if (Keyboard.GetState().IsKeyDown(Keys.D7))
			//{
			//    //Update Player1 health
			//    playerManager.Players["player3"].HealthPoints += 1;
			//    //updateHealthHUD(3);
			//}

			//if (Keyboard.GetState().IsKeyDown(Keys.D8))
			//{
			//    //Update Player1 health
			//    playerManager.Players["player4"].HealthPoints += 1;
			//    //updateHealthHUD(4);
			//}
            //***End Healthbar testing

			//if (Keyboard.GetState().IsKeyDown(Keys.D0))
			//{
			//    _parent.pushGameState(_parent.getGameState(WinGameState.GAME_STATE_NAME));
			//}
#endregion
		}

		/// <summary>
		/// Ends the program.
		/// </summary>
		public override void shutdown()
		{

		}

        /// <summary>
        /// Loads the HUD for the Game
        /// </summary>
        private void loadHUD()
        {
            Sprite sprite;
            Message msg;
            
            for (int i = 1; i <= playerManager.Players.Count; i++)
            {
				//NOTE*** Create one "avatar" to contain the player avatar image box, weapon image box, healthbar box, and kill/death box
				//sprite = GuiManager.getSingleton.load("player" + i + "avatar", "assets/sprites/player" + i + "avatar");
				//sprite.Position = new Vector2((i - 1) * 200 + 60, 35);
				//sprite.pause();
				//_hudItems.Add("player" + i + "avatar", sprite);

				sprite = GuiManager.getSingleton.load("player" + i + "avatar", "assets/sprites/playeravatar");
				sprite.Position = new Vector2((i - 1) * 200 + 60, 35);
				switch (i)
				{
					case 1:
						sprite.Color = PlayerManager.PLAYER_ONE_COLOR;
						break;
					case 2:
						sprite.Color = PlayerManager.PLAYER_TWO_COLOR;
						break;
					case 3:
						sprite.Color = PlayerManager.PLAYER_THREE_COLOR;
						break;
					case 4:
						sprite.Color = PlayerManager.PLAYER_FOUR_COLOR;
						break;
				}

				sprite.pause();
				_hudItems.Add("player" + i + "avatar", sprite);

				//sprite = GuiManager.getSingleton.load("player" + i + "health", "assets/sprites/healthbartest");
				//sprite.Position = new Vector2((i - 1) * 200 + 128, 10);
				//sprite.pause();
				//_hudItems.Add("player" + i + "health", sprite);

                //Set the initial Healthbar HUD Positions for each player
                _initHealthbarHUDPositions[i - 1] = new Vector2((i - 1) * 200 + 128, 13);
                _initCDHUDPositions[i - 1] = new Vector2((i - 1) * 200 + 128, 45);

				sprite = GuiManager.getSingleton.load("player" + i + "poshealth", "assets/sprites/healthbar");
				sprite.Position = new Vector2((i - 1) * 200 + 128, 13);
				sprite.pause();
				_hudItems.Add("player" + i + "poshealth", sprite);

                sprite = GuiManager.getSingleton.load("player" + i + "neghealth", "assets/sprites/neghealthbar");
                sprite.Position = new Vector2((i - 1) * 200 + 128, 13);
                sprite.pause();
                _hudItems.Add("player" + i + "neghealth", sprite);

                sprite = GuiManager.getSingleton.load("player" + i + "kdoutline", "assets/sprites/cdoutline");//killdeathoutline
				sprite.Position = new Vector2((i - 1) * 200 + 128, 29);
				sprite.pause();
				_hudItems.Add("player" + i + "kdoutline", sprite);

				//sprite = GuiManager.getSingleton.load("player" + i + "gunavatar", "assets/sprites/gunavatar");
				//sprite.Position = new Vector2((i - 1) * 200 + 30, 35);
				//sprite.pause();
				//_hudItems.Add("player" + i + "gunavatar", sprite);

                msg = new Message("K:", new Vector2((i - 1) * 200 + 80, 20), Color.Orange, "ug", "player" + i + "k");
                GuiManager.getSingleton.load("player" + i + "k", msg);
                _hudMsgItems.Add("player" + i + "k", msg);
                msg = new Message("D:", new Vector2((i - 1) * 200 + 130, 20), Color.Orange, "ug", "player" + i + "d");
                GuiManager.getSingleton.load("player" + i + "d", msg);
                _hudMsgItems.Add("player" + i + "d", msg);

				msg = new Message("0", new Vector2((i - 1) * 200 + 100, 19), Color.Orange, "ug", "player" + i + "kills");
                GuiManager.getSingleton.load("player" + i + "kills", msg);
                _hudMsgItems.Add("player" + i + "kills", msg);
				msg = new Message("0", new Vector2((i - 1) * 200 + 150, 19), Color.Orange, "ug", "player" + i + "deaths");
                GuiManager.getSingleton.load("player" + i + "deaths", msg);
                _hudMsgItems.Add("player" + i + "deaths", msg);

                //Add player names above the Avatars
                //sprite = GuiManager.getSingleton.load("player" + i + "HUDname", "assets/sprites/player" + i + "name");
                //sprite.Position = new Vector2((i - 1) * 200 + 60, 10);
                //sprite.pause();
                //_hudItems.Add("player" + i + "HUDname", sprite);
                //sprite.Scale = new Vector2(2.0f, 2.0f);

                msg = new Message("P" + i, new Vector2((i - 1) * 200 + 50, 1), Color.Orange, "ug", "player" + i + "HUDname");
                GuiManager.getSingleton.load("player" + i + "HUDname", msg);
                //msg.scale = 0.9f;
                _hudMsgItems.Add("player" + i + "HUDname", msg);

                //Add Cooldown bars for weapons
                sprite = GuiManager.getSingleton.load("player" + i + "cdOutline", "assets/sprites/cdoutline");
                sprite.Position = new Vector2((i - 1) * 200 + 128, 45);
                sprite.Scale = new Vector2(1.0f, 0.5f);
                sprite.pause();
                _hudItems.Add("player" + i + "cdOutline", sprite);

                sprite = GuiManager.getSingleton.load("player" + i + "weaponCD", "assets/sprites/neghealthbar");
                sprite.Position = new Vector2((i - 1) * 200 + 128, 45);
                sprite.pause();
                _hudItems.Add("player" + i + "weaponCD", sprite);
            }
        }

		/// <summary>
		/// Updates the HUD info for each player
		/// </summary>
        private void updateHUDInfo()
        {
            foreach (Player player in playerManager.Players.Values)
            {
                int playerNum = (int)player.PlayerNumber + 1;
                Sprite sprite = null;
                Message msg = null;

                if (_hudItems.ContainsKey("player" + playerNum + "poshealth"))
                {
                    sprite = _hudItems["player" + playerNum + "poshealth"];
                    //sprite.setAnimation(player.HealthPoints.ToString());
                    float scale = (player.HealthPoints / (float)Player.MAX_HEALTH);
                    float scalePos = (1 - scale) * (sprite.Width/2); //(100 - scale*100)/100
                    sprite.Position = new Vector2(_initHealthbarHUDPositions[playerNum - 1].X - scalePos, _initHealthbarHUDPositions[playerNum - 1].Y);
					//sprite.Scale = new Vector2((player.HealthPoints / 100), 1.0f);
                    //if (player.PlayerNumber == PlayerIndex.Four)
                    //{
                    //    Console.Out.WriteLine(player.HealthPoints);
                    //    Console.Out.WriteLine(scale);
                    //}
                    sprite.Scale = new Vector2(scale, 1.0f);
                }

                if (_hudItems.ContainsKey("player" + playerNum + "weaponCD"))
                {
                    sprite = _hudItems["player" + playerNum + "weaponCD"];
                    float scale = (player.PlayerControl.cooldown / player.CurrentWeapon.cooldown);
                    float scalePos = (1 - scale) * (sprite.Width / 2);
                    sprite.Position = new Vector2(_initCDHUDPositions[playerNum - 1].X - scalePos, _initCDHUDPositions[playerNum - 1].Y);
                    sprite.Scale = new Vector2(scale, 0.6f);
                }

                if (_hudMsgItems.ContainsKey("player" + playerNum + "kills"))
                {
                    msg = _hudMsgItems["player" + playerNum + "kills"];
                    msg.text = player.Kills.ToString();
                }

                if (_hudMsgItems.ContainsKey("player" + playerNum + "deaths"))
                {
                    msg = _hudMsgItems["player" + playerNum + "deaths"];
                    msg.text = player.Deaths.ToString();
                }
            }
        }

        /// <summary>
        /// Loads all the hull pieces for the background
        /// </summary>
		private void loadHull()
		{
			Layer hullLayer = SceneManager.getSingleton.getActiveScene().getLayer(3);
			
			//Starting pos 327,150

			Hull hull;
			int numHulls = 70;
			for (int k = 1; k <= numHulls; ++k)
			{
				if (k != 51 && k != 61 && k != 62 && k != 65 && k != 70)
				{
					SimpleEntity entity = EntityManager.getSingleton.getSimple("hulltile" + k);
					hull = new Hull("hulltile" + k, entity);
					//hull.Entity = EntityManager.getSingleton.getSimple("hulltile" + k);

					hullLayer.AddSprite(hull.Name, hull.Entity.Sprite);
					HullManager.getSingleton.Add(hull);
				}
			}
		}
	}
}
