/*
 *  Author: Josh Wilson
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using rhodes.graphics;
using rhodes.input;

namespace sedohr
{
	/// <summary>
	/// State the game enters when a player pauses the game. Freezes all action in the game and shows a menu overlay
	///  on top of the game.
	/// </summary>
	class PauseGameState : GameState
	{
		/// <summary>
		/// Contains the name of the game state
		/// </summary>
		public readonly static string GAME_STATE_NAME = "pause";

		/// <summary>
		/// Contains a list of all sprite items in the HUD
		/// </summary>
		private SortedDictionary<String, Sprite> _pauseItems;

		/// <summary>
		/// Contains a list of all message items in the hud
		/// </summary>
		private SortedDictionary<String, Message> _pauseMsgItems;

		/// <summary>
		/// Contains a list of the menu options
		/// </summary>
		private List<Message> _menu;

		/// <summary>
		/// Menu option that is currently selected
		/// </summary>
		private int _menuSelection = 0;

		/// <summary>
		/// The input system for this state
		/// </summary>
		private Input _input;

        /// <summary>
        /// Constructor for the Pause Game State
        /// </summary>
        /// <param name="parent">Instance of the GameStateManager</param>
		public PauseGameState(GameStateManager parent)
			: base(parent)
		{
			_input = parent.getInput();
			_pauseItems = new SortedDictionary<string, Sprite>();
			_pauseMsgItems = new SortedDictionary<string, Message>();
			_menu = new List<Message>();
		}

        /// <summary>
        /// Loads the Pause Game State
        /// </summary>
		public override void load()
		{
			//Load the graphics for the Pause Game State
			Sprite sprite = GuiManager.getSingleton.load("pausebox", "assets/sprites/pause");
			sprite.pause();
			sprite.Position = new Vector2(400, 300);
			_pauseItems.Add("pausebox", sprite);

			Message pauseMsg = new Message("Game is Paused.", new Vector2(300, 100), Color.Red, "ug", "pause");
			GuiManager.getSingleton.load("pause", pauseMsg);
			_pauseMsgItems.Add("pause", pauseMsg);

			Message msg = new Message("Resume", new Vector2(160, 140), Color.Red, "ug", "pauseresume");
			GuiManager.getSingleton.load("pauseresume", msg);
			_menu.Add(msg);
			msg = new Message("Controls", new Vector2(160, 180), Color.Yellow, "ug", "pausehelp");
			GuiManager.getSingleton.load("pausehelp", msg);
			_menu.Add(msg);
			//msg = new Message("Main Menu", new Vector2(160, 220), Color.Yellow, "ug", "pauseexit");
			//GuiManager.getSingleton.load("pauseexit", msg);
			//_menu.Add(msg);

			pause();
		}

        /// <summary>
        /// Unloads the Pause Game State
        /// </summary>
		public override void unload()
		{

		}

        /// <summary>
        /// Renders the Pause Game State
        /// </summary>
		public override void run()
		{
			GraphicsCore.getSingleton.present();
		}

        /// <summary>
        /// Enters the Pause Game State
        /// </summary>
		public override void enter()
		{
			foreach (Sprite sprite in _pauseItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _menu)
				msg.visible = true;

			foreach (Message msg in _pauseMsgItems.Values)
				msg.visible = true;

			_pause = false;
		}

        /// <summary>
        /// Exits the Pause Game State
        /// </summary>
		public override void exit()
		{
			_pause = true;

			foreach (Sprite sprite in _pauseItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _menu)
				msg.visible = false;

			foreach (Message msg in _pauseMsgItems.Values)
				msg.visible = false;
		}

        /// <summary>
        /// Pauses the Pause Game State
        /// </summary>
        /// <returns></returns>
		public override bool pause()
		{
			_pause = true;

			foreach (Sprite sprite in _pauseItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _menu)
				msg.visible = false;

			foreach (Message msg in _pauseMsgItems.Values)
				msg.visible = false;

			return true;
		}

        /// <summary>
        /// Resumes the Pause Game State
        /// </summary>
		public override void resume()
		{
			_pause = false;

			foreach (Sprite sprite in _pauseItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _menu)
				msg.visible = true;

			foreach (Message msg in _pauseMsgItems.Values)
				msg.visible = true;
		}

        /// <summary>
        /// Updates the Pause Game State
        /// </summary>
        /// <param name="gameTime">Snapshot of the game timing state</param>
		public override void update(GameTime gameTime)
		{
			if (_pause)
				return;

			// update here
			GuiManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);

            //When the player presses up, changes to the appropriate menu selection
			if (_input.current[(int)_parent.PlayerControllingMenu].up == ButtonState.Pressed &&
				_input.previous[(int)_parent.PlayerControllingMenu].up == ButtonState.Released)
			{
				_menu[_menuSelection].color = Color.Yellow;

				_menuSelection--;

				if (_menuSelection < 0)
					_menuSelection = _menu.Count - 1;

				_menu[_menuSelection].color = Color.Red;
			}

            //When the player presses down, changes to the appropriate menu selection
			if (_input.current[(int)_parent.PlayerControllingMenu].down == ButtonState.Pressed &&
				_input.previous[(int)_parent.PlayerControllingMenu].down == ButtonState.Released)
			{
				_menu[_menuSelection].color = Color.Yellow;

				_menuSelection++;

				if (_menuSelection >= _menu.Count)
					_menuSelection = 0;

				_menu[_menuSelection].color = Color.Red;
			}

            //When the player presses enter(start on the controller), changes to the appropriate game state
			if (_input.current[(int)_parent.PlayerControllingMenu].pause == ButtonState.Pressed &&
				_input.previous[(int)_parent.PlayerControllingMenu].pause == ButtonState.Released)
			{
				switch (_menuSelection)
				{
					case 0:
                        //Resuems the Game
						_parent.popGameState();
						break;
					case 1:
                        //Goes to the Help Screen
						_parent.pushGameState(_parent.getGameState(HelpGameState.GAME_STATE_NAME));
						break;
					//case 2:
					//    //Exits to the Main Menu
					//    _parent.popGameState();
					//    _parent.popGameState();
					//    break;
				}
			}
		}

        /// <summary>
        /// Shuts down the Pause Game State
        /// </summary>
		public override void shutdown()
		{

		}
	}
}
