/*
 *  Author: David Huynh, Robert Yates, Josh Wilson
 *  Copyright 2007. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using rhodes.input;
using rhodes.graphics;
using rhodes.audio;

namespace sedohr
{
	/// <summary>
	/// Stores information about the title "screen" being displayed in the program. 
	/// </summary>
	class TitleGameState : GameState
	{
		/// <summary>
		/// Contains the name of the game state
		/// </summary>
		public readonly static string GAME_STATE_NAME = "title";

		/// <summary>
		/// Contains a list of all sprite items in the title
		/// </summary>
		private SortedDictionary<String, Sprite> _titleItems;

		/// <summary>
		/// Contains a list of all message items in the title
		/// </summary>
		private SortedDictionary<String, Message> _msgItems;

		/// <summary>
		/// Contains a list of the menu options
		/// </summary>
		private List<Message> _menu;

		/// <summary>
		/// Menu option that is currently selected
		/// </summary>
		private int _menuSelection = 0;

		/// <summary>
		/// The manager of this game state.
		/// </summary>
		GameStateManager _gameStateManager;

	    /// <summary>
	    /// The input component of this game state
	    /// </summary>
	    Input _input;


        /// <summary>
        /// Contains the background sprite of the title screen
        /// </summary>
        Sprite titleBackground;

		/// <summary>
		/// Creates the title game state with a given GameStateManager.
		/// </summary>
		/// <param name="parent">The GameStateManager that is storing this game state.</param>
		public TitleGameState(GameStateManager parent)
			: base(parent)
		{
			_gameStateManager = parent;
			_input = parent.getInput();
			_titleItems = new SortedDictionary<string, Sprite>();
			_msgItems = new SortedDictionary<string, Message>();
			_menu = new List<Message>();
		}

		/// <summary>
		/// Loads all the assets for this state.
		/// </summary>
		public override void load()
		{
            //Scale the Background
            titleBackground = GuiManager.getSingleton.load(GAME_STATE_NAME, "assets/sprites/title_background");
            titleBackground.pause();
            titleBackground.Position = new Vector2(400.0f, 300.0f);
            _titleItems.Add("title_background", titleBackground);

			//Create Title/Main menu screen
			FontManager.getSingleton.load("ug");
			Message msg = new Message("Play Game", new Vector2(140, 280), Color.Red, "ug", "play");
			GuiManager.getSingleton.load("play", msg);
			_menu.Add(msg);
			msg = new Message("How to Play", new Vector2(140, 320), Color.Yellow, "ug", "help");
			GuiManager.getSingleton.load("help", msg);
			_menu.Add(msg);
			msg = new Message("Credits", new Vector2(140, 360), Color.Yellow, "ug", "credits");
			GuiManager.getSingleton.load("credits", msg);
			_menu.Add(msg);
			msg = new Message("Exit", new Vector2(140, 400), Color.Yellow, "ug", "exit");
			GuiManager.getSingleton.load("exit", msg);
			_menu.Add(msg);


			AudioManager.getSingleton.addMusic("spaceTech");
			//AudioManager.getSingleton.playMusic("spaceTech");
		}

		/// <summary>
		/// Unloads all the assets for this state. Usually called when shutdown is called.
		/// </summary>
		public override void unload()
		{

		}

		/// <summary>
		/// Tells the engine to render this game state
		/// </summary>
		public override void run()
		{
			GraphicsCore.getSingleton.Graphics.GraphicsDevice.Clear(Color.TransparentBlack);
			GuiManager.getSingleton.render();
		}

		/// <summary>
		/// What this state does when it becomes the main state.
		/// </summary>
		public override void enter()
		{
			foreach (Sprite sprite in _titleItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _menu)
				msg.visible = true;

			//AudioManager.getSingleton.playMusic("spaceTech");
		}

		/// <summary>
		/// What this state does when it exits.
		/// </summary>
		public override void exit()
		{
			foreach (Sprite sprite in _titleItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _menu)
				msg.visible = false;
		}

		/// <summary>
		/// Pauses the game state.
		/// </summary>
		/// <returns>
		/// True, if the game is paused. False, if it is running.
		/// </returns>
		public override bool pause()
		{
			_pause = true;

			foreach (Sprite sprite in _titleItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _menu)
				msg.visible = false;

			return true;
		}

		/// <summary>
		/// Resumes the game state.
		/// </summary>
		public override void resume()
		{
			_pause = false;

			foreach (Sprite sprite in _titleItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _menu)
				msg.visible = true;
		}

		/// <summary>
		/// Updates the game state. Updates parts of the engine that's important to the states. Does state change here.
		/// </summary>
		/// <param name="gameTime">The current values for time.</param>
		public override void update(GameTime gameTime)
		{
			if (_pause)
				return;

			// update here
			GuiManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);

			// handle input
			if (Keyboard.GetState().IsKeyDown(Keys.D1))// || Keyboard.GetState().IsKeyDown(Keys.Enter))
			{
				//Creates a new game state, loads it, adds it to the game state manager and then pushes
				// it onto the stack
				GuiManager.getSingleton.removeAll();
				GameState demoGameState = new DemoGameState(_gameStateManager);
				demoGameState.load();
				_gameStateManager.addGameState(DemoGameState.GAME_STATE_NAME, demoGameState);
				_parent.pushGameState(_parent.getGameState(DemoGameState.GAME_STATE_NAME));
			}

			if (Keyboard.GetState().IsKeyDown(Keys.D2))
			{
				//Creates a new game state, loads it, adds it to the game state manager and then pushes
				// it onto the stack
				GuiManager.getSingleton.removeAll();
				GameState davidGameState = new DavidGameState(_gameStateManager);
				davidGameState.load();
				_gameStateManager.addGameState(DavidGameState.GAME_STATE_NAME, davidGameState);
				_parent.pushGameState(_parent.getGameState(DavidGameState.GAME_STATE_NAME));
			}

			if (Keyboard.GetState().IsKeyDown(Keys.D4))
			{
				//Creates a new game state, loads it, adds it to the game state manager and then pushes
				// it onto the stack
				//GuiManager.getSingleton.removeAll();
				GameState joshGameState = new JoshGameState(_gameStateManager);
				joshGameState.load();
				_gameStateManager.addGameState(JoshGameState.GAME_STATE_NAME, joshGameState);
				_parent.pushGameState(_parent.getGameState(JoshGameState.GAME_STATE_NAME));
			}

			if (Keyboard.GetState().IsKeyDown(Keys.D5))
			{
				//Creates a new game state, loads it, adds it to the game state manager and then pushes
				// it onto the stack
				GuiManager.getSingleton.removeAll();
				GameState anthonyGameState = new AnthonyGameState(_gameStateManager);
				anthonyGameState.load();
				_gameStateManager.addGameState(AnthonyGameState.GAME_STATE_NAME, anthonyGameState);
				_parent.pushGameState(_parent.getGameState(AnthonyGameState.GAME_STATE_NAME));
			}

            //When the player presses up, changes to the appropriate menu selection
			if (_input.current[(int)_parent.PlayerControllingMenu].up == ButtonState.Pressed &&
				_input.previous[(int)_parent.PlayerControllingMenu].up == ButtonState.Released)
			{
				_menu[_menuSelection].color = Color.Yellow;

				_menuSelection--;

				if (_menuSelection < 0)
					_menuSelection = _menu.Count - 1;

				_menu[_menuSelection].color = Color.Red;
			}

            //When the player presses down, changes to the appropriate menu selection
            if(_input.current[(int)_parent.PlayerControllingMenu].down == ButtonState.Pressed &&
                _input.previous[(int)_parent.PlayerControllingMenu].down == ButtonState.Released)
			{
				_menu[_menuSelection].color = Color.Yellow;

				_menuSelection++;

				if (_menuSelection >= _menu.Count)
					_menuSelection = 0;

				_menu[_menuSelection].color = Color.Red;
			}

            //When the player presses enter(start on the controller), changes to the appropriate game state
			if (_input.current[(int)_parent.PlayerControllingMenu].pause == ButtonState.Pressed &&
				_input.previous[(int)_parent.PlayerControllingMenu].pause == ButtonState.Released)
			{
				switch (_menuSelection)
				{
					case 0:
						//Go to Game
						foreach (Message msg in _menu)
							msg.visible = false;
						GameState demoGameState = new DemoGameState(_gameStateManager);
						demoGameState.load();
						_gameStateManager.addGameState(DemoGameState.GAME_STATE_NAME, demoGameState);
						_parent.pushGameState(_parent.getGameState(DemoGameState.GAME_STATE_NAME));
						break;
					case 1:
						//Go to Option menu
						foreach (Message msg in _menu)
							msg.visible = false;
						_parent.pushGameState(_parent.getGameState(HelpGameState.GAME_STATE_NAME));
						break;
					case 2:
						//Go to Credits menu
						foreach (Message msg in _menu)
							msg.visible = false;
						_parent.pushGameState(_parent.getGameState(CreditGameState.GAME_STATE_NAME));
						break;
					case 3:
						//Exits the game
						Console.Out.WriteLine(GAME_STATE_NAME + " escape");
						_parent.shutdown();
						break;
				}
			}

			if (Keyboard.GetState().IsKeyDown(Keys.F11))
			{
				//Toggles fullscreen mode
				GraphicsCore.getSingleton.Graphics.ToggleFullScreen();
			}

			if (Keyboard.GetState().IsKeyDown(Keys.Escape))
			{
				//Exits the game
				Console.Out.WriteLine(GAME_STATE_NAME + " escape");
				_parent.shutdown();
			}
		}

		/// <summary>
		/// Ends the program.
		/// </summary>
		public override void shutdown()
		{

		}
	}
}
