/*
 *  Author: Josh Wilson
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.graphics;
using rhodes.physics;
using rhodes.engine;

using sedohr.player;
using sedohr.weapon;

namespace sedohr.objects
{
    /// <summary>
    /// Class used to implement a single hull piece.
    /// </summary>
    class Hull
    {
        /// <summary>
        /// Health of the Hull piece
        /// </summary>
        private const float MAX_HEALTH = 200.0f;

        /// <summary>
        /// Amount of time before the hull can be damaged again
        /// </summary>
        private const int COOLDOWN = 50;

		/// <summary>
		/// Linear Force that will be applied to the hull tile when it breaks off
		/// </summary>
		private const float HOLE_FORCE = 10.0f;

		/// <summary>
		/// Angular Force that will be applied to the hull tile when it breaks off
		/// </summary>
		private const float TORQUE_FORCE = 100000.0f;

        /// <summary>
        /// Name of the hull piece
        /// </summary>
        private string _name;
        public string Name { get { return _name; } set { _name = value; } }

        /// <summary>
        /// Duration since the hull was last damaged
        /// </summary>
        private int _cooldown;
        public int Cooldown { get { return _cooldown; } set { _cooldown = value; } }

        /// <summary>
        /// Hull entity
        /// </summary>
        private SimpleEntity _entity;
        public SimpleEntity Entity { get { return _entity; } set { _entity = value; } }

        /// <summary>
        /// Current health of the hull
        /// </summary>
        private float _health;
        public float Health { get { return _health; } set { _health = value; } }

		/// <summary>
		/// If the hull tile has been knocked off
		/// </summary>
		private bool _knockedoff;

		/// <summary>
		/// Initial distance from the center of the black hole, used to scale the hull tile
		/// </summary>
		private float _scaleSize;

		/// <summary>
		/// Current distance from the center of the black hole, used to scale the hull tile
		/// </summary>
		private float _curScale;

		/// <summary>
		/// A vector representing the scale of a sprite.
		/// </summary>
		private Vector2 spriteScale;

        /// <summary>
        /// Hull Constructor
        /// </summary>
        /// <param name="name">Name of the hull piece</param>
        /// <param name="entity">Entity of the hull piece</param>
		public Hull(string name, SimpleEntity entity)
		{
			_name = name;
			_health = MAX_HEALTH;
			_cooldown = 0;
			_entity = entity;
			_knockedoff = false;

            //Set the geometry for the hull piece
			foreach (Geom geom in _entity.PhysicsBody.GeomList)
			{
				geom.CollisionHandler += this.collisionHandler;

			}

			_entity.PhysicsBody.Body.IgnoreGravity = true;
		}

        /// <summary>
        /// Updates the hull piece
        /// </summary>
        /// <param name="elapsedTime">Time since the last update was called</param>
        public void update(float elapsedTime)
        {
			//If the hull piece is falling into the black hole
			if (_knockedoff)
			{
				//Calculate and apply force towards the black hole
				Sprite hole = SpriteManager.getSingleton.get("background");
				Vector2 dir = hole.Position - _entity.PhysicsBody.Body.Position;
				_curScale = dir.Length() / _scaleSize;
				spriteScale.X = _curScale;
				spriteScale.Y = _curScale;

				dir.Normalize();
				Vector2 force = Vector2.Multiply(dir, HOLE_FORCE);
				_entity.PhysicsBody.Body.ApplyForce(force);
				_entity.Sprite.Scale = spriteScale;

				//Despawn the hull tile when it hits the center of the black hole or has beomce too small
				if (_curScale < 0.1f ||
					_entity.PhysicsBody.Position.X >= hole.Position.X - 5 &&
					_entity.PhysicsBody.Position.X <= hole.Position.X + 5 &&
					_entity.PhysicsBody.Position.Y >= hole.Position.Y - 5 &&
					_entity.PhysicsBody.Position.Y <= hole.Position.Y + 5)
				{
					_entity.Active = false;
				}
			}

			//If the hull piece has fallen off the screen then deactivate it
            if (_entity.PhysicsBody.Position.Y > 3000)
            {
                //Destroy hull tile.
				//Console.Out.WriteLine(_name + " pos: " + _entity.PhysicsBody.Body.Position);
				_entity.Active = false;
				
            }
        }

        /// <summary>
        ///  Handles the collision when something collides with the hull piece
		/// </summary>
		/// <param name="g1">The hull piece geometry</param>
		/// <param name="g2">The geometry of the other object</param>
		/// <param name="cl">The list of contact points</param>
		/// <returns></returns>
        public bool collisionHandler(Geom g1, Geom g2, ContactList cl)
        {
            //Update the cooldown and if its > 0 then don't process the collision
            _cooldown -= 1;
            if (_cooldown >= 0)
                return true;

            //If the name of the first geometry is the name of the hull piece and the second geometry isn't empty
            if (g1.Tag.Equals(_name) && g2.Tag != null)
            {
                //If the hull collided with a projectile then process the collision
                string collidedObjectName = g2.Tag.ToString();
                if (collidedObjectName.Contains("projectile"))
                {
                    //Start the cooldown
                    _cooldown = COOLDOWN;

                    //Get the projectile and if it's a rocket or pistol do rocket/pistol damage,
                    // otherwise do laser damage
                    Projectile projectile = ProjectileManager.getSingleton.get(collidedObjectName);
                    if (projectile != null && projectile.Active)
                    {
                        //If the damage causes the hull piece to "break off" then apply a random linear impulse
                        // and a random angular impuls
                        if (takeDamage(projectile.damage))
                        {
							knockOff();
                        }
                    }
                    else 
                    {
                        //If the damage causes the hull piece to "break off" then apply a random linear impulse
                        // and a random angular impuls
                        if (takeDamage(LaserBeam.damage))
                        {
							knockOff();
                        }
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// Applies damage to the hull piece
        /// </summary>
        /// <param name="damage">The damage to apply to the hull piece</param>
        /// <returns>True if the damage "destroys" the hull piece, false otherwise</returns>
        public bool takeDamage(float damage)
        {
            _health -= damage;

            //Console.Out.WriteLine(_name + " current health: " + _health);

            if (_health <= 0)
            {
                _entity.PhysicsBody.Body.IsStatic = false;

                foreach (Geom geom in _entity.PhysicsBody.GeomList)
                {
                    geom.CollisionHandler -= this.collisionHandler;
                }
                return true;
            }
            else
            {
                return false;
            }
        }

		/// <summary>
		/// Knocks the hull tile off the hull and makes it move towards the black hole
		/// </summary>
		public void knockOff()
		{
			_knockedoff = true;

			Sprite hole = SpriteManager.getSingleton.get("background");

			SceneManager.getSingleton.getActiveScene().getLayer(3).Remove(_name);
			SceneManager.getSingleton.getActiveScene().getLayer(2).AddSprite(_name, _entity.Sprite);
			Vector2 dir = hole.Position - _entity.PhysicsBody.Body.Position;
			_scaleSize = dir.Length();
			dir.Normalize();
			Vector2 force = Vector2.Multiply(dir, HOLE_FORCE);
			_entity.PhysicsBody.Body.ApplyForce(force);

			Random rand = new Random();
			int torque = rand.Next(-5, 5);
			_entity.PhysicsBody.Body.ApplyTorque(torque * TORQUE_FORCE);
		}
    }
}
