/*
 *  Author: David Huynh and Josh Wilson
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.graphics;
using rhodes.graphics.particles;
using rhodes.graphics.particles.emitters;
using rhodes.physics;
using rhodes.engine;

using sedohr.weapon;
using sedohr.player;

namespace sedohr.objects
{
	/// <summary>
	/// Represents simple (cingular) objects in the level.
	/// </summary>
    class SimpleObject : Object
    {
		/// <summary>
		/// Max health the object has.
		/// </summary>
		public const float MAX_HEALTH = 100.0f;

		/// <summary>
		/// Entity of the simple object
		/// </summary>
        protected SimpleEntity _simpleEntity;

		/// <summary>
		/// Particle effect for when the object is destroyed
		/// </summary>
		private static ParticleEngine _destroyedEffect;

		/// <summary>
		/// Current health of the object
		/// </summary>
		private float _health;
		public float Health { get { return _health; } set { _health = value; } }
		private Light _light;
		public Light Light { get { return _light; } set { _light = value; } }
		/// <summary>
		/// SimpleObject Constructor
		/// </summary>
		/// <param name="name">Name of the simple object</param>
		/// <param name="simple">Entity of the simple object</param>
        public SimpleObject(String name, SimpleEntity simple) : base(name) 
        {
            _simpleEntity = simple;
			_health = MAX_HEALTH;

			//If the object has a particle effect, load it
			if (_destroyedEffect == null)
			{
				_destroyedEffect = ParticleManager.getSingleton.load("destroyed", "assets/particles/explosionEffect");
			}

			//Sets the collision geometry for theobject
            foreach (Geom geom in _simpleEntity.PhysicsBody.GeomList)
            {
                geom.CollisionHandler += this.collisionCallback;
            }
        }

		/// <summary>
		/// Updates the simple object
		/// </summary>
		/// <param name="elapsedTime">Time since last update was called</param>
        public override void update(float elapsedTime)
        {
			if (_light != null)
			{
				_light.Position = new Vector3(_simpleEntity.PhysicsBody.Body.Position, 10);
			}            
        }

		/// <summary>
		/// Handles the collision when something collides with the simple object
		/// </summary>
		/// <param name="geometry1">Geometry of the simple object</param>
		/// <param name="geometry2">Geometry of the other object</param>
		/// <param name="contactList">List of contact points from the collision</param>
		/// <returns></returns>
        public override bool collisionCallback(Geom geometry1, Geom geometry2, ContactList contactList)
        {
			string geom2 = geometry2.Tag.ToString();

			//If the 2nd geometry is a projectile, cause damage to the simple object
			if (geometry1.Tag.Equals(_name) && geom2.Contains("player"))
			{
				if (geom2.Contains("projectile"))
				{
					Projectile projectile = ProjectileManager.getSingleton.get(geom2);
					if (projectile != null && projectile.Active)
					{
						//If the damage doesn't cause the object to be destroyed show the object as more damaged
						if (takeDamage(projectile.damage))
						{
							float damageColor = 1f - ((MAX_HEALTH - _health) / MAX_HEALTH * .3f / 2);
							if (damageColor > .7)
							{
								_simpleEntity.Sprite.Color = new Vector4(Vector3.One, damageColor);
							}
						}
					}
					else
					{
						//If the damage doesn't cause the object to be destroyed show the object as more damaged
						if (takeDamage(LaserBeam.damage))
						{
							float damageColor = 1f - ((MAX_HEALTH - _health) / MAX_HEALTH * .3f);
							if (damageColor > .7)
							{
								_simpleEntity.Sprite.Color = new Vector4(Vector3.One, damageColor);
							}
						}
					}
				}
			}

            return true;
        }

		/// <summary>
		/// Gets and Sets the entities property
		/// </summary>
        public SimpleEntity SimpleEntity { get { return _simpleEntity; } set { _simpleEntity = value; } }
        
		/// <summary>
		/// Gets and Sets the active property
		/// </summary>
		public Boolean Active
        {
            get
            {
                return _active;
            }
            set
            {
                if (_active != value)
                {
                    _active = value;

                    if (_active)
                    {
                        _simpleEntity.Active = true;
                    }
                    else
                    {
                        _simpleEntity.Active = false;
                    }
                }
            }
        }

		/// <summary>
        /// Applies damage to the simple object
        /// </summary>
        /// <param name="damage">The damage to apply to the hull piece</param>
        /// <returns>True if the damage "destroys" the hull piece, false otherwise</returns>
		public bool takeDamage(float damage)
		{
			_health -= damage;

			//Console.Out.WriteLine(_health);

			//If the object is destroyed by the damage create the destroyed object particle affect
			if (_health <= 0)
			{
				//If the object is the generator deactivate the smoke particle affect
				if (_simpleEntity.Sprite.Name.Equals("generator_2"))
				{
					foreach (DirectionalEmitter dEmitter in Level._generatorSmoke.Emitters)
					{
						dEmitter.IsActive = false;
					}
				}

				_simpleEntity.PhysicsBody.Body.Position = new Vector2(-8000.0f, 8000.0f);
				foreach (PointEmitter pEmitter in _destroyedEffect.Emitters)
				{
					pEmitter.EmitterPosition = _simpleEntity.Sprite.Position;
					pEmitter.IsActive = true;
				}
				Active = false;

				//Deactivate the collision geometry for the object
				foreach (Geom geom in _simpleEntity.PhysicsBody.GeomList)
				{
					geom.CollisionHandler -= this.collisionCallback;
				}
				return false;
			}
			else
			{
				return true;
			}
		}
    }
}
