using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Collisions;
using rhodes.engine;
using rhodes.graphics;
using rhodes.input;
using rhodes.physics;
using sedohr.weapon;

namespace sedohr.player
{
	/// <summary>
	/// Player state for when they are on the ground (usually platforms)
	/// Author: David Huynh
	/// </summary>
    class OnGroundPlayerState : PlayerState
	{
		#region Attributes

		/// <summary>
		/// The vector used for movement
		/// </summary>
		private Vector2 _moveVector;

		/// <summary>
		/// Whether the player is idle or not
		/// </summary>
        private bool _idle;

		/// <summary>
		/// Whether the player is moving forward or not
		/// </summary>
        private bool _forward;

		/// <summary>
		/// Whether the player is moving backward or not
		/// </summary>
        private bool _backward;

		/// <summary>
		/// Whether the player has landed on a platform or not
		/// </summary>
        private bool _doneLanding;

		/// <summary>
		/// A vector representing this player's linear velocity
		/// </summary>
		private Vector2 linearVelocity;

		/// <summary>
		/// A point to check if the player is on the ground
		/// </summary>
		private Vector2 collisionPoint;

		/// <summary>
		/// The force that makes a player jump
		/// </summary>
		private Vector2 jumpForce;

		#endregion

		#region Constructor

		/// <summary>
        /// Constructor for the state
        /// </summary>
        /// <param name="player">The player that this state is for</param>
        public OnGroundPlayerState(Player player)
            : base(player)
        {
			Name = "OnGround";
            _moveVector = new Vector2(0, 1);
			jumpForce = new Vector2(0, Player.PLAYER_JUMP_FORCE);
            _doneLanding = true;
            _idle = true;
            _forward = false;
            _backward = false;
		}

		#endregion

		#region Functions

		/// <summary>
        /// Used for updating the state
        /// </summary>
		public override void update(float elapsedTime)
        { }

        /// <summary>
        /// Handles all input from the player.
        /// </summary>
        public override void handle()
        {
            // if the player avatar pushed left on the controller or keyboard,
            // then move the player avatar left
            if (_player.PlayerControl.isPushingLeft())
			{
				if (_player.Face)
				{
					if (!_backward)
					{
						_player.Entity.Sprite.setAnimation("backward", true);
						_backward = true;
						_forward = false;
						_idle = false;
					}
				}
				else
				{
					if (!_forward)
					{
						_player.Entity.Sprite.setAnimation("forward", true);
						_forward = true;
						_backward = false;
						_idle = false;
					}
				}
                if (_player.Entity.PhysicsBody.Body.LinearVelocity.X >= -Player.MAX_HORIZONTAL_PLAYER_VELOCITY)
                {
                    _player.Entity.PhysicsBody.Body.ApplyForce(_moveVector * -Player.HORIZONTAL_PLAYER_FORCE_ON_GROUND);
                }
            }
            // if the player avatar pushed right on the controller or keyboard,
            // then move the player avatar right
			else if (_player.PlayerControl.isPushingRight())
			{
				if (_player.Face)
				{
					if (!_forward)
					{
						_player.Entity.Sprite.setAnimation("forward", true);
						_forward = true;
						_backward = false;
						_idle = false;
					}
				}
				else
				{
					if (!_backward)
					{
						_player.Entity.Sprite.setAnimation("backward", true);
						_backward = true;
						_forward = false;
						_idle = false;
					}
				}
				if (_player.Entity.PhysicsBody.Body.LinearVelocity.X <= Player.MAX_HORIZONTAL_PLAYER_VELOCITY)
				{
					_player.Entity.PhysicsBody.Body.ApplyForce(_moveVector * Player.HORIZONTAL_PLAYER_FORCE_ON_GROUND);
				}
			}
			else
			{
				if (!_idle)
				{
					_player.Entity.Sprite.setAnimation("default", true);
					_idle = true;
					_forward = false;
					_backward = false;
				}

				_player.Entity.PhysicsBody.Body.LinearVelocity =
					new Vector2(_player.Entity.PhysicsBody.Body.LinearVelocity.X - _player.Entity.PhysicsBody.Body.LinearVelocity.X/8, 
					_player.Entity.PhysicsBody.Body.LinearVelocity.Y);
			}
            
            if (_player.PlayerControl.hasJustPushedJumpButton())
            {
				Vector2 move = Vector2.Zero;
				// account for left/right directional force 
				if (_player.Entity.PhysicsBody.Body.LinearVelocity.X != 0.0f)
				{
					move.X = (_player.Entity.PhysicsBody.Body.LinearVelocity.X / Math.Abs(_player.Entity.PhysicsBody.Body.LinearVelocity.X)) *
						Player.HORIZONTAL_PLAYER_FORCE_ON_GROUND;
					move.Y = 0.0f;
				}

				linearVelocity.X = _player.Entity.PhysicsBody.Body.LinearVelocity.X;
				linearVelocity.Y = -Player.MAX_HORIZONTAL_PLAYER_VELOCITY;
				_player.Entity.PhysicsBody.Body.LinearVelocity = linearVelocity;

				_player.Entity.PhysicsBody.Body.ClearForce();
				_player.Entity.PhysicsBody.Body.ApplyForce(move);
                _player.Entity.PhysicsBody.Body.ApplyForce(jumpForce);
				_player.Entity.Sprite.setAnimation("jump", false);
				
				_player.changeState("InAir");
                _doneLanding = false;
            }
            
            foreach (Geom geom in _player.Entity.PhysicsBody.GeomList)
            {
				collisionPoint.X = _player.Entity.Sprite.Position.X;
				collisionPoint.Y = _player.Entity.Sprite.Position.Y + _player.Entity.Sprite.Height / 2.0f + 2.0f;
				if (PhysicsManager.getSingleton.PhysicsSimulator.Collide(collisionPoint) == null)
                {
                    _player.changeState("InAir");
                    _player.Entity.Sprite.setAnimation("air", true);
                    _doneLanding = false;
                }
            }
        }

        /// <summary>
        /// Function that is run when a player enters the state
        /// </summary>
        public override void enter()
        { }

        /// <summary>
        /// Function that is run when a player enters the state
        /// </summary>
        public override void exit()
        { }

        /// <summary>
        /// Function that is run when a collision occurs
        /// </summary>
        /// <param name="g1">The geometry that is being hit</param>
        /// <param name="g2">The geometry that hit the first</param>
        /// <param name="cl">The list of contacts</param>
        /// <returns>Returns true</returns>
        public override bool collisionCallback(Geom g1, Geom g2, ContactList cl)
        {
            foreach (Contact contact in cl)
            {
                if ((contact.Normal.Y < 0) && (Math.Abs(contact.Normal.X) < 0.5f))
                {
                    _moveVector.X = Math.Abs(contact.Normal.Y);
                    _moveVector.Y = -contact.Normal.X;
                }
            }

            return true;
        }

        /// <summary>
        /// The callback that is run when the animation is switched
        /// </summary>
        /// <param name="animation">The name of the animation</param>
        public override void animationCallback(string animation)
        {
            if (animation.Equals("land"))
            {
                _doneLanding = true;
                _idle = false;
                _forward = false;
                _backward = false;
            }
			else if (animation.Equals("kick"))
			{
				_player.Entity.Sprite.setAnimation("default", true);
			}
        }

		#endregion
    }
}
