/*
 *  Author: David Huynh and Robert Yates
 *  Copyright 2007. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using rhodes.engine;
using rhodes.input;
using rhodes.graphics;
using rhodes.graphics.particles;
using rhodes.graphics.particles.emitters;
using sedohr.weapon;
using sedohr.objects;

namespace sedohr.player
{
	/// <summary>
	/// Class that manages the Players
	/// Author: David Huynh & Robert Yates
	/// Modified: Josh Wilson
	/// </summary>
    class PlayerManager
	{
		#region AttrD
		/// <summary>
		/// The number of players if no controllers are plugged in
		/// </summary>
		private const int DEFAULT_NUMBER_OF_PLAYERS = 4;

		/// <summary>
		/// The primary color used for colorign sprites
		/// </summary>
		private const float PRIMARY_COLOR = .9f;

		/// <summary>
		/// The secondary color used for colorign sprites
		/// </summary>
		private const float SECONDARY_COLOR = .5f;

		/// <summary>
		/// The color of player 1. (Red)
		/// </summary>
		public static Vector4 PLAYER_ONE_COLOR = new Vector4(PRIMARY_COLOR, SECONDARY_COLOR, SECONDARY_COLOR, 1);

		/// <summary>
		/// The color of player 2. (Blue)
		/// </summary>
		public static Vector4 PLAYER_TWO_COLOR = new Vector4(SECONDARY_COLOR, SECONDARY_COLOR, PRIMARY_COLOR, 1);

		/// <summary>
		/// The color of player 3. (Green)
		/// </summary>
		public static Vector4 PLAYER_THREE_COLOR = new Vector4(SECONDARY_COLOR, PRIMARY_COLOR, SECONDARY_COLOR, 1);

		/// <summary>
		/// The color of player 4. (Yellow)
		/// </summary>
		public static Vector4 PLAYER_FOUR_COLOR = new Vector4(PRIMARY_COLOR, PRIMARY_COLOR, SECONDARY_COLOR, 1);

		/// <summary>
		/// The collection of players
		/// </summary>
        private Dictionary<String, Player> _players;

		/// <summary>
		/// The collection of particles for the laser
		/// </summary>
		private ParticleEngine[] _laserParticles;

		/// <summary>
		/// The layer to draw the players, their weapons, and their bullets.
		/// </summary>
		private Layer sceneLayer;

        /// <summary>
        /// List of the Respawn Points for players
        /// </summary>
        private List<Vector2> _respawnPoints;

		/// <summary>
		/// List of the cooldown timers for each spawn point
		/// </summary>
		private List<float> _spawnpointTimers;

		#endregion

		#region Accessors

		/// <summary>
		/// Obtains the collection of players
		/// </summary>
		public Dictionary<String, Player> Players
		{
			get { return _players; }
		}

		/// <summary>
		/// Obtains the particle engine for the laser particles
		/// </summary>
		public ParticleEngine[] LaserParticles
		{
			get { return _laserParticles; }
		}

		/// <summary>
		/// Obtains the respawn points
		/// </summary>
        public List<Vector2> RespawnPoints
        {
            get { return _respawnPoints; }
		}

		/// <summary>
		/// Obtains the spawnpoint timers
		/// </summary>
		public List<float> SpawnpointTimers
		{
			get { return _spawnpointTimers; }
		}

		#endregion

		#region Constructor

		/// <summary>
		/// Creates a collection of players and loads the
		/// sprite images onto the scene. Players are added
		/// based on whether or not their controllers
		/// are connected. If no controllers are connected,
		/// it defaults to two players.
		/// </summary>
		/// <param name="input">The input to control the players</param>
		/// <param name="layer">The layer where the sprites will be displayed</param>
		/// <param name="objectManager">Manages platforms and dynamic objects in the level</param>
		public PlayerManager(Input input, Layer layer, ObjectManager objectManager)
		{

			_players = new Dictionary<String, Player>();
			sceneLayer = layer;

			int playerCount = 0;
			Player player = null;
			if (input.current[(int)PlayerIndex.One].connected)
			{
				player = new Player(PlayerIndex.One, input, objectManager, this);
				_players.Add(player.PlayerName, player);
				playerCount++;
			}
			if (input.current[(int)PlayerIndex.Two].connected)
			{
				player = new Player(PlayerIndex.Two, input, objectManager, this);
				_players.Add(player.PlayerName, player);
				playerCount++;
			}

			if (input.current[(int)PlayerIndex.Three].connected)
			{
				player = new Player(PlayerIndex.Three, input, objectManager, this);
				_players.Add(player.PlayerName, player);
				playerCount++;
			}
			if (input.current[(int)PlayerIndex.Four].connected)
			{
				player = new Player(PlayerIndex.Four, input, objectManager, this);
				_players.Add(player.PlayerName, player);
				playerCount++;
			}


			// display game with two players if there are
			// no Xbox 360 controllers plugged in
			if (playerCount == 0)
			{
				if (playerCount < DEFAULT_NUMBER_OF_PLAYERS)
				{
					player = new Player(PlayerIndex.One, input, objectManager, this);
					_players.Add(player.PlayerName, player);
					playerCount++;
				}

				if (playerCount < DEFAULT_NUMBER_OF_PLAYERS)
				{
					player = new Player(PlayerIndex.Two, input, objectManager, this);
					_players.Add(player.PlayerName, player);
					playerCount++;
				}

				if (playerCount < DEFAULT_NUMBER_OF_PLAYERS)
				{
					player = new Player(PlayerIndex.Three, input, objectManager, this);
					_players.Add(player.PlayerName, player);
					playerCount++;
				}

				if (playerCount < DEFAULT_NUMBER_OF_PLAYERS)
				{
					player = new Player(PlayerIndex.Four, input, objectManager, this);
					_players.Add(player.PlayerName, player);
					playerCount++;
				}
			}

			_laserParticles = new ParticleEngine[playerCount];

			_respawnPoints = new List<Vector2>();
			_spawnpointTimers = new List<float>();

			_respawnPoints.Add(new Vector2(1000, 600));
			_respawnPoints.Add(new Vector2(1000, 1600));
			_respawnPoints.Add(new Vector2(1100, 1050));
			_respawnPoints.Add(new Vector2(1900, 1050));
			_respawnPoints.Add(new Vector2(2100, 1600));
			_respawnPoints.Add(new Vector2(2200, 600));

			for (int k = 0; k < _respawnPoints.Count; ++k)
			{
				_spawnpointTimers.Add(0.0f);
			}

			int i = 0;

			// set colors and particles
			foreach (Player tempPlayer in this.Players.Values)
			{
				// set the color of the player
				if (tempPlayer.PlayerNumber == PlayerIndex.One)
				{
					tempPlayer.Entity.Sprite.Color = PLAYER_ONE_COLOR;
				}
				else if (tempPlayer.PlayerNumber == PlayerIndex.Two)
				{
					tempPlayer.Entity.Sprite.Color = PLAYER_TWO_COLOR;
				}
				else if (tempPlayer.PlayerNumber == PlayerIndex.Three)
				{
					tempPlayer.Entity.Sprite.Color = PLAYER_THREE_COLOR;
				}
				else if (tempPlayer.PlayerNumber == PlayerIndex.Four)
				{
					tempPlayer.Entity.Sprite.Color = PLAYER_FOUR_COLOR;
				}

				// set the color of the player's weapons
                foreach(Weapon weapon in tempPlayer.Weapons.Values)
				{
                    if(tempPlayer.PlayerNumber == PlayerIndex.One) {
						weapon.Sprite.Color = PLAYER_ONE_COLOR;
					}
					else if (tempPlayer.PlayerNumber == PlayerIndex.Two)
					{
						weapon.Sprite.Color = PLAYER_TWO_COLOR;
					}
					else if (tempPlayer.PlayerNumber == PlayerIndex.Three)
					{
						weapon.Sprite.Color = PLAYER_THREE_COLOR;
                    } else if(tempPlayer.PlayerNumber == PlayerIndex.Four) {
						weapon.Sprite.Color = PLAYER_FOUR_COLOR;
                    }
					sceneLayer.AddSprite(weapon.Name, weapon.Sprite);

					if (weapon.Genre == Weapon.weaponType.LASER)
					{
						
						// set particles for laser
						sceneLayer.AddSprite(tempPlayer.PlayerName, tempPlayer.Entity.Sprite);
						_laserParticles[i] = ParticleManager.getSingleton.load("laser", "assets/particles/laserEffect");
						sceneLayer.AddParticle("laser" + tempPlayer.PlayerName, _laserParticles[i]);
						i++;

						// add laser beam sprites to scene
						Laser laser = (Laser)weapon;
						foreach (LaserBeam laserBeam in laser.LaserBeamDictionary.Values)
						{
							sceneLayer.AddSprite(laserBeam.Sprite.Name, laserBeam.Sprite);
						}
					}
				}
			}

			// add pistol projectile sprites to scene
			foreach (Projectile pistolProjectile in
				ProjectileManager.getSingleton.PistolProjectiles.Values)
			{
				sceneLayer.AddSprite(pistolProjectile.Sprite.Name, pistolProjectile.Sprite);
			}

			// add rocket projectile sprites to scene
			foreach (Projectile rocketProjectile in
				ProjectileManager.getSingleton.RocketProjectiles.Values)
			{
				sceneLayer.AddSprite(rocketProjectile.Sprite.Name, rocketProjectile.Sprite);
			}
		}

		#endregion

		#region Functions

		/// <summary>
		/// Updates all the players.
		/// </summary>
		/// <param name="time">The time since this method was last called.</param>
		public void update(float elapsedTime)
        {
            ProjectileManager.getSingleton.update(elapsedTime);

            //foreach (float timer in _spawnpointTimers)
            for (int j = 0; j < _spawnpointTimers.Count; ++j)
            {
                if (_spawnpointTimers[j] > 0.0)
                {
                    _spawnpointTimers[j] -= elapsedTime;
                    if (_spawnpointTimers[j ]< 0.0)
                        _spawnpointTimers[j] = 0.0f;
                }
            }

			int i = 0;
            foreach (Player player in _players.Values)
            {
				player.update(elapsedTime);

				if (sceneLayer != null)
				{
					foreach (Weapon weapon in player.Weapons.Values)
					{
						// if this weapon is a laser,
						// then activate/deactivate laser spline particles
						if (weapon.Genre == Weapon.weaponType.LASER)
						{
							foreach (SplineEmitter laserSpline in _laserParticles[i].Emitters)
							{
								if (weapon.isFiring)
								{
									if (!laserSpline.IsActive)
									{
										laserSpline.IsActive = true;
									}
									laserSpline.EmitterPosition = ((Laser)weapon).LaserBeamParticleList[0];
									laserSpline.EmitterOffsets = ((Laser)weapon).LaserBeamParticleList;
								}
								else
								{
									if (laserSpline.IsActive)
									{
										laserSpline.IsActive = false;
									}
								}
							}
							((Laser)weapon).LaserBeamParticleList.Clear();
						}
					}
				}
                i++;
            }
        }

		/// <summary>
		/// Handles the players' logic, input, etc.
		/// </summary>
        public void handle()
        {
            foreach (Player player in _players.Values)
            {
                player.handle();
            }
        }

		/// <summary>
		/// Starts the spawn point timer
		/// </summary>
		/// <param name="spawnPoint">The spawn point index number</param>
        public void startSpawnPointTimer(int spawnPoint)
        {
            _spawnpointTimers[spawnPoint] = 3.0f;
		}

		#endregion
	}
}
