using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using FarseerGames.FarseerPhysics.Collisions;
using FarseerGames.FarseerPhysics.Dynamics;
using rhodes.physics;

namespace sedohr.weapon
{
    /// <summary>
    /// Class that represents an explosion
	/// Author: Andrew Kane
    /// </summary>
    class Explosion
	{
		#region Constants

		/// <summary>
        /// The amount of time an explosion lasts for
        /// </summary>
        public const float EXPLOSION_TIME = 2.0f;

		#endregion

		#region Attributes

		/// <summary>
        /// The timer to hold how long the explosion has been on for
        /// </summary>
        protected float _timer;

        /// <summary>
        /// The name of the explosion for the physics body key
        /// </summary>
        protected string _name;

		#endregion

		#region Accessors

		/// <summary>
        /// Obtains the name of the explosion instance
        /// </summary>
        public string name
        {
            get { return _name; }
		}

		#endregion

		#region Constructor

		/// <summary>
        /// Constructor for Explosion
        /// </summary>
        /// <param name="splashBodies">The List that contains all of the explosion physics bodies</param>
        /// <param name="position">The position of the explosion</param>
        /// <param name="splash">The radius ("splash range") of the explosion</param>
        /// <param name="funct">The collision delegate to call</param>
        public Explosion(Dictionary<string, PhysicsBody> splashBodies, Vector2 position, float splash, Geom.CollisionHandlerDelegate funct)
        {
            _name = "splash_" + splashBodies.Count;

            Body body = BodyFactory.Instance.CreateCircleBody(
                PhysicsManager.getSingleton.PhysicsSimulator,
                splash, 1);
            body.Tag = "splash_" + splashBodies.Count;
            body.IsStatic = true;
            body.Position = position;

            Geom splash_geom = GeomFactory.Instance.CreateCircleGeom(
                PhysicsManager.getSingleton.PhysicsSimulator,
                body, splash, Convert.ToInt32(splash * 3));
            splash_geom.Tag = body.Tag = "splash_" + splashBodies.Count;
            splash_geom.CollisionEnabled = true;
            splash_geom.CollisionResponseEnabled = false;

            List<Geom> splashGeoms = new List<Geom>();
            splashGeoms.Add(splash_geom);

            PhysicsBody physicsBody = PhysicsManager.getSingleton.addPhysicsBody("splash_" + splashBodies.Count, body, splashGeoms);
            foreach(Geom sGeom in physicsBody.GeomList)
            {
                sGeom.Tag = "splash_body" + splashBodies.Count;
                sGeom.CollisionHandler += funct;
            }
            splashBodies.Add(_name, physicsBody);
            _timer = EXPLOSION_TIME;
            return;
        }

        /// <summary>
        /// Updates the explosion
        /// </summary>
        /// <param name="elapsedTime">The elapsed time</param>
        public void update(float elapsedTime)
        {
            _timer -= elapsedTime;
            if(_timer <= 0)
                ExplosionManager.getSingleton.remove(_name);
            return;
		}

		#endregion
	}
}
