using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.engine;
using rhodes.graphics;
using rhodes.physics;
using rhodes.audio;

namespace sedohr.weapon
{
	/// <summary>
	/// A laser weapon that shoots a laser beam.
	/// Author: Robert Yates
	/// Modifications: Andrew Kane
	/// </summary>
    class Laser : Weapon
	{
		#region Constants

		/// <summary>
		/// True, if the player is allowed to rotate the laser 360 degrees
		/// after initially firing it.
		/// False, if the player is only allowed to rotate the laser
		/// 180 degrees after initially firing it.
		/// </summary>
		private const bool ENABLE_FULL_ROTATION = true;

		/// <summary>
		/// The number of segments in the beam shot from the laser
		/// (Note: Increasing the number of laser beam segments makes
		/// the game take a really long time to load.)
		/// </summary>
		private const int NUMBER_OF_LASER_BEAM_SEGMENTS = 60;

		/// <summary>
		/// The length of time the player must wait
		/// before s/he can reshoot a laser
		/// </summary>
		private const float LASER_COOLDOWN = 2.0f;

		/// <summary>
		/// The length of time the laser beam exists after it is shot
		/// </summary>
		private const float LASER_DURATION = 2.0f;

		/// <summary>
		/// The size of rotation queue divided by
		/// the number of laser beam segments
		/// </summary>
		private const int ROTATION_QUEUE_MULTIPLE = 16;

		/// <summary>
		/// The amount that the rotation queue iterator
		/// decreases in each iteration
		/// </summary>
		private const int ROTATION_QUEUE_SUBTRACTION = 4;

		/// <summary>
		/// The distance between laser beam segments when
		/// the laser beam at its maximum length.
		/// </summary>
		private const int MAX_LASER_BEAM_SEGMENT_DISTANCE = 40;

		#endregion

		#region Attributes

		/// <summary>
		/// A vector representing the laser weapon's rotation.
		/// </summary>
		private Vector2 currentRotationVector; 

		/// <summary>
		/// The distance between laser beam segments.
		/// </summary>
		private int laserBeamSegmentDistance = 0;

		/// <summary>
		/// The vector representing the angle 180 degrees (pi radians)
		/// from the weapon's original rotation
		/// </summary>
		private Vector2 oppositeOriginalRotationVector;

		/// <summary>
		/// The queue of rotations from calls to update
		/// </summary>
		private Queue<float> rotationQueue = new Queue<float>();
	
		/// <summary>
		/// The laser beams segments of this weapon
		/// </summary>
		private Dictionary<string, LaserBeam> laserBeamDictionary;

		/// <summary>
		/// The list of particles representing laser beam segments
		/// </summary>
		private List<Vector2> laserBeamParticleList = new List<Vector2>();

		/// <summary>
		/// True if the player just spawned and the fire button must be
		/// released before the player can shoot the laser. False,
		/// if the player had released the fire button (if it was
		/// held down) after the player spawned.
		/// </summary>
		private bool playerJustSpawned = true;

		/// <summary>
		/// The image used in the sprite placeholder for
		/// the laser beam.
		/// </summary>
		protected SpriteMap laserBeamSpriteMap;

		/// <summary>
		/// The rotation of a laser beam's segment.
		/// </summary>
		private Vector2 segmentRotationVector;

		#endregion

		#region Accessors

		/// <summary>
		/// The list of particles representing laser beam segments
		/// </summary>
		public List<Vector2> LaserBeamParticleList
		{
			get { return laserBeamParticleList; }
		}

		/// <summary>
		/// The laser beams segments of this weapon
		/// </summary>
		public Dictionary<string, LaserBeam> LaserBeamDictionary
		{
			get { return laserBeamDictionary; }
		}

		#endregion

		#region Constructor

		/// <summary>
		/// Laser weapon constructor
		/// (Visibility defaults to false.)
		/// </summary>
		/// <param name="playerIndex">The index of the player firing this laser</param>
        public Laser(PlayerIndex playerIndex) : base(playerIndex)
        {
            // set name and genre
			weaponName = "laser " + "player" + ((int)playerIndex + 1);
            weaponGenre = weaponType.LASER;
            
			// load sprites
			weaponSprite = SpriteManager.getSingleton.load(
				weaponName, "assets/sprites/laserWeapon");
			laserBeamSpriteMap =
				SpriteResourceManager.getSingleton.load(
				"assets/sprites/laserBeam");


			// create laser beam segments
			laserBeamDictionary = new Dictionary<string, LaserBeam>();
			for (int i = 0; i < NUMBER_OF_LASER_BEAM_SEGMENTS; i++) {
				laserBeamDictionary.Add(weaponName + i, createLaserBeam(i));
			}

			// set cool down and weapon duration
            cooldownAmount = LASER_COOLDOWN;
            weaponDuration = LASER_DURATION;

			// deactivate laser beam segements
			foreach (LaserBeam laserBeam in laserBeamDictionary.Values)
			{
				laserBeam.deactivate();
			}

			// set previously pressing weapon
			previouslyPressingWeaponButton = false;

			// prepare for when the player spawns
			playerJustSpawned = true;

			Visible = false;
		}

		#endregion

		#region Functions
		/// <summary>
		/// Update the weapon
		/// </summary>
		/// <param name="elapsedTime">The time since the last update</param>
		/// <param name="press">True, if the fire button is being pressed. False, if not.</param>
		/// <param name="position">The weapon's position</param>
		/// <param name="rotation">The weapon's rotation</param>
		public override void update(float elapsedTime, bool press,
			Vector2 position, float rotation)
		{
			if (playerJustSpawned)
			{
				// if player has just spawned and
				// not holding down fire button
				if (!press)
				{
					// player has not just spawned
					playerJustSpawned = false;
				}
			}
			else
			{
				// if player has just spawned, allow player to fire weapon

				// just pressed fire button
				if (press && !previouslyPressingWeaponButton)
				{
					// set laser beam segment distance and original rotation 
					laserBeamSegmentDistance = 0;

					float oppositeOriginalRotation = rotation + (float)(Math.PI);
					oppositeOriginalRotationVector.X = (float)Math.Cos(oppositeOriginalRotation);
					oppositeOriginalRotationVector.Y = (float)Math.Sin(oppositeOriginalRotation);

					// enqueue the current rotation multiple times
					for (int i = 0; i < (ROTATION_QUEUE_MULTIPLE *
						NUMBER_OF_LASER_BEAM_SEGMENTS); i++)
					{
						rotationQueue.Enqueue(rotation);
					}

					// play laser sound
					AudioManager.getSingleton.playSoundFXs("laser");

					// activate laser beam segements
					foreach (LaserBeam laserBeam in laserBeamDictionary.Values)
					{
						laserBeam.Active = true;
					}
				}

				// just released fire button
				if (!press && previouslyPressingWeaponButton)
				{
					destroyLaserBeam();
				}

				
				// if we want to diable full rotation and the
				// player is currently pressing the button
				if (!ENABLE_FULL_ROTATION && press)
				{
				    currentRotationVector.X = (float)Math.Cos(rotation);
				    currentRotationVector.X = (float)Math.Sin(rotation);

				    // compute the distance the current rotation vector
				    // is along the opposite of the original rotation vector
				    float distOppositeOriginalRotation = Vector2.Dot(
				        currentRotationVector, oppositeOriginalRotationVector);

				    // if the current rotation is on the
				    // opposite side of the original rotation
				    if (distOppositeOriginalRotation > 0)
				    {
				        // Compute half the vector that must be subtracted
				        // from the current rotation vector and
				        // obtain the final rotation
				        Vector2 halfOppositeVector = distOppositeOriginalRotation *
				            oppositeOriginalRotationVector;
				        Vector2 finalRotationVector = currentRotationVector -
				            halfOppositeVector - halfOppositeVector;

				        // set the current rotation to be the final rotation
				        rotation = (float)Math.Atan2(
				            finalRotationVector.Y, finalRotationVector.X);
				    }

				}

				// enqueue the current rotation and
				// dequeue the earliest rotation
				rotationQueue.Enqueue(rotation);
				rotationQueue.Dequeue();

				// if the player is holding down the weapon button
				if (press)
				{
					// increase laser beam segment distance to shoot out laser
					if (laserBeamSegmentDistance <
						MAX_LASER_BEAM_SEGMENT_DISTANCE)
					{
						laserBeamSegmentDistance++;
					}

					// create rotation array and iterator
					float[] rotationArray = rotationQueue.ToArray();
					int rotationIter = rotationArray.Length - 1;

					// initialize loop variables
					int i = 0;
					Vector2 segmentOffset = Vector2.Zero;

					bool disableOuterLaserBeamSegments = false;

					// draw laser beam segments
					foreach (LaserBeam laserBeam in laserBeamDictionary.Values)
					{
						// calculate the rotation of this laser beam segment
						float segmentRotation = rotationArray[rotationIter];

						// calculate the position of this laser beam segment
						segmentRotationVector.X = (float)Math.Cos(segmentRotation);
						segmentRotationVector.Y = (float)Math.Sin(segmentRotation);

						segmentOffset +=
							(laserBeamSegmentDistance * segmentRotationVector);
						Vector2 segmentPosition = position + segmentOffset;

						// update laser
						laserBeam.update(elapsedTime, segmentPosition,
							segmentRotation, disableOuterLaserBeamSegments);

						// if this is the most inner inactive segment,
						// make all outer segments inactive and
						// make this segment active
						if (!laserBeam.Active && !disableOuterLaserBeamSegments)
						{
							disableOuterLaserBeamSegments = true;
							laserBeam.Active = true;
						}

						// draw laser beam particles for active segments
						if (laserBeam.Active)
						{
							laserBeamParticleList.Add(segmentPosition);
						}

						// decrease rotation iterator and
						// increment regular iterator
						rotationIter -= ROTATION_QUEUE_SUBTRACTION;
						i++;
					}
				}

				// set previously pressing weapon
				previouslyPressingWeaponButton = press;
			}

			// set the gun's position and rotation
			weaponSprite.Position = position;
			weaponSprite.Rotation = rotation;
		}

		/// <summary>
		/// Creates a laser beam segment.
		/// </summary>
		/// <param name="uniqueId">A unique number for this laser beam.</param>
		private LaserBeam createLaserBeam(int uniqueId)
		{
			// create laser beam name
			string playerName = weaponName.Substring(weaponName.LastIndexOf('p'));
			string shortenedWeaponName = weaponName.Substring(0, weaponName.LastIndexOf(' '));

			// laser beam must have the word " projectile" in it for the laser
			// to damage players
			string laserBeamName =	playerName + " projectile" +
				uniqueId + " " + shortenedWeaponName;

			// create laser beam sprite
			Sprite laserBeamSprite = SpriteManager.getSingleton.load(
				laserBeamName, laserBeamSpriteMap);

			// create body
			Body body = BodyFactory.Instance.CreateRectangleBody(
				PhysicsManager.getSingleton.PhysicsSimulator,
				laserBeamSprite.Width, laserBeamSprite.Height, 1);
			body.IsStatic = false; // make body not static so it can move

			// create geometry
			List<Geom> geomList = new List<Geom>();
			Geom geom = GeomFactory.Instance.CreateRectangleGeom(
				PhysicsManager.getSingleton.PhysicsSimulator,
				body, laserBeamSprite.Width, laserBeamSprite.Height);
			geom.Tag = body.Tag = laserBeamName;
			geom.CollisionEnabled = true;
			geom.CollisionResponseEnabled = false;
			geomList.Add(geom);

			// create laser beam
			PhysicsBody physicsBody = PhysicsManager.getSingleton.addPhysicsBody(laserBeamName, body, geomList);
			LaserBeam laserBeam = new LaserBeam(laserBeamName, laserBeamSprite, physicsBody, playerNum);
			return laserBeam;
		}

		/// <summary>
		/// Destroys the laser beam segments when the weapon button
		/// is released or the player holding this weapon dies.
		/// </summary>
		public void destroyLaserBeam()
		{
			// deactivate laser beam segements
			foreach (LaserBeam laserBeam in laserBeamDictionary.Values)
			{
				laserBeam.deactivate();
			}

			// clear rotation queue
			rotationQueue.Clear();

			// set previously pressing weapon
			previouslyPressingWeaponButton = false;

			// prepare for when the player spawns
			playerJustSpawned = true;
		}

		#endregion

	}
}
