using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.engine;
using rhodes.graphics;
using rhodes.physics;
using rhodes.audio;
using sedohr.player;

namespace sedohr.weapon
{
	/// <summary>
	/// This class represents a laser beam segment. It inherits from
	/// rhodes.engine.SimpleEntity which contains a sprite and physics body.
	/// Author: Robert Yates
	/// </summary>
	class LaserBeam : SimpleEntity
	{
		/// <summary>
		/// True, if we want to see the laser beam rectangles for debuging.
		/// False, if we just want to see the particles of the laser beam.
		/// </summary>
		private const bool DISPLAY_BEAM_RECTANGLE = false;

		/// <summary>
		/// The name of the player that fired this laser beam
		/// </summary>
		private string playerName;
		/// <summary>
		/// The player who shot this laser beam
		/// </summary>
		protected PlayerIndex playerNum;
		/// <summary>
		/// The type of weapon that fired this laser beam
		/// </summary>
		protected Weapon.weaponType weaponGenre;

		/// <summary>
		/// The position where laser beams go when they are deactivated
		/// </summary>
		private Vector2 OFF_SCREEN_POSITION = new Vector2(-5000.0f, -5000.0f);

		/// <summary>
		/// True if the laser in turned on. False if it is turned off.
		/// (Overrides active method in SimpleEntity.)
		/// </summary>
		public new Boolean Active
		{
			get
			{
				return _active;
			}
			set
			{
				_physicsBody.Body.Enabled = _active = value;
				
				if (DISPLAY_BEAM_RECTANGLE)
				{
					_sprite.Visible = value;
				}
			}
		}

		/// <summary>
		/// Gets the amount of damage the laser beam will inflict on the player
		/// </summary>
		static public float damage
		{
			get	{ return 0.1f;	}
		}

		/// <summary>
		/// The LaserBeam constructor
		/// </summary>
		/// <param name="name">The unique name of this laser beam</param>
		/// <param name="sprite">The sprite image of this laser beam</param>
		/// <param name="physicsBody">The physics body used for collision detection</param>
		/// <param name="playerIndex">What player fired this laser beam</param>
		public LaserBeam(string name, Sprite sprite, PhysicsBody physicsBody,
			PlayerIndex playerIndex) : base(name, physicsBody, sprite)
		{

			if (!DISPLAY_BEAM_RECTANGLE)
			{
				// sprite is always invisible
				_sprite.Visible = false;
			}

			// set initial values
			playerNum = playerIndex;
			playerName = "player" + ((int)playerIndex + 1);
			weaponGenre = Weapon.weaponType.LASER;

			// set geometry tags, collision categories, and collision categories
			// this object collides with
			foreach (Geom geom in physicsBody.GeomList)
			{
				geom.Tag = _name;
				if (playerIndex == PlayerIndex.One)
				{
					geom.CollisionCategories = Enums.CollisionCategories.Cat1;
					geom.CollidesWith =
						Enums.CollisionCategories.Cat2 |
						Enums.CollisionCategories.Cat3 |
						Enums.CollisionCategories.Cat4 |
						Enums.CollisionCategories.Cat5 |
						Enums.CollisionCategories.Cat6;
				}
				else if (playerIndex == PlayerIndex.Two)
				{
					geom.CollisionCategories = Enums.CollisionCategories.Cat2;
					geom.CollidesWith =
						Enums.CollisionCategories.Cat1 |
						Enums.CollisionCategories.Cat3 |
						Enums.CollisionCategories.Cat4 |
						Enums.CollisionCategories.Cat5 |
						Enums.CollisionCategories.Cat6;
				}
				else if (playerIndex == PlayerIndex.Three)
				{
					geom.CollisionCategories = Enums.CollisionCategories.Cat3;
					geom.CollidesWith =
						Enums.CollisionCategories.Cat1 |
						Enums.CollisionCategories.Cat2 |
						Enums.CollisionCategories.Cat4 |
						Enums.CollisionCategories.Cat5 |
						Enums.CollisionCategories.Cat6;
				}
				else if (playerIndex == PlayerIndex.Four)
				{
					geom.CollisionCategories = Enums.CollisionCategories.Cat4;
					geom.CollidesWith =
						Enums.CollisionCategories.Cat1 |
						Enums.CollisionCategories.Cat2 |
						Enums.CollisionCategories.Cat3 |
						Enums.CollisionCategories.Cat5 |
						Enums.CollisionCategories.Cat6;
				}
			}
		}

		/// <summary>
		/// Updates the properties of the projectile
		/// </summary>
		/// <param name="elapsedTime">The amount of time that has passed</param>
		/// <param name="position">The position of the laser beam segment</param>
		/// <param name="rotation">The rotation of the laser beam segment</param>
		/// <param name="disableLaserBeamSegment">
		/// True, if we need to disable this segment. False,
		/// if we use collision detection to see whether or not
		/// this segment is active.
		/// </param>
		public void update(float elapsedTime, Vector2 position,
			float rotation, bool disableLaserBeamSegment)
		{
			base.update(elapsedTime);
			//elapsed time is about 0.016; 1 = 0.016 * 62.5

			// set physics body sprite position and rotation
			_physicsBody.Body.Position = _sprite.Position = position;
			_physicsBody.Body.Rotation = _sprite.Rotation = rotation;

			
			// assume there is no collision, so set segment to be active
			Active = true;

			// if we are disabling this segment
			if (disableLaserBeamSegment)
			{
				//set segment to be inactive
				Active = false;
			}
			else
			{
				// otherwise get the geometry this laser beam is colliding with
				Geom geometryLaserBeamIsCollidingWith =
					PhysicsManager.getSingleton.PhysicsSimulator.Collide(
						_sprite.Position);
				// if the geometry this laser beam is colliding with is from
				// category 5 (static objects like platforms) or
				// category 6 (dynamic objects that move in environment)
				if (geometryLaserBeamIsCollidingWith != null)
				{
					if (geometryLaserBeamIsCollidingWith.CollisionCategories == Enums.CollisionCategories.Cat5 ||
						geometryLaserBeamIsCollidingWith.CollisionCategories == Enums.CollisionCategories.Cat6)
					{
						// there is a collision, so set segment to be inactive
						Active = false;
					}
				}
			}
			
		}

		/// <summary>
		/// Deactivate segment and set it to be off the screen
		/// </summary>
		public void deactivate()
		{
			Active = false;
			_physicsBody.Body.Position = OFF_SCREEN_POSITION;
			foreach (Geom geom in _physicsBody.GeomList)
			{
				geom.Tag = _name;
				geom.CollisionHandler = null;
			}
		}

	}
}
