using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;

using rhodes.graphics;
using rhodes.physics;

using sedohr.player;

using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

/*
 * ProjectileManager.cs
 * 
 * Allows projectiles to be reused by weapons after
 * they are shot, hit something, and deactivated.
 *
 * Created By: Andrew Kane
 * Modified By: Anthony Reese and Rob Yates
 * Commented By: Robert Yates and Andrew Kane
 */
namespace sedohr.weapon
{
	/// <summary>
	/// This class allows projectiles to be reused by weapons after
	/// they are shot, hit something, and deactivated.
	/// Two dictionaries are used to keep track of
	/// rocket projectiles and pistol projectiles.
	/// </summary>
    class ProjectileManager
	{
		#region Constants
		/// <summary>
        /// The number of projectiles of each type (rocket or pistol) that belong to a player
        /// </summary>
        private const int NUMBER_OF_PLAYER_PROJECTILES = 20;

		/// <summary>
		/// The maximum number of possible players
		/// </summary>
		private const int MAXIMUM_NUMBER_OF_PLAYERS = 4;

		#endregion

		#region Attributes

		/// /// <summary>
        /// Static instance of the ProjectileManager class so that we can use it as a singleton
        /// </summary>
        private static volatile ProjectileManager instance;

        /// <summary>
        /// Object to sync/lock around to make sure nothing is re-initialized
        /// </summary>
        private static object syncRoot = new Object();

        /// <summary>
		/// A dictionary to store all the rocket projectiles.
        /// </summary>
		private Dictionary<String, Projectile> rocketProjectiles;

		/// <summary>
		/// A dictionary to store all the rocket projectiles.
		/// </summary>
		public Dictionary<String, Projectile> RocketProjectiles
		{
			get { return rocketProjectiles; }
		}

		/// <summary>
		/// A dictionary to store all the pistol projectiles.
		/// </summary>
		private Dictionary<String, Projectile> pistolProjectiles;

        /// <summary>
		// Creates a hierarchy where only projectiles of a greater level
		// will not be destroyed upon collision. For example, a rocket will
		// destroy pistol bullets, but will not be destroyed itself.
        /// </summary>
        private SortedDictionary<string, int> weaponLevels;

		/// <summary>
		// Creates a hierarchy where only projectiles of a greater level
		// will not be destroyed upon collision. For example, a rocket will
		// destroy pistol bullets, but will not be destroyed itself.
		/// </summary>

		/// <summary>
		/// Sprite Map to represent the rocket projectiles
		/// </summary>
		private SpriteMap rocketProjectileSpriteMap;

		/// <summary>
		/// Sprite Map to represent the pistol projectiles
		/// </summary>
		private SpriteMap pistolProjectileSpriteMap;

		#endregion

		#region Accessors

		/// <summary>
		/// Obtains the list of pistol projectiles
		/// </summary>
		public Dictionary<String, Projectile> PistolProjectiles
		{
			get { return pistolProjectiles; }
		}

		/// <summary>
		/// Obtains the list of weapon levels
		/// </summary>
        public SortedDictionary<string, int> WeaponLevels
        {
            get { return weaponLevels; }
		}

		/// <summary>
		/// Returns a ProjectileManager object that is a singleton
		/// </summary>
		public static ProjectileManager getSingleton
		{
			get
			{
				if (instance == null)
				{
					lock (syncRoot)
					{
						if (instance == null)
							instance = new ProjectileManager();
					}
				}
				return instance;
			}
		}
		/// <summary>
		/// A list of all the players
		/// </summary>
		private Dictionary<String, Player> players;
		
		/// <summary>
		/// A list of all the players
		/// </summary>
		public Dictionary<String, Player> Players {
			get { return players; }
			set { players = value; }
		}

		#endregion

		#region Constructor

		/// <summary>
        /// Default Constructor for ProjectileManager
        /// </summary>
        public ProjectileManager()
        {
			// load rocket and pistol sprites
			rocketProjectileSpriteMap = SpriteResourceManager.getSingleton.load("assets/sprites/rocketLauncherProjectile");
			pistolProjectileSpriteMap = SpriteResourceManager.getSingleton.load("assets/sprites/pistolProjectile");

			// initialize projectile dictionaries
			rocketProjectiles = new Dictionary<String, Projectile>();
			pistolProjectiles = new Dictionary<String, Projectile>();

			// initialize weapon levels
            weaponLevels = new SortedDictionary<string, int>();
            weaponLevels.Add("pistol", 1);
            weaponLevels.Add("rocket", 2);
			weaponLevels.Add("laser", 3);

			// initialize the pistol and rocket projectiles
			for (int playerNum = 1; playerNum <= MAXIMUM_NUMBER_OF_PLAYERS; playerNum++)
			{
				for (int projectileNum = 0; projectileNum < NUMBER_OF_PLAYER_PROJECTILES; projectileNum++)
				{
					// obtain projectile name, sprite and physics body for the pistol projectile
					String pistolProjectileName = "player" + playerNum + " projectile" +	projectileNum + " pistol";
					Sprite pistolProjectileSprite = SpriteManager.getSingleton.load(pistolProjectileName, pistolProjectileSpriteMap);
					PhysicsBody pistolPhysicsBody = generatePhysicsBody(pistolProjectileName, pistolProjectileSprite);

					// create a pistol projectile, set it to be inactive,
					// and add it to the pistol projectiles dictionary
					Projectile pistolProjectile = new Projectile(pistolProjectileName, pistolPhysicsBody, pistolProjectileSprite);
					pistolProjectile.Active = false;
					pistolProjectiles.Add(pistolProjectileName, pistolProjectile);


					// obtain projectile name, sprite and physics body for the rocket projectile
					String rocketProjectileName = "player" + playerNum + " projectile" + projectileNum + " rocketLauncher";
					Sprite rocketProjectileSprite = SpriteManager.getSingleton.load(rocketProjectileName, rocketProjectileSpriteMap);
					PhysicsBody rocketPhysicsBody = generatePhysicsBody(rocketProjectileName, rocketProjectileSprite);

					// create a rocket projectile, set it to be inactive,
					// and add it to the pistol projectiles dictionary
					Projectile rocketProjectile = new Projectile(rocketProjectileName, rocketPhysicsBody, rocketProjectileSprite);
					rocketProjectile.Active = false;
					rocketProjectiles.Add(rocketProjectileName, rocketProjectile);
				}
			}
		
            return;
		}

		#endregion

		#region Functions

        /// <summary>
        /// Creates a new Projectile which will be added to the manager
        /// </summary>
        /// <param name="index">The player which will shoot the projectile</param>
        /// <param name="_type">The weapon type of the projectile</param>
        /// <param name="_pos">The initial position of the projectile</param>
        /// <param name="_rot">The rotational amount of the projectile</param>
        /// <returns>Returns the projectile we just created</returns>
		public Projectile createPistolProjectile(PlayerIndex index, Weapon.weaponType _type, Vector2 _pos, float _rot)
        {

			string projectileName = getNextPistolProjectile(index);

			Projectile pistolProjectile = pistolProjectiles[projectileName];
			//!! if KeyNotFoundException occurs here it means all the pistol projectiles
			//!! are active and a pistol projectile cannot be created
			
			pistolProjectile.use(projectileName, index, _type, _pos, _rot);
			return pistolProjectile;
        }

		/// <summary>
		/// Creates a new Projectile which will be added to the manager
		/// </summary>
		/// <param name="index">The player which will shoot the projectile</param>
		/// <param name="_type">The weapon type of the projectile</param>
		/// <param name="_pos">The initial position of the projectile</param>
		/// <param name="_rot">The rotational amount of the projectile</param>
		/// <returns>Returns the projectile we just created</returns>
		public Projectile createRocketProjectile(PlayerIndex index, Weapon.weaponType _type, Vector2 _pos, float _rot)
		{
			string projectileName = getNextRocketProjectile(index);

			Projectile rocketProjectile = rocketProjectiles[projectileName];
			//!! if KeyNotFoundException occurs here it means all the rocket projectiles
			//!! are active and a rocket projectile cannot be created

			rocketProjectile.use(projectileName, index, _type, _pos, _rot);
			return rocketProjectile;
		}
		
		/// <summary>
		/// Returns the next available pistol projectile this player can use.
		/// </summary>
		/// <param name="index">
		/// The index of the player whowants to shoot a projectile.
		/// </param>
		/// <returns>The name of an available pistol projectile</returns>
		public string getNextPistolProjectile(PlayerIndex index)
		{
			foreach (String projectileName in pistolProjectiles.Keys)
			{
				if ("player" + ((int)index+1) ==
					projectileName.Substring(0, projectileName.IndexOf(' ')) &&
					!pistolProjectiles[projectileName].Active)
				{
					return projectileName;
				}
			}
			return "";
		}

		/// <summary>
		/// Returns the next available rocket projectile this player can use.
		/// </summary>
		/// <param name="index">
		/// The index of the player whowants to shoot a projectile.
		/// </param>
		/// <returns>The name of an available rocket projectile</returns>
		public string getNextRocketProjectile(PlayerIndex index)
		{
			foreach (String projectileName in rocketProjectiles.Keys)
			{
				// Check if we have an inactive projectile to use for this player
				if ("player" + ((int)index+1) ==
					projectileName.Substring(0, projectileName.IndexOf(' ')) &&
					!rocketProjectiles[projectileName].Active)
				{
					return projectileName;
				}
			}
			return "";
		}

        /// <summary>
        /// Updates all of the projetiles
        /// </summary>
        /// <param name="time">The elapsed time in the game</param>
        public void update(float time)
        {
			foreach (Projectile rocketProjectile in rocketProjectiles.Values)
			{
				rocketProjectile.update(time);
			}
			foreach (Projectile pistolProjectile in pistolProjectiles.Values)
			{
				pistolProjectile.update(time);
			}
            return;
        }

		/// <summary>
		/// Gets a projectile by name.
		/// </summary>
		/// <param name="name">The name of the projectile.</param>
		/// <returns>
		/// Returns either a rocket or pistol projectile.
		/// Or null if a projectile with that name cannot be found.
		/// </returns>
		public Projectile get(String name)
		{
			if (pistolProjectiles.ContainsKey(name))
			{
				return pistolProjectiles[name];
			}

			if (rocketProjectiles.ContainsKey(name))
			{
				return rocketProjectiles[name];
			}

			return null;
		}

		/// <summary>
		/// Sets the number of players in the game
		/// </summary>
		/// <param name="players">The dictionary of players</param>
		public void setPlayers(Dictionary<String, Player> players)
		{
			this.players = players;
		}

		/// <summary>
		/// Notifies a player that they have killed someone.
		/// </summary>
		/// <param name="player">The name of the player</param>
		public void killed(String player)
		{
			players[player].Kills++;
		}

		/// <summary>
		/// Creates a new physics body for a projectile
		/// </summary>
		/// <param name="projectileName">The name of the projectile</param>
		/// <param name="projectileSprite">The sprite that represents the projectile</param>
		/// <returns></returns>
		private PhysicsBody generatePhysicsBody(string projectileName, Sprite projectileSprite)
		{

			// create body
			Body body = BodyFactory.Instance.CreateRectangleBody(
				PhysicsManager.getSingleton.PhysicsSimulator,
				projectileSprite.Width, projectileSprite.Height, 1);
			// Mass will not remain 1. Mass will be set in Projectile's constructor.
			body.IsStatic = false; // make body not static so it can move

			// create geometry
			List<Geom> geomList = new List<Geom>();
			Geom geom = GeomFactory.Instance.CreateRectangleGeom(
				PhysicsManager.getSingleton.PhysicsSimulator,
				body, projectileSprite.Width, projectileSprite.Height);
			geom.Tag = body.Tag = projectileName;
			geom.CollisionEnabled = true;
			geom.CollisionResponseEnabled = false;
			geomList.Add(geom);

			// create projectile
			PhysicsBody physicsBody = PhysicsManager.getSingleton.addPhysicsBody(
				projectileName, body, geomList);
			return physicsBody;
		}

		#endregion
	}
}
