using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.graphics;
using rhodes.physics;
using sedohr.player;
using rhodes.audio;

namespace sedohr.weapon
{
	/// <summary>
	/// A weapon that shoots a projectile.
	/// Author: Robert Yates
	/// </summary>
	class ProjectileWeapon : Weapon
	{
		#region Variables

		/// <summary>
		/// The recoil of the rocket launcher
		/// See also: Weapon.Recoil and Player.cs applying recoil
		/// </summary>
		private const int ROCKET_LAUNCHER_RECOIL = -400;

		/// <summary>
		/// The x value of the weapon offset
		/// </summary>
		const int X_WEAPON_OFFSET = 60;

		/// <summary>
		/// The y value of the weapon offset
		/// </summary>
		const int Y_WEAPON_OFFSET = 60;

		/// <summary>
		/// The offset from the center of the weapon to shoot the projectile.
		/// </summary>
		private Vector2 projectileOffset;

		/// <summary>
		/// The projectiles this weapon has fired
		/// </summary>
		private Dictionary<string, Projectile> projectileDictionary;

		/// <summary>
		/// The projectiles this weapon has fired
		/// </summary>
		public Dictionary<string, Projectile> ProjectileDictionary
		{
			get { return projectileDictionary; }
		}

		#endregion

		#region Constructor 

		/// <summary>
		/// Weapon constructor
		/// </summary>
		/// <param name="playerIndex">
		/// The index of the player firing this weapon
		/// </param>
		public ProjectileWeapon(PlayerIndex playerIndex)
			: base(playerIndex)
		{
			projectileDictionary = new Dictionary<string, Projectile>();
		}

		#endregion

		#region Functions

		/// <summary>
		/// Update the weapon
		/// </summary>
		/// <param name="elapsedTime">The time since the last update</param>
		/// <param name="press">
		/// True, if the fire button is being pressed. False, if not.
		/// </param>
		/// <param name="position">The weapon's position</param>
		/// <param name="rotation">The weapon's rotation</param>
		public override void update(float elapsedTime, bool press, Vector2 position, float rotation)
		{
			// just pressed fire button
			if (!previouslyPressingWeaponButton && press)
			{
				// create a projectile
				projectileOffset.X = (float)(X_WEAPON_OFFSET * Math.Cos(rotation));
				projectileOffset.Y = (float)(Y_WEAPON_OFFSET * Math.Sin(rotation));
				Projectile projectile = shootProjectile((position + projectileOffset), rotation);

				// apply recoil if shooting a rocket
				if (weaponGenre == weaponType.ROCKET)
				{
					Player playerShootingProjectile = 
						ProjectileManager.getSingleton.Players[
							weaponName.Substring(weaponName.LastIndexOf('p'))];
					applyRecoil(playerShootingProjectile, projectile.PhysicsBody.Position);	
				}
			}

			// set the gun's position and rotation
			weaponSprite.Position = position;
			weaponSprite.Rotation = rotation;

			// set previously pressing weapon
			previouslyPressingWeaponButton = press;
		}

		/// <summary>
		/// Shoots a projectile.
		/// </summary>
		/// <param name="position">The position to shoot the projectile from.</param>
		/// <param name="rotation">The initial rotation of the projectile.</param>
		private Projectile shootProjectile(Vector2 position, float rotation)
		{
			// get projectile object
			Projectile projectileObject;
			if (weaponGenre.Equals(weaponType.ROCKET)) {
				projectileObject = ProjectileManager.getSingleton.createRocketProjectile(
					playerNum, weaponGenre, position, rotation);
			} else { //if (weaponGenre.Equals(weaponType.PISTOL)) {
				projectileObject = ProjectileManager.getSingleton.createPistolProjectile(
					playerNum, weaponGenre, position, rotation);
			}

			// add projectile to projectileDictionary dictionary
			// or replace projectile if key already exists
			if (!projectileDictionary.ContainsKey(projectileObject.Name)) {
				projectileDictionary.Add(projectileObject.Name, projectileObject);
			} else {
				projectileDictionary[projectileObject.Name] = projectileObject;
			}
			
			// play fire sound
			if (weaponGenre.Equals(weaponType.ROCKET)) {
				AudioManager.getSingleton.playSoundFXs("rocketShot");
			} else { //if (weaponGenre.Equals(weaponType.PISTOL)) {
				AudioManager.getSingleton.playSoundFXs("pistolShot");
			}

			return projectileObject;

		}

		/// <summary>
		/// Apply recoil to the player who shot this projectile.
		/// </summary>
		/// <param name="player">The player shooting this weapon</param>
		/// <param name="projectilePosition">The position of the projectile.</param>
		private void applyRecoil(Player player, Vector2 projectilePosition)
		{
			Vector2 direction = projectilePosition - player.ShoulderPosition;
			player.Entity.PhysicsBody.Body.ApplyImpulse(direction * ROCKET_LAUNCHER_RECOIL);
		}

		#endregion
	}
}
